﻿
/***************************************************************************

                                 ChessV

                  COPYRIGHT (C) 2012-2017 BY GREG STRONG

This file is part of ChessV.  ChessV is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.

ChessV is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
more details; the file 'COPYING' contains the License text, but if for
some reason you need a copy, please visit <http://www.gnu.org/licenses/>.

****************************************************************************/

using System;
using System.Collections.Generic;

namespace ChessV
{
	public class HashKeys
	{
		protected int nextKey;
		protected int nextMaterialKey;

		public HashKeys()
		{
			nextKey = 256;
			nextMaterialKey = 256;
		}

		public int TakeKeys( int count )
		{
			if( nextKey + count > Keys.Length )
				throw new Exception( "Not enough Zobrist keys!" );
			int given = nextKey;
			nextKey += count;
			return given;
		}

		public int TakeMaterialKeys( int count )
		{
			if( nextMaterialKey + count > Keys.Length )
				throw new Exception( "Not enough Zobrist keys!" );
			int given = nextMaterialKey;
			nextMaterialKey += count;
			return given;
		}

		public static UInt64[] Keys = { 
			//	The first 256 hash keys are zero - these are used for "Pawn" hashs of non-pawn 
			//	pieces so that they will not affect the Pawn Hash.  The "real" pieces start at 
			//	index 256
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 0UL, 
			//	The real keys start here, at index 256
			0x2d0559d72925f8afUL, 0x5cc2a1ee91890a55UL, 0xf93130b1bfa93589UL, 0xafd514414023bfb3UL, 
			0x4ce0871210002fadUL, 0x029659688965e0aaUL, 0x727c7fd49b5500a9UL, 0xb2c6cfda6e386a32UL, 
			0xc0cfccfa064a0558UL, 0x5e6231b848673389UL, 0xcaa7e47d73d2daaaUL, 0x05eadbe2bd809bb7UL, 
			0x53bffa9df22dd787UL, 0xd1a2bd94a36c07feUL, 0x3db7e9e1dfdce9b9UL, 0xa0e91d8c0c8d0128UL, 
			0x8fe3174adb33da4eUL, 0xc68ceb2a849fd2a2UL, 0x9fb0fd51eb1d909cUL, 0x127365537fb5612aUL, 
			0xd97c094a8204d57bUL, 0x5f4e16c90a85d8caUL, 0xb03ec415d99c7c19UL, 0x5d50ecb30bc00e51UL, 
			0x7a26dd2093961e0bUL, 0xb47f8538d87defbcUL, 0xb35a95e50c89e330UL, 0x835e47813090886cUL, 
			0xa59f16ff5199a9d6UL, 0xb58b7172f81c95d1UL, 0x230c48b40ecac170UL, 0x8ee91e310bba4d16UL, 
			0x77cc64c69b75a886UL, 0xebac80817541c8cdUL, 0x3af94b224fcbfa00UL, 0x2726da336231c4c2UL, 
			0x3ada1039670cfd5fUL, 0xd5fae619c92eb91bUL, 0x65bd89c386983c7dUL, 0xe85e7dd90547d3d5UL, 
			0x0e5752d3d6f04fa0UL, 0x38a319620515893cUL, 0xe59de3907904d3c9UL, 0x9fc8edbe27195dceUL, 
			0x41eb32ca58ff6726UL, 0xfc4ce2f612bf15acUL, 0xd8faddf23a41e8deUL, 0x68419d7218c5f749UL, 
			0x5af05d643b7e92aaUL, 0xa580614b1b440ffdUL, 0x7abfb44447ac8f82UL, 0x7ae3dcfc72992c79UL, 
			0x7db5a868878b1303UL, 0x7c8ee166e71f7d06UL, 0x9a732dbe9f55c300UL, 0x6d0940c849fa435bUL, 
			0x19d07767d774a54fUL, 0x61cd3cb86f2cd0ccUL, 0xe5804bdb5be33bd6UL, 0x56b3590f693fb3ffUL, 
			0x29863ec47870e4c4UL, 0x979cb4058a61e04cUL, 0x3259ea69c312b20bUL, 0x07cf504c1c66b542UL, 
			0xaae3a5de358f9e88UL, 0x5ce71b344c3ab692UL, 0x6c2238798e24c173UL, 0xe0614a8716fa4e09UL, 
			0xad2c99225c5ff1a6UL, 0x6a8e759b50b8d9f3UL, 0x81707206b3f82450UL, 0x158de2fa3a02314aUL, 
			0xe8e42089c41b0646UL, 0x1e57a93cb83679afUL, 0x3d78ef383ebff5d6UL, 0x3363746cdd268751UL, 
			0xab1af763e2216b39UL, 0x7b53a83eda979535UL, 0x44bd8693c21e104bUL, 0x6bb888d6f40d8e88UL, 
			0x3e464daed5f8af60UL, 0x1c0a459558b58a6cUL, 0x1ef00b5b1e957338UL, 0x156328f0680fecceUL, 
			0xa72cc4dc6412c12aUL, 0xa374ac318b11f34eUL, 0xa98d59d40d1ab939UL, 0xbb8df143d1acac18UL, 
			0x1c0421cc9d512f4bUL, 0x659aeff29aeb7387UL, 0x1bf163904f93c07dUL, 0x6098bfcc6491fb2aUL, 
			0x2aaaba822fd32a6bUL, 0xfba1720bfb663c3dUL, 0x73e942e72b82161dUL, 0xfa9824badf4a37aeUL, 
			0x2d64610322ce8076UL, 0x8d68a186068c0cb7UL, 0xc2d94faf325531b9UL, 0x6944f62fc530b6d3UL, 
			0x037877e441da8802UL, 0xcec115d17d0e94a5UL, 0x3883dd9a4ba12f2fUL, 0xf262e017c45258c2UL, 
			0xc32cb1fca942afeeUL, 0x1f9f15f8b35db435UL, 0x2a19bdea8a8a72f3UL, 0x20762c1f4aa0fa76UL, 
			0xab024a3fc30517a5UL, 0x4ba211949dabf9d0UL, 0x23674a3a5c859e76UL, 0xfecd163a88ee9bc9UL, 
			0x5f3158eefcb56993UL, 0x7dcf349aeda29f9bUL, 0x7f309112c0e9f94eUL, 0xe9400ec025c6a784UL, 
			0xb0ec5bd45f431a35UL, 0xbe2de7bedc3b4199UL, 0xfd79437a1a5606a2UL, 0x113db81796c10128UL, 
			0x02c8b34780502f80UL, 0x043ba3404223215cUL, 0x6d3467dc4d5752d6UL, 0x3dd91cd3305c7cb3UL, 
			0xc2a437040aff9dfeUL, 0xb76477873cdc140eUL, 0x1107231741befa6aUL, 0x0e6cce3a1f1f250aUL, 
			0x1eee4d4faef9ffe7UL, 0x7c1be6144c92d8f4UL, 0x8acdcbd04f446ab0UL, 0x0361e2821537a723UL, 
			0x0803949531f76ce1UL, 0x8f3fa0ae2340e2b7UL, 0xf0b7b405ed3cd041UL, 0xac8bdc368a9cc1c9UL, 
			0xdfb17f8a202aa448UL, 0x0cf251ef96ca8c4cUL, 0xa42b3ebf7d67d568UL, 0x22fafefd68ca79efUL, 
			0x2190e16577e8f3aeUL, 0xc553b4c06274d821UL, 0x4da276788db56676UL, 0x67cc6ef6aa7e8f61UL, 
			0x3a25c61bf0444c8fUL, 0x3bb9cdccb761a9a4UL, 0x22cf4c79b3ce7ba9UL, 0x5204274cb65c3157UL, 
			0xda11f9d7766865eaUL, 0x433324bad10f20dfUL, 0xfdd551fbfc243de8UL, 0x4ecd277c51146fd3UL, 
			0xf8eb4e6e26d33f9aUL, 0x67333665ba087e6eUL, 0x4e00942351c69dbfUL, 0x822fdc4709f29112UL, 
			0x233a997f5786b873UL, 0xa9c1abe965543b5cUL, 0x9f806929c99344abUL, 0x9bcfcbff89de1465UL, 
			0x24c6f9fc3e083081UL, 0x86720f2386e0dce0UL, 0xac35606fc92eef18UL, 0xf59eb98c3f48b9daUL, 
			0x0224a1c0f7f78a97UL, 0x3ea4924f2b81f2e1UL, 0x4df589931bbd2c11UL, 0x1b65eb1725e9cb18UL, 
			0x6b10f0dda3c4549eUL, 0x432a10b1aa53e7edUL, 0x4a2c56dcedd88214UL, 0xc9dfcf7ac94e8167UL, 
			0x6f1676c00e7d3077UL, 0x9577a8e05d82b90cUL, 0xe5187289dcdad235UL, 0x515fef52a078dcccUL, 
			0xccaba807b3c66667UL, 0x92bfd188a9c87fa4UL, 0xc099df566bb9144aUL, 0x2f060642eae37524UL, 
			0xb7363cd312c23389UL, 0x1da46e312b044f25UL, 0x6fae2e282b99ae88UL, 0x163ed9136ce484d2UL, 
			0xf49f9aed85a6f17fUL, 0xc072d837e9229f49UL, 0x4bd73c582ca13e1dUL, 0xae558bfec7a66c79UL, 
			0x43a3d50a87353b66UL, 0x63bb84a722fe419bUL, 0xc2db2eda30c5e258UL, 0xefc4033e5c2f220dUL, 
			0x2511331d7999efdbUL, 0x0501d130b904f32aUL, 0x3af52c4e673f4dcdUL, 0x5c8ed2244d9f297dUL, 
			0x75c411b178b51a11UL, 0x2b76832a78b966a6UL, 0x19c3938210be0aa0UL, 0x89dcd0f1bf228b01UL, 
			0x0ac24a536b34f409UL, 0xb5f3647ce31396f4UL, 0xbb50edcb1910bfa5UL, 0xbfee26c320bcc2b4UL, 
			0x0ecd567575564783UL, 0x1311a12f1a10d5bcUL, 0x6de97ed23a423540UL, 0xa6209bf6820965afUL, 
			0x2760bf76990a699aUL, 0xc767be7534b5fb10UL, 0x2a1d7f2d80007d6cUL, 0x763d2488a2a17504UL, 
			0x7a92436b28283ed2UL, 0x1f2917631fef8166UL, 0x509b900a64d1838bUL, 0xd67f429669e969dcUL, 
			0xb77cf33178ec2267UL, 0xfbf58d202d997290UL, 0x6d1293c58d596e6aUL, 0x8dcc5ea4d6a7c3c4UL, 
			0x033cc1a5373ab718UL, 0xe79eebaf9778af9cUL, 0x596aa9e6dc335c7eUL, 0x1face3bb49193bd2UL, 
			0xef3aeb18d6bf37deUL, 0x90838cc73f8b4d76UL, 0x5f228360fcf1c8f0UL, 0x3169e926dd6da4e8UL, 
			0x8c910fa13bd53154UL, 0x701b3724692e5f17UL, 0x1f1fbfd2d01a5e98UL, 0x2ff77bae2bed4b29UL, 
			0x2a80568f408eb50eUL, 0x038a81aecb89b2b5UL, 0x89a6ce339cda12cfUL, 0xb0ab50c534c1e4d9UL, 
			0x859e4e79d4911109UL, 0x0ccc19be5d6e64f7UL, 0x15b08461f1c92d31UL, 0x2997259c7e880399UL, 
			0xe1fc577e70cde87bUL, 0xc0ffd3d24c1ccf94UL, 0x6f5e92aed1a1c227UL, 0xf076ef95ad4f0f51UL, 
			0xc4a2acf80dd97534UL, 0xb0d0b5fb76dead3bUL, 0x15b5803162fd9ecfUL, 0x94ad031a5778110fUL, 
			0xf5890de52aeff589UL, 0xe06e7de81b2771c5UL, 0x74129a6b965b9016UL, 0xf80e168796477aebUL, 
			0xcc4ef43aa7c9c9d3UL, 0x1795d2fb108c65fcUL, 0xcb8c5c67bd9b7fd3UL, 0x3ee6c5abf07749b0UL, 
			0x109c7aaea4d7b0e9UL, 0xbf44133127dfe4fdUL, 0x763202a2d77328bfUL, 0x0a69e5208368d1efUL, 
			0x99e355a73de2365eUL, 0x37c578112ce1d409UL, 0x566eb5b1adf52e2bUL, 0x842e286c2fefff92UL, 
			0x16a53c15f9a42907UL, 0x3becb85d0a72473cUL, 0x869df12923355f35UL, 0xad39bbb334e7443fUL, 
			0x0b3a750513e255f0UL, 0x6e37bcb888d1fc09UL, 0x2cfd29c99f68070eUL, 0x76c2c61b855e2685UL, 
			0x7ed76be2fa0d6cc9UL, 0x69af34240cf070edUL, 0xde99c8000abd0b68UL, 0x58d498992e256037UL, 
			0x7ff512a3b5608fc6UL, 0xbef78af74aa857ecUL, 0x6100d32f6864d9f6UL, 0xf165ca5249bc15b1UL, 
			0x7310bdd9c3d77cfdUL, 0xe9654b9dafa59620UL, 0xf18d5c179f4ea57fUL, 0x4be86abbb7cb8155UL, 
			0x38ee64cddfd89e08UL, 0xf22c51ce80bbc42bUL, 0x6c307881499f3bf0UL, 0x601958eedfa5ce2aUL, 
			0xd001bd6512ec3925UL, 0xb6f9d1c914ad1c31UL, 0xc49f8c36f47ab943UL, 0xffef79f8bc9e04ccUL, 
			0xecc9e87487811258UL, 0xe990e36e99207ff3UL, 0x26b9b734c7cb2b16UL, 0x029ef821ae3af03aUL, 
			0x1cf1168afeeb96d6UL, 0x1b574731731c7ea0UL, 0x0f5cd0e7fda338d7UL, 0x99a0a39ee6413288UL, 
			0xb912081fdd2d9231UL, 0xa7efb03e498b576dUL, 0x9021e953935f331eUL, 0x2906e187593c2582UL, 
			0x9a1749f4509f76b6UL, 0xe5a3d11cfa17563aUL, 0xe07f5a0b005c9d28UL, 0xcc8feda99c7fdcf7UL, 
			0xa329c5d158589b4cUL, 0xf6198cf7ac192674UL, 0x6fc03c958419b17fUL, 0x6c5ab2c11d908768UL, 
			0xcf7066ce82dc7d9fUL, 0x3f45f61c1c7d3cfbUL, 0x73ba8eaaea0dd983UL, 0xcc93ba6f970f6d40UL, 
			0xa19aa9f05fa43a40UL, 0x13345b0141ddcea4UL, 0xcf4c2ffebe3de08bUL, 0xae8050fbc3fc265cUL, 
			0x8491404b5a2dda39UL, 0xd583132a15aeb599UL, 0x4d23abce659cad8dUL, 0x9545668fd5ed7181UL, 
			0x5ec4e33f7859027bUL, 0xa1439b977e22ea0eUL, 0xa34da702eb39ebd9UL, 0x31beb825f0490c5fUL, 
			0xc3ff74d34449e436UL, 0x0d10fbc78aa56d89UL, 0x05e508cef7dc8fd2UL, 0xbde605a225d61b61UL, 
			0xbd8b6293963067e1UL, 0x8e8270818edf8d54UL, 0x167aef9af7b4e773UL, 0x679c233771b01830UL, 
			0x30e390d49937b011UL, 0x1dd38a133aec3d91UL, 0x82391abeb19e89acUL, 0x85628122dbf5e916UL, 
			0x9c5efa9675ab15d8UL, 0xf1cfa30841c7769cUL, 0x88d5e503fb0fedf8UL, 0x77473da6e4f9da2cUL, 
			0x9c06182f799fbffbUL, 0x011421c76152c1bfUL, 0x3c4f3d4478165ed8UL, 0x6731c833e92ae4c8UL, 
			0x4cfdc55a45387f50UL, 0x738137566c6cff6dUL, 0xd26254ea4dd24e4fUL, 0x0777a1c1914583aeUL, 
			0x7137c9ac3d192d26UL, 0xf70fa9c960782893UL, 0xecab4a70a0da868dUL, 0x734ff18e2843a2ffUL, 
			0x60b62bbfe512af53UL, 0x00fe4cef46e16427UL, 0x81c88cea90ce7acdUL, 0x17c5e3a702e1433eUL, 
			0x265a348b12b4d021UL, 0xeadd9ca7efe35e91UL, 0x329268ff78dead27UL, 0x26512f2a4e17b089UL, 
			0x01ccd9bab6a9ce8dUL, 0x644d0a72913853bdUL, 0x95041a003520e727UL, 0x801b23e3dd8c3d9fUL, 
			0x10a0a4188886ae55UL, 0x1fa2f30d5125e054UL, 0xfb22419e979bff2fUL, 0x9db365158c2b80c0UL, 
			0x6f40f5e2b3ba280cUL, 0x6bd70202612f1c5aUL, 0xd59c5d862ae3a9aeUL, 0x2e60031adba8f4a5UL, 
			0x8d23f3eb23b8b380UL, 0x664648646bbaeb41UL, 0xc59ad33dab584e77UL, 0x82155e51f3e33031UL, 
			0x15b96c3fd0f4c4a7UL, 0x9e4e5f73d59e4badUL, 0xf8901d2346da09a0UL, 0x376746b59c61a239UL, 
			0x1bee5a4546c52960UL, 0x50f165d205b82892UL, 0x0a12e3b94cdf4a52UL, 0x8e17e87ed4f2cd63UL, 
			0x23a84555f92446d0UL, 0x51aa3bd1b2fc0531UL, 0xb63acfbedf973f8cUL, 0x963881c485991354UL, 
			0x736581c710ac3bf6UL, 0xf260d18de726d1f0UL, 0x91d24f4ed2d0ea8aUL, 0x3cf30960644e32b1UL, 
			0xb0eab93bea932694UL, 0x87a95c93365741f1UL, 0x9edc00e850120884UL, 0xff48b3f07fdff520UL, 
			0xa83a19dc9b1d56e2UL, 0xc59d42caafd8e5c3UL, 0x93a5515e061578ceUL, 0xcf7564178f45c448UL, 
			0x7401ce3cc9112c73UL, 0x92a251140f4ff4c1UL, 0xbfe7af73cb7efc21UL, 0x1541e93b41829565UL, 
			0x44f4ad0e065b56c1UL, 0x92163df02adeb1c6UL, 0x82bbf73e9da5e9baUL, 0xb3694637235c5443UL, 
			0xaa641f5428223f12UL, 0x872df8bf4a6c94f9UL, 0x366876042aae8ffbUL, 0xf2264c07f57e5bd8UL, 
			0x9bff6bd70e253c73UL, 0xe5f9f79d4bb41153UL, 0xaac4e0e1f5cea077UL, 0xc3fd5107a6ebea47UL, 
			0xf34888041f78030aUL, 0x926e6fcef2348b14UL, 0xbb4729c2a9323560UL, 0xfb89f3b46b9f8ab0UL, 
			0x6f4913e6f016e5b2UL, 0x62144c12ee69df54UL, 0x38337100c4b9b3ceUL, 0x3b65a183192efdf9UL, 
			0xa1aa3d17fe9fcb91UL, 0x6787506aabdb7f27UL, 0x23acc5ac31367683UL, 0xc61cad67c6f83105UL, 
			0x762466da8ce8ab37UL, 0xfc18bd44548709b4UL, 0x26b5c54f5f791947UL, 0x8af6e2dccf03515dUL, 
			0x941501d41318bb06UL, 0xbe5df6afa0c61fddUL, 0xff41be4eae8f6a0dUL, 0x522be8ee4313f44aUL, 
			0x6315cb117805d4cfUL, 0xe28ef0d21dd13cdfUL, 0xec6a82261ee3d1e6UL, 0xb533e5357ccf82a9UL, 
			0x647a9d6ff8c8279bUL, 0xd8514f4e52e418f9UL, 0xddbd645d67496664UL, 0x93c826a5beea5cccUL, 
			0x3b769d05303f6592UL, 0x0568cabcfa85b88fUL, 0xa8eb146f08b68df2UL, 0x22113f4beb9d3753UL, 
			0x628fc5832aadf97fUL, 0xf2927700e2070612UL, 0x2e41f01a82f44e65UL, 0x56493e8c124c6061UL, 
			0xdca8eee428d0ddcfUL, 0x0abd125ca659475fUL, 0xb1602b2273d439f8UL, 0x265c74cdf68e4cd2UL, 
			0x15ddc957e59c5cc4UL, 0xd9999221d29ca392UL, 0x836369e7bf4707d3UL, 0x365f06179afc24a9UL, 
			0x56ba768994461e11UL, 0x04d8e3c47586a72cUL, 0x58edb82aadb09e4aUL, 0x740dcf832981105bUL, 
			0x4414cd9613e6b2a8UL, 0x7f3b67e4c94a2996UL, 0x8ed87de65f64b596UL, 0xf2593549f63d5ca4UL, 
			0x292774ec18de054bUL, 0x1017b56a8a175697UL, 0xe39e45e41f321e93UL, 0x222149962174bbcaUL, 
			0x95add5079ad2a50dUL, 0x9bf95bb8d5ea2b13UL, 0x4a77064baec52a4cUL, 0x47ed4b97352881c5UL, 
			0x3733dff7576b3d6dUL, 0x4709d1bcc6de86e3UL, 0x2968c560f58692e7UL, 0x7be7cebbca809cabUL, 
			0x8a5852357126b4b5UL, 0xce8ad510c5ad7326UL, 0x0a3773e0a1cfb808UL, 0x88a5c8d495a5b4a6UL, 
			0x7e04d4547b6526fbUL, 0xfdf48415adf8af3aUL, 0x5c967604b33e3afeUL, 0x57f2b4be051da523UL, 
			0xba6f5baa9ab7b45cUL, 0x5f3ede1cb5007068UL, 0x6a22f48e4610cf55UL, 0xb4c12e52be63f938UL, 
			0x337c6ac3729c0097UL, 0x46083071e12d9f7eUL, 0xc9ec3447ab772a91UL, 0x045dfa40c0c4b4eaUL, 
			0xd2313fba1a12d48dUL, 0x1e65b2c88c3a3f9aUL, 0x38340de437a3b63cUL, 0x2faa7eab585a4b3cUL, 
			0xbe226c16dd486c09UL, 0xc0f3e6d73b54c97aUL, 0xaf5df40b0f521322UL, 0x550c32f76be21c41UL, 
			0x911e8bdae7d12cf7UL, 0x93b0a81a93c3baf1UL, 0x90adc8e19b1783e1UL, 0x53e7d27b9ac6d712UL, 
			0x98d946e08fa07b02UL, 0xb813118e043fbf11UL, 0x49de60ad22a6b5faUL, 0xac55e956627e0e1dUL, 
			0x4ad4c6c2173630fcUL, 0xed7187df66a5c1daUL, 0x1b7ae0aa1ed49ac2UL, 0xdf4166121fb5e123UL, 
			0x621b88d30e278adeUL, 0x184e4864495c071aUL, 0x338d71ee1262d39dUL, 0x1a24e04aeb5e8551UL, 
			0xe56c0863c7c2a182UL, 0x5862e86666369699UL, 0xf0d406b87646eaeeUL, 0x24b938c8e2d6687cUL, 
			0x70d2e861eec6a0aeUL, 0xf0f74faec7410b36UL, 0x44d5b152358bdb0cUL, 0x2a5561566e37ba78UL, 
			0xe1e97754d44d7df7UL, 0x748643a8c516107dUL, 0x51d75c2ee6b7ad5cUL, 0x681f96087285a332UL, 
			0x10fe654ca7fb382dUL, 0x08e3fdc8e50793f8UL, 0xb840b72a42a2a430UL, 0x0d57fffe40e522fbUL, 
			0x78ffe4c6b8d10d8dUL, 0x079c47db7ab8b272UL, 0x571360a842e5775aUL, 0xc6b7b8d976b9e316UL, 
			0x3de0516736dacd4fUL, 0x57ed11af8e6cead9UL, 0xb16bc6c9470177a3UL, 0xc1c9430ff277d571UL, 
			0x2019d0ad87d64845UL, 0x9135ab789536156cUL, 0x54e96e81de654c16UL, 0x3c58bfb63e9dde20UL, 
			0xdc3884847a27cdb6UL, 0x4c2d53253c50b0d5UL, 0x81de0831e2533dcbUL, 0xf4c78c99fa2a4e01UL, 
			0x4d366989334e5eaaUL, 0x5a0726f59f37a131UL, 0xa3f44a18b0953e06UL, 0x6d7f3677b91e64a0UL, 
			0x6a160ffd077cf53fUL, 0xf8355d328bc88567UL, 0xe48b456efa6caa22UL, 0x10c86512a721055cUL, 
			0xfc4043e957bdc50aUL, 0xbec0fd9ae73e4dc8UL, 0x54aaa71a448573e7UL, 0xf71625df8e61d37cUL, 
			0x43c47e70f9721f56UL, 0x5c1923329efa2509UL, 0xcf08715563c85b10UL, 0x1d896daf63a959a2UL, 
			0x6c3a6bb1ea671d33UL, 0x540c4ecbb4771c04UL, 0x61af15de40556dbbUL, 0x719a7a1d0f513559UL, 
			0x784fbcbc63a29e9fUL, 0x555c87f50045d6c1UL, 0x1a572746cecece74UL, 0x4aac8685f1c21df0UL, 
			0x9987f95840091130UL, 0x8c3376f202fa74aeUL, 0x17dc0669a0b6a0adUL, 0x3c06794a7bb8c966UL, 
			0x420df1fe4f6bb164UL, 0x375dbe0c01bd65bbUL, 0xdb43a700445f79e4UL, 0x6e56df1fde5fe087UL, 
			0x9574c230317ad6dfUL, 0x302ef6092a23e0c6UL, 0x543cfd7dfd1d7217UL, 0x90980e78580b004aUL, 
			0x011d81b04497ddf2UL, 0x587bde102b87ac1bUL, 0xea79ffe8edbcc671UL, 0xb4b5c557d50b4dcbUL, 
			0xbbc1dcc29cbac747UL, 0xf4c03e5df792e957UL, 0xc0eb07119e2d72c3UL, 0xad69bdc3e3c788f9UL, 
			0xb6e1bc7803b75dc4UL, 0x7bdbe70abbb45dd6UL, 0x2e8f8f23f7065f94UL, 0x1b6f8949bcb49da0UL, 
			0x123746139498405dUL, 0x6d744b3ad168da57UL, 0xfa977c118c5cc8baUL, 0x9b096c3a20e49c15UL, 
			0x11314c1561579f63UL, 0x9ac89e5fa3b6c21bUL, 0xc5eb6259015b28dcUL, 0x024c7413a3eaf2ceUL, 
			0x5085127f3187defdUL, 0xca414261b4953937UL, 0xf0a0a1d33882aa13UL, 0x5ffb0cb43b3275f2UL, 
			0x27fa71602e91627fUL, 0xd535aa0c7e4e7b8eUL, 0x1d2d8b1dad807a3aUL, 0x145e71e2b73db47fUL, 
			0xd6362ccc187736b7UL, 0xa38dc1909e34610cUL, 0x7078d659f24d8c19UL, 0x6faeec63c16c5643UL, 
			0xe39500888b0c8ceeUL, 0xb44a73e5d04d5a5bUL, 0x1eb57cbe8d426eb9UL, 0xfbe00fbce8fd44b1UL, 
			0x0a2df61ada434ff2UL, 0xa95b4dbe1998cdbdUL, 0xc406f390631998d4UL, 0x292ffe377841b0e7UL, 
			0x6ae0c6ebb7329f0dUL, 0x21c11786ebc7a304UL, 0xdf944759b7c50bbfUL, 0xf0d4532871b5356bUL, 
			0x6b03d9ba3fe2edccUL, 0x486ba4667fddf20aUL, 0xbea4d259dc805126UL, 0x9861364ebee2eff1UL, 
			0xebdf05ed240652b3UL, 0x95a0d0bee1fca0abUL, 0xa67cb0b0e859bbcbUL, 0x570f53bea23cd490UL, 
			0x150649cd89e0f513UL, 0xf9e4755ece412d48UL, 0x9babae3751c82b5dUL, 0x70660e7fdac946b8UL, 
			0x0f423f05ef48a85cUL, 0xf2296f49a3686899UL, 0xc5fc4c38f3ca2bf3UL, 0x0aeae6ae5463ceafUL, 
			0xe7c7d012c7776724UL, 0xd6d9279cf861305aUL, 0xf5a09ea7511a94ceUL, 0x04831c732b06d366UL, 
			0xb9f75c42ba17dbfdUL, 0x61323b7670d6ce51UL, 0x839f1dddd2b8f479UL, 0x520912ffe69a3fe3UL, 
			0x2e7abcc58c25875cUL, 0xf25b28da7317cbacUL, 0x0e09251110ef0534UL, 0x1a7580e32a60d971UL, 
			0xb97dcd189b2dd852UL, 0x73ce3365d2de075cUL, 0xb8c815f147ca49e8UL, 0x5d1cef49847ca025UL, 
			0x76734ebbb5673bcfUL, 0x0c32abf577ca9d6dUL, 0xbade48a5d6e20b1cUL, 0x9caba688004597d5UL, 
			0xc76075125f2624d0UL, 0x011f711b2638a880UL, 0xb5959d8aa94c794cUL, 0x7669a0bb628fbba6UL, 
			0x5325a39945e704a9UL, 0x57cf340b81bc75f2UL, 0x16299755cdb579d5UL, 0x33c8d9fcfc690922UL, 
			0xfcf4c0aae8a55576UL, 0x119bf08c45d38d1aUL, 0xcf772e066e67e443UL, 0x41c1a147f19557fdUL, 
			0x2922961c483093a9UL, 0xbd62b6f1bc9c4f5aUL, 0x154a25a5260285f7UL, 0xa1e04f43d8816fcfUL, 
			0xf5a5ff06275646e1UL, 0x4749d1e3f1e99cd8UL, 0x5ec28fb75acd3ce2UL, 0xe5e85308fcb894a4UL, 
			0x91a76671552ba189UL, 0x92330ee418a8a5abUL, 0x3016d87004d80873UL, 0x88a830367f6f878dUL, 
			0xd19ec815705de10cUL, 0x6a2f178fcdf3be55UL, 0xd1fcf01fcac9a1c1UL, 0xf02c5a9c208b9a5aUL, 
			0xc4166a2351b5b1edUL, 0x5097c38743590c8dUL, 0x3bf4217b166a5b48UL, 0xc9a66202c62f1eecUL, 
			0x5b507fb724b37f47UL, 0xcb1b155c14acf8adUL, 0xc9d98ec5aa5c516aUL, 0x60fc567eb5b4f7fdUL, 
			0x482c633e9f964732UL, 0x8caa21a71b6d7278UL, 0x14f8aa10b198c94fUL, 0x056628e1af07cfe1UL, 
			0x045b89fdc25565e9UL, 0xb4c9146188df83ecUL, 0x3e8259661530de99UL, 0x2e5eb1d6d2c7f472UL, 
			0x4d294e81a7a0406fUL, 0xf83a31885fe024acUL, 0xe1d1be92b4427388UL, 0xdcd99c0f908ca6a9UL, 
			0x8a4ae45134a893fdUL, 0x3d32ec79fd1b0004UL, 0xf0d25c9693b9b268UL, 0xc0402ca53d87dc70UL, 
			0x26e8026d7e47fd6bUL, 0x6809e7f8e705a26bUL, 0x2373f83f0bfad43bUL, 0x43b049a69f1c0af1UL, 
			0x84ba0b9f0f37167bUL, 0x805a1943e16754d9UL, 0x0173a94f4c16b03bUL, 0x602d7cc7a53c9e6cUL, 
			0xb40ff130164ae2bcUL, 0x41ea8e57e32812bcUL, 0x4862639046257c40UL, 0x6404cedadb8014ddUL, 
			0x8bd7f254a54e1ad5UL, 0xc993fa4c00821840UL, 0x99459ffa9f157b8aUL, 0xd46e6cc25aa3cfe6UL, 
			0x5f1c0fc633badd85UL, 0xde76bcc77c2b6b87UL, 0xf27e67cf48862f82UL, 0x7567db9a83c32255UL, 
			0x86e77e9455a180dfUL, 0x49a559bc7bdbd0f8UL, 0xe9adee10d1065523UL, 0x217e9fa532b4bd62UL, 
			0xba2ee8dc43d5dc81UL, 0xa09954a1ca07494eUL, 0xb31cf8b85a0a7f8bUL, 0x66119701b93f698eUL, 
			0x2e416efba118ece7UL, 0xa2e8a748643e70edUL, 0x90940f091cd93adaUL, 0x16ef73ffdbfe2f32UL, 
			0x4702ef74c025cf1cUL, 0xd784cc789fd8d530UL, 0xd27e0126d93659afUL, 0x55c98c16f6c07d75UL, 
			0x1822865761642374UL, 0x67993ca45a2814e1UL, 0xf1dd47f8897c4acdUL, 0x548ac5d46f860a14UL, 
			0xdc805b899c383adfUL, 0x4902943b5bf83431UL, 0x17830d2fc269b88aUL, 0x96a662f4d9173c79UL, 
			0xc407238c0fe7fcf0UL, 0x50cde33eebb1b9bcUL, 0xd5dfc49348a79899UL, 0xd63e613135f03095UL, 
			0x9619d40f2ad16b00UL, 0xe2bb31132607c55dUL, 0xac6ba2a6e555abcaUL, 0xc9c4f7e97dd53a01UL, 
			0x5cd1f25fef7d1976UL, 0xa2fcabeaf9bf0426UL, 0xdd335a5070833a1bUL, 0xb9c8a833bb6a97a4UL, 
			0x24013a468ea1efb3UL, 0xb5290c3a36f0c3c6UL, 0xf36c5d811413a594UL, 0x5f3bb9ced886822aUL, 
			0xf06df6fb129f3d4cUL, 0xfe3de98534679afdUL, 0x2244df9aa7a941b8UL, 0x7a8a8c48a72d4e76UL, 
			0x48359274b811bc69UL, 0x7820dcbd25ef8f02UL, 0x8d5a1cf70da4a56bUL, 0x27ef36df7263fccdUL, 
			0x28e0fbf4eab364cdUL, 0x15597ade85cd6735UL, 0x2330a2422dd5ca53UL, 0x08a7b4ee170ab936UL, 
			0x1f9120b9cc924fbcUL, 0x1354b01e94e7b7daUL, 0xe4e0d185985b9f9dUL, 0xdff74c05b04c8c53UL, 
			0x51b2004b893756ddUL, 0xef5b43025b42fbbfUL, 0xe3dc72a549510015UL, 0x57377454cc97ad01UL, 
			0x5bda4c02390680f4UL, 0xeb509e889c59aee3UL, 0xb2c8976e6af690e2UL, 0x352e8ac1fc2fa308UL, 
			0xeda3d4aa2fc3f0bcUL, 0x7e7cd604f1bbe897UL, 0xd7f85474f11ea414UL, 0x15ce41afaae729f1UL, 
			0xd2b3d4f6867c313cUL, 0xa0b6216790156ce6UL, 0xd9f18aa7fbf77996UL, 0x57bf371ea803a89fUL, 
			0x181f8288259db0c3UL, 0xea0bc00d1db207d1UL, 0x4e2b9c099ab9aba1UL, 0xc0986ed4d900eff6UL, 
			0x6dfcf4bf2792b6f8UL, 0x491996b5b089dba7UL, 0xa36da79b60f34f2dUL, 0x758e1c569499939fUL, 
			0x324b068e9ab57321UL, 0x077753dd690389f7UL, 0x74cde85106571ffaUL, 0x16180a87132daebeUL, 
			0x079fddb0a5d5d9bbUL, 0x7f5fc4a91010d4beUL, 0x3b573586fbb33f16UL, 0x8e3bbca5383dee10UL, 
			0xe6c68ee537ceaa54UL, 0xba12339eeebe4cf3UL, 0xead3e8395f335f87UL, 0x83df82e19facded0UL, 
			0x7fd556973bef05acUL, 0x646198d34423d35bUL, 0x2a22dbcd64b484d8UL, 0x4dcdae9f7f347805UL, 
			0x156c071731e8bbfbUL, 0xeb0a3a8816305dd8UL, 0x3cd9c475d37789f5UL, 0x8a7c6e2b5a479489UL, 
			0x1719ce8a403ddcb1UL, 0xeb066e491adcb9e6UL, 0x55b8fa2c9d83c2c4UL, 0x8d792fb75a6bbaf7UL, 
			0xf56bd4113b9a6d06UL, 0xb99c89fe2eca25e5UL, 0xe1a0eb82ec563556UL, 0x6d60aeafbf28584dUL, 
			0xca27e3362e331541UL, 0xedbae68c38a8c7f9UL, 0x13a922f71b2573caUL, 0x66403136c5833b66UL, 
			0xa657dd443ecb661dUL, 0x415dd8052f625c77UL, 0xfcd8e4609006c94fUL, 0xca3aa825e236df16UL, 
			0x8a0583a5c2c2647bUL, 0xb64f92f240d38831UL, 0x9c16d5d798127315UL, 0x2fb69cb4ebaec25fUL, 
			0x3f28fb7d341da64dUL, 0xba7bb7291e9b358dUL, 0x3a44d49f46112d23UL, 0x053e03c471e89b31UL, 
			0x720b81280df2b259UL, 0x3cfcb02493ff83a9UL, 0x5900ae69adfe1001UL, 0x9781c8b2b395180cUL, 
			0xe9f161450d2bf1c3UL, 0x18a7c57962febe6bUL, 0xf17bb85be4ff5babUL, 0xa8911bb7d2cc3f5eUL, 
			0xa7edfedb933a7930UL, 0xc40d9949e2835de2UL, 0x687f6d552c196bc5UL, 0x736e35ef1cfcf50dUL, 
			0x0abe799160f6bbf3UL, 0x580007ef6c418268UL, 0x88ccd08144111f52UL, 0x2921658570465f97UL, 
			0x4181912190d75815UL, 0xd96ce995099bf440UL, 0x81143989cf0e6321UL, 0x582f4f261360a944UL, 
			0x8e0136e276ace950UL, 0x2a484a8471913dffUL, 0xc1e69aefeb9e0b4fUL, 0x2c579ceff44f927bUL, 
			0x609c098941453ff0UL, 0xf595bace8a93e139UL, 0xffac608a0359fb8dUL, 0x43779509cb692fb7UL, 
			0xbf9166e46ffbaf72UL, 0xb4b934667e3a5ba2UL, 0x06bf463d512424c3UL, 0x79df1e169f065532UL, 
			0x78ae4d291236e35cUL, 0xd8d1c59fdb037fedUL, 0x9a3cd44a83aa6e84UL, 0xfe3bf767aa7bb094UL, 
			0x97ea9c4f47cd2e4fUL, 0xc37b54a5e5c31a55UL, 0xce0e604f0b485275UL, 0x748924df837b382eUL, 
			0xff61462d08d96b8bUL, 0x79fa998938a92926UL, 0x0fa86852ed669f23UL, 0x1656bd8757bd03ddUL, 
			0x5bfdd7693bae5ee4UL, 0xf040096e24352829UL, 0x5d629dc2a1a4cb98UL, 0xca837db1dc3a051aUL, 
			0x3e112ab98bf05708UL, 0x5bfcfe682f672101UL, 0x14f68c6754486f22UL, 0x4f5ed8c92d6c9d65UL, 
			0x391640cf29ba2cb5UL, 0x9de110df280fb3b3UL, 0xc2f27a4e181d53dbUL, 0x4ffaf69b28943d2dUL, 
			0xe7cede42cd9ba4f7UL, 0x8da30fb580fd39c4UL, 0x31125675be658558UL, 0xb992785760b8b195UL, 
			0xf155a12e508138d1UL, 0xee79e22860b908c7UL, 0x74c6918d82fe5567UL, 0xabf7c9d429606058UL, 
			0x2d453b18d557116aUL, 0x33f7b4ab175b3fbeUL, 0xef2f6948d0eafb16UL, 0x957aed0e7933c52fUL, 
			0x268fdb49f4bd8a0fUL, 0xc7b022220403abb0UL, 0x78a412cbb1bede28UL, 0x09aa66372d3e4ef3UL, 
			0x789152f19eb82f23UL, 0xa17222cbbab7cdf2UL, 0x5d6cbff2c7c400ecUL, 0xea6ff328f08b87a7UL, 
			0xdf16892318fc3249UL, 0x5b4ad645021daa2fUL, 0x61a30bd7f59913dfUL, 0x7c0b28b04600e1e6UL, 
			0x0050043e3d8179ebUL, 0xb241e00c3db68345UL, 0xf54af0221449d348UL, 0x47a3b629e7bca208UL, 
			0x721d1b15a065e6b2UL, 0x7bff34c4dd79ca5cUL, 0x8f6b278f174d6da8UL, 0x8666f6d326f7a2f3UL, 
			0xc69b79e47e63c3ccUL, 0x17308fbb9485056dUL, 0x21ae3e8c5533bf53UL, 0xe42791bf359d9f55UL, 
			0x705772e69c9166a6UL, 0x2d968c56fe6400cfUL, 0xf6de30cbb476f501UL, 0x044c96eb8c6a54aaUL, 
			0xd05da0980b8195caUL, 0x5769c133d871f4daUL, 0xeaa7bda077aec572UL, 0xddf40f1d78b2d013UL, 
			0x40ffab4226e82312UL, 0x4aa8940fd5b947f3UL, 0x169fe765be389f76UL, 0x28e8d78cf3472980UL, 
			0xe812047a9e504c00UL, 0x37641d86fd84678eUL, 0x6202e0d2f2781393UL, 0x46b4f3efa61c17c0UL, 
			0x482ca621f40b7733UL, 0x96207975b65fc272UL, 0x169e4bc49dae2f16UL, 0x869bb442af853272UL, 
			0x4815793eb152cb66UL, 0x299ca880a179f758UL, 0xa4b834e1bf6a4a35UL, 0xe12f46da6e887902UL, 
			0x20bf31fb7dc4bf48UL, 0x0ddb9e58f7a513cdUL, 0xfd0fba10ec0c75caUL, 0xa07fb3fece649a8cUL, 
			0x5f5b46beffa5c4fdUL, 0xf14c4909c104d5e8UL, 0xd85b24fcbeb91c88UL, 0x60e9e5db8ede9913UL, 
			0x93ef361d614bd29fUL, 0xcb2294f1cbe8da1cUL, 0x7babdfcb64e52220UL, 0x629f44802e8bcd38UL, 
			0x0b613e2906141082UL, 0x8d4eb3bf547f502aUL, 0x7a39789a364a7333UL, 0x2c94d7d9dc2fcf49UL, 
			0x7347d515d6045581UL, 0x481c5a51f035eab4UL, 0x46ec65d68c80fa2bUL, 0x02be74deeff1592aUL, 
			0x11106352f561f823UL, 0xc1fc2ac8fe33d905UL, 0xfe928e10390b5905UL, 0x24e213d3e0db58f3UL, 
			0x9cf690323c6242f2UL, 0xd2c758744dd82eaaUL, 0x5449de9a1b40f98aUL, 0xc74b3d741e0e8b26UL, 
			0x29e303c7d286c54aUL, 0x31ab0e0f547658a7UL, 0xd9d7c4c0f4a83ca4UL, 0x862bdf5dee7f7f65UL, 
			0xb2ec05979fc4c25fUL, 0x3e19cf9717c04eedUL, 0xea3f033f307f2698UL, 0xcaadc154a84bdf91UL, 
			0xbcdd6b11bf974ad6UL, 0x7121570893b35132UL, 0xeb584d4fdd88d6cdUL, 0xe53115fb96d963f8UL, 
			0x9796ebcb3dca7aa6UL, 0x44e8ddb90ddda23bUL, 0xeea5a8bc44f326a8UL, 0x3822e687a4a25de8UL, 
			0xec7cce87a709a556UL, 0x4be803ed2d12992eUL, 0x86d0408261514ecbUL, 0x490e3c1c120c40e9UL, 
			0xe124112cd27a19beUL, 0xc1df18fdb97b0b43UL, 0xa22241dcdc2b16c6UL, 0x89ecf780543288e4UL, 
			0x192f91ce85bd65f2UL, 0x0934e4cfced94e16UL, 0xfa7c2a4919251b5cUL, 0xa55cd01c33b5d617UL, 
			0xfb0c1082726e7143UL, 0x96610a6c6272eb25UL, 0x7a2476c8547862aaUL, 0xec87d0689bbdd25cUL, 
			0x4a82b94eb2fb15a5UL, 0x86440bbd18ec8603UL, 0x1eeab451ae4da0b5UL, 0x1a8e3803b2cde2e6UL, 
			0x45a1678b53f1bdd6UL, 0x04bc8c7d016bc252UL, 0x083f067efbab7823UL, 0x1b303789d4c071f4UL, 
			0xb7567751d39d618aUL, 0x9f2b7735527493daUL, 0x78e6c89dcfd0039aUL, 0xc9212151fef217a2UL, 
			0x78bff93d3ba4cdf3UL, 0xd3cf99264fdaa047UL, 0xf3efbaa80817f021UL, 0x9adb2ea51386d590UL, 
			0xf4334fb8a054c15dUL, 0xf8dc554de8c84510UL, 0x82773dda568e4946UL, 0x91dc2bd0cccd3751UL, 
			0x59133d837b24984aUL, 0xa8881f24ed49ba52UL, 0xee41d1d5bb70a05cUL, 0xe9a824e7d5cb7c6aUL, 
			0x3a2ef4ca101ae667UL, 0xc392aec3ad7c764fUL, 0x9a0fe928f1991ea5UL, 0x1f6c44e3e4a66696UL, 
			0x2de54479595216e7UL, 0x740fbeac576a42a3UL, 0x36c26af4ccc4af4cUL, 0xe461eb5a769306c1UL, 
			0x49bb141ccd791777UL, 0xc81f4bd272f95363UL, 0x577134f4e104496aUL, 0x5a96114b78b1302bUL, 
			0x3ccb4e730971ff3bUL, 0xca6a925712499354UL, 0x9b49c4a5ed856d86UL, 0x6ec1525a2530825eUL, 
			0x56f495d020a0be0eUL, 0x0a465f896224bfe6UL, 0x377d48de7e8fac39UL, 0xac2c5a86be4a50a2UL, 
			0xe5aa980b7a9b9062UL, 0x09fa694b714d057dUL, 0xf8270c27b3cace2dUL, 0xa011ad28b674f8d0UL, 
			0xece45db21ab919eeUL, 0xba2e6efb7ea93874UL, 0xf2c626e6c9cbeb82UL, 0x77ad89cdf6fa9b7eUL, 
			0x9130d0cd3d35bdaaUL, 0xe7853b2c2e7075d7UL, 0x5cb78257703dea5aUL, 0x163670c9b93528fdUL, 
			0xf486d384cf0aed4cUL, 0x40e76385f4248770UL, 0x61f44e828c4d74c2UL, 0x0b8ef06211d9c941UL, 
			0xcb2250947b6de4acUL, 0x643beb4d2616a5f6UL, 0xafa5c70a9271af68UL, 0x84856ae39e6b5f2bUL, 
			0xf0aa408648182439UL, 0x96368c07e455f525UL, 0xca93a028bb867fd7UL, 0xcb96b3bb3d5dfc0cUL, 
			0x16fe907df40028bbUL, 0x0f1a027f9e7edcb0UL, 0x2b100bd9572b646dUL, 0x6b5dc1f121dc8b76UL, 
			0x632657e5ed33239bUL, 0xdb0bf1c87d9aa4f9UL, 0xf1b9d4e77c9acd27UL, 0xe7e47d2ea9a05a3fUL, 
			0x9ce2f7051fb7a967UL, 0x797d429f5055fafcUL, 0x6d396ea0c8401781UL, 0x786f95e8347e4a22UL, 
			0x517fce971bd91d54UL, 0x870550232dc07c6bUL, 0xabce3429be25973dUL, 0x1455e128b04e196eUL, 
			0x25fce1d00af75ed2UL, 0x15b86a7b1bc928c4UL, 0x17f5bb8218e00866UL, 0xa5ca3d10199ecad4UL, 
			0xb86e9bd9aa45b4a8UL, 0x10fcdd501f76f5e5UL, 0xa24080231fd0ab9aUL, 0x2923a5f1dd72754aUL, 
			0xc26b25f9a2017271UL, 0x5af1bceb82a3b00bUL, 0x3c6a891fafce791cUL, 0x8e134ea91569d9e9UL, 
			0x2797870aee0783e7UL, 0xa1130130471a4b42UL, 0x4109aadb7605fdc2UL, 0x24985cc853652e5eUL, 
			0x59afd588b7977305UL, 0x3afd9b1a050b3ea6UL, 0x3568ab95d2b17770UL, 0xf38f5d651953971aUL, 
			0x63e3b99dea2f1c36UL, 0xb6c7ccc7554a830dUL, 0x3bba1aacd87c33e7UL, 0x00780a92626d0b4eUL, 
			0xa91caf66c1cf39feUL, 0x3acb2d34e28a7a52UL, 0xe986eb943416147bUL, 0x3c653065446d62f9UL, 
			0xa07f73ca9f3cbb57UL, 0xbc34c7439f58fa4cUL, 0x8c8ff63f87eb0507UL, 0xc01ea193bfa19761UL, 
			0x9470caebbd9a233cUL, 0x9dd824dcf1464d48UL, 0xcd1358454562362fUL, 0xea051d9af3ef39e5UL, 
			0x625d53b1f8b647e2UL, 0x7709b509e8a90c08UL, 0x375ce090d3171cdeUL, 0x8135ef210d5b4668UL, 
			0x20a2f4ffeb9788c6UL, 0x5f9f621390297733UL, 0xa4ac526abfdc9643UL, 0xf976bd001b517f42UL, 
			0xce13f8bef4694677UL, 0x774d7d9eac8f12a4UL, 0xcca0fc7004c4d9aeUL, 0x85e9510611418752UL, 
			0xa3dfaf8e51260dd2UL, 0xe8f32d20abf18c06UL, 0x6a6f4975d25b5dabUL, 0x8d11102d1d5b17e5UL, 
			0x92224ac64f6cdecdUL, 0x4a69de89ba13b7c6UL, 0x03fe369a9a5a5879UL, 0x4cb5b64b14af77b8UL, 
			0x67b0cefb77f87f80UL, 0x715574b96a840167UL, 0xb208dd3fbdf035d7UL, 0x13c3a907e172d7bdUL, 
			0x27aa7d1bc36954c8UL, 0x8f12c63546c267a3UL, 0x0afedc58508dc42dUL, 0x6e1c6432b1ec8ccaUL, 
			0x81e3efbe69b687d4UL, 0x329c58f155266cd1UL, 0xa5e560d1387ef9aaUL, 0x349cbec994b17fe7UL, 
			0x698f0dfcfae4d39eUL, 0x2a2ee3199949491cUL, 0xf8bc7a80c76021feUL, 0xd5e1939342dbf4e5UL, 
			0xfc7ba96bdf619bc4UL, 0xf89ce0e716f0d6e9UL, 0x07020c4fcb2028c0UL, 0xe2c2e83f4b3fb9e7UL, 
			0x4004048f1eed970aUL, 0x470da0ebd559c56bUL, 0xd5da8e31cdc9b2b0UL, 0x4d9fae50b34c6b8aUL, 
			0x7d29ca3f180ead2fUL, 0x949279c5578aec32UL, 0xf061618a989ab0dfUL, 0x4feec7965585af11UL, 
			0x200d6fb08ad755baUL, 0x319684586fefae8bUL, 0x0928f93394ad0aebUL, 0xba8318b7cd0119e5UL, 
			0xb75f3cf321d4ad00UL, 0xa9730d5988307730UL, 0x9f53ed4f137ab9ccUL, 0x2a4726cb5ad6cfc2UL, 
			0x96e2a025d2e48c8cUL, 0x9195e87c59c1673aUL, 0xbf1429b8708f3d59UL, 0x6769bbdece54eccaUL, 
			0xc807984173b09b78UL, 0x37c18265ba6cc98cUL, 0x38487c8d33a3fd0dUL, 0x30075da233a5a39aUL, 
			0xa8c3e1076b32b7edUL, 0xda39154f563a7052UL, 0x1e2f5669351b7cb2UL, 0xde73fcf14f51e308UL, 
			0x3d2fcdc62c6f54a4UL, 0x04dce3bc78679709UL, 0x951898e53d3bb285UL, 0x651b3b4bc2878b0fUL, 
			0x44a94997a472ab51UL, 0x5df6d89bc1c2002aUL, 0xffd75c317ec268bcUL, 0x683290176511b409UL, 
			0x7b656bf9dfa5f4bbUL, 0x8ffea23a59be634aUL, 0xfd1c201223b1794aUL, 0x3f3071694d8db1e3UL, 
			0x01e540224b82c3b8UL, 0xba80ebbebaf4b855UL, 0x0eb8597ca047cff9UL, 0x0e28fd6f5a019306UL, 
			0x6b75694763cc8fa6UL, 0xfc711bb8784fb188UL, 0xf2948454290e0b8aUL, 0xd3fc0c62726a1a9dUL, 
			0xeb9c8ab97f3fcb63UL, 0xcbd8dc5d7b271ae0UL, 0xd42b85a43a6f07bcUL, 0xbd647b637f21c4c3UL, 
			0xf8895d980deebe03UL, 0x6e7225bc1bd2c125UL, 0x5a8581d4d6e42052UL, 0x2d13e0159d2c8873UL, 
			0x987d73238d587653UL, 0xd0dd94dd4c94068cUL, 0x0ba1aa053291b211UL, 0xbaf491b77c60744fUL, 
			0x5f97ea9fd3ac1594UL, 0x1ec1f87ddd507623UL, 0x56b7eac6f77f3d22UL, 0x6104a8deead673bdUL, 
			0x682dd310b0792035UL, 0x6698b68f0b8b2a4dUL, 0xc41a6fd5d33073deUL, 0x46835c80ec5a0c0dUL, 
			0xa5f5cfa199e0f728UL, 0x6c0b55178fd9f78bUL, 0x21bfaf866b9146d1UL, 0xb86df4156a98d1cbUL, 
			0x525c64132c4e7058UL, 0x9dcbabde725c25a2UL, 0xfe7a4925a0d162e7UL, 0xd8300ebd46648a92UL, 
			0x150a6da29beb2d0eUL, 0x9953d50c2de92972UL, 0xb30fb44520bbbe28UL, 0x59d1691e33da4ff3UL, 
			0x13a4b2d7007af0a9UL, 0xf1a3e5c257c72656UL, 0x1b6414512a66dfe2UL, 0xd58146d49f753bb3UL, 
			0x126287094912e195UL, 0x1b5076c2f9c48e3fUL, 0xc35814847d270b87UL, 0xbf7a4423f3dc4248UL, 
			0xdc723a7a49ad333dUL, 0x311d76dd6ae24623UL, 0xd4597fe86eceb8e0UL, 0x0df2c052be93400fUL, 
			0x566d39249722df46UL, 0x28e5e28f8ad4b58bUL, 0xf20a1d0e017f2c6fUL, 0xbf541ac397db2154UL, 
			0x14f040ea7551807aUL, 0xe757e79d5264dd34UL, 0xc0fb66549c50e891UL, 0x85cc6c41a53b2885UL, 
			0xa9f570857dc49248UL, 0x4356051e95711513UL, 0xd27b17ae383bd02bUL, 0x9900c99b499d9854UL, 
			0x9a82077230a3113dUL, 0x3f5fc2ce9135e3a7UL, 0x3de86407f7d0f676UL, 0x9f6e1afe8bc816e1UL, 
			0xa854d20cf8a37532UL, 0x70703ee8462ed8b4UL, 0x841a92f9b369e864UL, 0xd341e4fdf17e1c5dUL, 
			0x50c4a7cca7658a6eUL, 0x687d1d74887d5433UL, 0x1364380465435109UL, 0x34682754184f9b7aUL, 
			0x9f725a82901a3b8bUL, 0x0527670335e1eff0UL, 0x42826ef011bc2493UL, 0xc971f14827bae2d8UL, 
			0xe1b8dff22b65bf28UL, 0xd148d7c7dab18973UL, 0xc3b261eac4b58047UL, 0x84c1c063ad360fb7UL, 
			0x9acf8c728d18aa7aUL, 0xc86c29e0effcf657UL, 0x19cfcd1807c7eabaUL, 0x71acef13b93408c4UL, 
			0x1cb29b4a50f9ec38UL, 0xd6c3faf4c23cb828UL, 0x9e51d27912a82807UL, 0x040550a3389d9c6eUL, 
			0xe11b2870b03f6cefUL, 0xe2b46b2f28487490UL, 0xe1ffe885bf011cdbUL, 0xce24ee9803715cc9UL, 
			0xdde3abcdfc5991c8UL, 0xe2a76f163392b758UL, 0xc3f59a811135b8baUL, 0xa4407f90ab53a5f8UL, 
			0xf5b98c5470f83005UL, 0x97fbce3f33467d48UL, 0x768626261ac65094UL, 0x294c961a75e09da9UL, 
			0x5c4d08768fdee1f8UL, 0x26dd75fabc4f1413UL, 0xf6db98a0196fc27dUL, 0x83ae045f9c8f58e7UL, 
			0x3553eebf861527d1UL, 0x41bcfd6e9ca95d03UL, 0xb149f6a19660d23dUL, 0x85f178d254e952baUL, 
			0x945db0a6ac93fd49UL, 0x491ababd52469b66UL, 0xaca4bafd2dd81510UL, 0xdcbf5714bc48286dUL, 
			0x98415fe3600fdf12UL, 0xd746e1ac9290687bUL, 0x26aa6d883e7b987bUL, 0x616e3fcd35f89427UL, 
			0xcd3df3d8c792e8e5UL, 0xc7043540e2b4af49UL, 0x768e8a1775c1fce7UL, 0x6bf37cdd0452cbacUL, 
			0x6e8a3ae38bf28b32UL, 0xdaf610afa26468aaUL, 0x4af1ce44a27d94b0UL, 0xb356fa087dec4953UL, 
			0x29da69cb4ce34db0UL, 0x4a862f7d6eed4ff7UL, 0x93c59ffae11778e6UL, 0x1f29d5043db5d99bUL, 
			0xb5e257ce7340806dUL, 0x6673d588ce28d81cUL, 0x0c5f3c2d19f13677UL, 0x3f4a62ea2e4de161UL, 
			0x8525c91cd87a0f59UL, 0x37ebe62884c7418aUL, 0x2a961ac577a09bc7UL, 0x341d8ca3d7ce9747UL, 
			0x011d7564226d0bd6UL, 0x1ddebf16126a6bc4UL, 0xcd448216a64f1288UL, 0x72506e176017545aUL, 
			0x55e5e1d3b4706cfeUL, 0xc98025ae72e86073UL, 0x1b49552106902eb7UL, 0x410187aaf5bba687UL, 
			0x2cb232b6d72360ddUL, 0x29083283e351e220UL, 0xd3b03b690ae915adUL, 0x500b198f72974552UL, 
			0x41839eb5b8b86f44UL, 0x172758974327d0a3UL, 0xaa6d7008906d870aUL, 0xe8428e8d92337bbaUL, 
			0xda010a7c29bd9f19UL, 0x74b29ff0be6cf933UL, 0x99bc5872344473f6UL, 0x3dc16562acbebd0aUL, 
			0xbd2943c32f7cc3b0UL, 0xd45576a90390dc95UL, 0xa64f85081726efa4UL, 0x42a0d17c33fae4d9UL, 
			0x7879c4aa78833677UL, 0xededb781723abf4cUL, 0x86b38116db9220e7UL, 0xdb13c96c81f737c0UL, 
			0xd0fe2646de452ef3UL, 0x34d3d847abeb6768UL, 0x72ef43c810b19b6fUL, 0xb57dd78bd990b446UL, 
			0xd96781af10674a54UL, 0xc50f5d1f7079d7afUL, 0x6ec1961d48e64450UL, 0x9656e4f1c247ba00UL, 
			0x1cfc01565dfa5a8cUL, 0xebe891d8071fa87dUL, 0xfa1302d8da67202bUL, 0xca4564721358b8e7UL, 
			0x5a83dc1f6ba0fcabUL, 0xcd8ebf4c4120fa5fUL, 0x7e98055b8497dd73UL, 0xc3147cf3addb50efUL, 
			0xa92bc290d43c4fcdUL, 0x760b2d05797cfbc0UL, 0xa241b2dbbe48a5b5UL, 0x0a2d096eb0665957UL, 
			0x74f0cac1ef4f6850UL, 0x612d9856f808cfa3UL, 0x236a8d29595ea935UL, 0x554141f991cb7a42UL, 
			0x918a00eefc7da5cbUL, 0x8ac27ef40e2b44e8UL, 0xe7ca179c08026e6eUL, 0x8c8979865126a4e3UL, 
			0x30f7f00738ba7a8cUL, 0x12f783f3159017a8UL, 0x06ab4fd3bcbee152UL, 0x7c2dc419e0de8aafUL, 
			0xc1900b31c78d7642UL, 0x4b406025e7bb062aUL, 0x2560be118537fbccUL, 0xa7f75c2288c020b0UL, 
			0x470a017ad2e23caaUL, 0xcf51fd1031102337UL, 0xf01fc501ba3d2c55UL, 0xd975adef286fa4c0UL, 
			0x10a20e1608654b92UL, 0x899e0344e6e2c714UL, 0xa418d934039f37edUL, 0x3a90c70c2e1992c8UL, 
			0x8a0a33303c27ede6UL, 0xe9c9157a2e4ab1cbUL, 0x15b49987706b30f4UL, 0x57df0bc8f419828aUL, 
			0xd62a7bb58f4f6b2eUL, 0x409de99dc74c0779UL, 0xef550a5968125d0eUL, 0x44ebe7dccddbe4a9UL, 
			0x3c5562cf9454f14aUL, 0xaf1b3f8210dc9585UL, 0x23ca253770baf8c2UL, 0x5116b6f89cb1862cUL, 
			0xecd40c3fb763999dUL, 0x23b3d7acf748080eUL, 0xd78d218e5931e433UL, 0x6bebedb5ba624767UL, 
			0xe51615ac0317b662UL, 0x8090d199dd5f6569UL, 0xc982200acc2aba93UL, 0x61fcb3f26a89a237UL, 
			0x6b7cab6ea2a515cbUL, 0x912000a08f25a091UL, 0x719838bd15170cfbUL, 0x4532da55e7325921UL, 
			0x30e6e89edd6fe017UL, 0xe82f8dee95269573UL, 0x5d48e11e2fd32db4UL, 0xcfea581cfc2416f2UL, 
			0xcf6b1345e5e5e7acUL, 0x03313d5436c8dc91UL, 0xd8d260e712b3a950UL, 0x6881ae829a10a6f3UL, 
			0x133732e609525cfeUL, 0x3289fe80d480eb5bUL, 0x85b1c4bced4b5b3cUL, 0x16479bfa5ebfd5a6UL, 
			0x28dbd14071009ad3UL, 0x8ac59101b31d6886UL, 0x51535831751c222dUL, 0x9f2b6c2edf37e9e2UL, 
			0x1ecbc381b701a6c2UL, 0xce647118c1925203UL, 0xf7df8df5d5bd28b3UL, 0x6dcb6f599db01bcdUL, 
			0xef13457cc3909a33UL, 0xe9c4799e9384dac3UL, 0x056331e253c29e8bUL, 0x4c9c73005e11925eUL, 
			0xecd26981ae3b9f75UL, 0x308b41a11505ec52UL, 0x8f86452c39130f69UL, 0xfffae591d4e96173UL, 
			0xb641b68da47bc04aUL, 0xb4e5b7c4a9b29c75UL, 0xbd531ec72f4ee467UL, 0x1c099c7a7a17709bUL, 
			0xd3e065159a7e686dUL, 0x8cced0ade5005ec4UL, 0xcb04217ad3243b66UL, 0x71e6fcf7334a02c4UL, 
			0xfe8b59975ccf1285UL, 0x63ed26495bff0cd7UL, 0x3ebc0dad361a594bUL, 0x6b7b850c185eeaaaUL, 
			0x5c617b645be9f474UL, 0x4d76bb81fcc2ac6fUL, 0x2986c6a2b4e2a5c9UL, 0x093e406351b1267cUL, 
			0x6b20ba6a2cc140d6UL, 0xccac4c168ca29501UL, 0x1ac134338d50d6dbUL, 0xbbf0fe844269d9efUL, 
			0x6ab5b18a23260475UL, 0x7ce820f5abfcec0fUL, 0xb9a2ae0ba9e345e8UL, 0x4875aa48dee47c29UL, 
			0x958cca2c30701fbdUL, 0xe2c7397f00708c60UL, 0xc4114a320e11b2a5UL, 0xa26cc43f3cbddb6aUL, 
			0x2a0e641d675361b4UL, 0x62fde291b7d365a0UL, 0x17c11d785951b6afUL, 0x81513acf7bd322caUL, 
			0x98b0efb8d8e1400bUL, 0x1eb8a6a6ca6d26acUL, 0x8b67287320ee53beUL, 0x126c634f3d39ea98UL, 
			0xf0384373fca1b5b4UL, 0x6e102cc1849a6e47UL, 0x5fce5c2b7f818d55UL, 0xd02fafaa1f558b0bUL, 
			0x60386548ece63cccUL, 0xeabbb8cad882eafeUL, 0x0bfd2b83a2152dfcUL, 0xa28dda3817bb3145UL, 
			0x7de258d2a35ef132UL, 0xfbb8de08eab97bd8UL, 0xd8a02750e9f732b2UL, 0x23712e06cccca25cUL, 
			0x159c1b626c7603c9UL, 0x572b4843098d1a37UL, 0xb7bdb176a6e0dac1UL, 0xfd649803c07b32eeUL, 
			0xaffeec4e3471ac59UL, 0xf74a7e896cd5a808UL, 0x322e1819fb4440c9UL, 0x2a4c6c5b1822c49eUL, 
			0xadd4c2f4f3c74f1bUL, 0x047a57e0eb748fe9UL, 0x9900be756262f4c1UL, 0x3ddd8c3632090ff0UL, 
			0x39140c2b7892281aUL, 0x69e72922a7db4c77UL, 0x6dcc848ec648e89fUL, 0xe6efea5673e523eeUL, 
			0x27b4d83fc597af7bUL, 0x8c54b080f607dd7eUL, 0xffec6b715548849bUL, 0x63e9a42458f16bd1UL, 
			0xe07437d39c203dc6UL, 0xd9ec27fbfee476daUL, 0x2cde9667685a6e28UL, 0x4f926092d8e9022eUL, 
			0x645deb4fb2109760UL, 0x7be11c5670d5f2c2UL, 0x6e9c1480d6682d55UL, 0xca09d3e56b01b2a9UL, 
			0xca0214a131504eddUL, 0x3d7623bef06a5e47UL, 0xe290e664f95399f9UL, 0xb4ed7c9608e93e3bUL, 
			0x562876c4394b9d14UL, 0xa3f21a54c4a0e969UL, 0x6460fec67a6b0130UL, 0x6cddb2ebe0729ec4UL, 
			0xd4260a568d702e19UL, 0x513835b901a34f68UL, 0x1beb462b9783b778UL, 0x37db6c54b93f6381UL, 
			0x35b098f36d00b54bUL, 0xc630e26c808a83d7UL, 0xade730765c1c6a92UL, 0xb07031644a3fc1d6UL, 
			0x936ab90717eebe27UL, 0x9ebeb56cc4b4b4c3UL, 0x2d0aefb80c3bdd7cUL, 0xa19bc4fb2eba7e38UL, 
			0x02d8060b93602cc3UL, 0x4c2ff143cb9a70bbUL, 0xc9249efd8fe20214UL, 0x3f0814b3e2bb9a39UL, 
			0xe24cb6fd1c83b6b7UL, 0x35acfb2a005eed37UL, 0x4b6ef51f554f2ac6UL, 0xf9df3d2ece0cfde3UL, 
			0xbeda3c6e17640245UL, 0x06669983b736735aUL, 0x40e1623a1a3d93eaUL, 0x4b04683cb3dc1a6dUL, 
			0x52aee598323a452dUL, 0x3e1a9c30741005aeUL, 0x7582156fc365134aUL, 0xb21e1cb2846446b7UL, 
			0xf52c39dca357d3ffUL, 0x7155bfff7e58da49UL, 0x6a47c07b051e3c3eUL, 0x3641aef9e5964526UL, 
			0xb45e804a74194dc7UL, 0x92781375bf6ae516UL, 0xeccf277aafe598d7UL, 0x75bd668152ea3ec9UL, 
			0x8c9d62dc21ace92aUL, 0x0d78cdd0a60cd7caUL, 0xd75279d6aa43946dUL, 0xa99dac4e3c1e3ccaUL, 
			0xfebca3e5aa3ddfacUL, 0x417712e93d0569fbUL, 0xd6dd2abbfa85b573UL, 0x242a8122feada618UL, 
			0xfc9f810a87b80891UL, 0xfbfaf35d7109de76UL, 0x5f51751f383be117UL, 0xb7e43b6412f42c64UL, 
			0xb2c20ae6a4689936UL, 0x80686381865a598bUL, 0xfdafed02f51a0d0cUL, 0xb6cd9aff42108db6UL, 
			0x5681ee40806fe327UL, 0x380a7c1601258cddUL, 0x7ad6807c4008a5cdUL, 0xb6196953f1ea0350UL, 
			0xbdc498bfce42fea4UL, 0x0cb604a61ec7f61dUL, 0xa47cb1d9e18a5caeUL, 0xc3aaf4c41cbebdd4UL, 
			0x4b6903fec67bbe63UL, 0x69cfbd0d3e39b363UL, 0xb9e4235e017cf502UL, 0xaa85ca50c32d57a3UL, 
			0x8a220e26baeacc7fUL, 0x7825360951afd08cUL, 0x52a9ceae999ade79UL, 0x9255ff07fd62ffb0UL, 
			0xf7bcd3c8e0e73db1UL, 0x7f3bbfd5170852d0UL, 0xa9d828ca08b05378UL, 0x65147d7a9229cb25UL, 
			0xe1e2c3c6e72cacdbUL, 0xdbe26424f8703091UL, 0x62717f4a93e340c9UL, 0xcc635a45f834f760UL, 
			0xa51aa752baf28cb5UL, 0x5a3cd51cbe85d509UL, 0x7f6a348fb24ed88eUL, 0xaafd560d4a59e812UL, 
			0x6e971d3726bf57dbUL, 0x8d52b0237bc5920bUL, 0x8ff5238db58bac9aUL, 0x5e75bc5f18e31ac3UL, 
			0x73c1301e8b21f7f9UL, 0x3f05e8cd022fa65eUL, 0xd665499d19dcfcb6UL, 0xc0f8bd3459cfaff6UL, 
			0x599165de05ca30aaUL, 0xdf322d950f6751bbUL, 0x2d2e56ac56d2e493UL, 0xd3950819e199280eUL, 
			0x5980ff1415218463UL, 0x0f56b9dea3715628UL, 0xc22d35b5a92d82dbUL, 0x3adfc59a723a4a22UL, 
			0xb66c077dffc80a6fUL, 0x0c4e5b18af1f4534UL, 0x0c4de9c65c5015a8UL, 0x7ecdc07429ec7190UL, 
			0x833bceee3fc7db50UL, 0x8f1a5e3e49720c88UL, 0x658ebf0b7a1b0763UL, 0xd70b65e4185d5fb0UL, 
			0x1ead146ec98f7be6UL, 0x5bc1ba8063599480UL, 0x7cc72af53178cf5eUL, 0x137be9e17360cf80UL, 
			0x873db56978063554UL, 0x72b9bfddfb60323fUL, 0xc419973d3edd4e43UL, 0xe69d8d9acc4624bdUL, 
			0xc341b1b04a0cd1edUL, 0xc8cbe2ac6b328f6cUL, 0xa3617ccad0b8f5acUL, 0xe03118c81ee5f322UL, 
			0x117ccb82ac167225UL, 0xd1da146a60739628UL, 0x1b8ae5bf805e685cUL, 0xfd48dbd0f6b1f1c0UL, 
			0x4da989f7377d8517UL, 0xb534ea12a16564a4UL, 0x20b361c692412df9UL, 0x6f5e88782ee4b96dUL, 
			0x9278845630f18aa2UL, 0xdd949db0d9fea02eUL, 0x00d2c89fa6d1c959UL, 0xefe6f403c9067c7dUL, 
			0xc1de26ae71af5630UL, 0x12de7d79045e964fUL, 0x38397a678e17a5c9UL, 0x508bdbd2bf1d288aUL, 
			0xdb174f9454925432UL, 0x014469634a88d36bUL, 0xcd512e3bd1664a37UL, 0xd83053495004621eUL, 
			0xaf58074672f8afddUL, 0xa28f9359adb71709UL, 0x977b767ee4ca97b2UL, 0xa77d228adfd2e011UL, 
			0xceacf06e0ee75458UL, 0xc1d6dd8dbfd02e24UL, 0xe8206eaa9bc6706fUL, 0x9ded012622a8e2f3UL, 
			0x0f340be25f54aedaUL, 0x9316ebc74ea006a9UL, 0x061e486fb3004e6aUL, 0xe462eb22bd5cbea0UL, 
			0x9b571620ec8fc573UL, 0x6bb508b4c5644c9eUL, 0xdbb2a7b8fb6ce946UL, 0xa2af2db01ad7a63eUL, 
			0x5d0758a3b0d80a0eUL, 0x65b0798b16f73b27UL, 0xc9888c520cafe8e9UL, 0xd5a41b1c19d0687bUL, 
			0xba0fd01a172799c1UL, 0xb66ebcee133d8c64UL, 0x499c7a92ab8bbd4aUL, 0x9c95297611be1678UL, 
			0x59a4de7810cce5daUL, 0x60768dd6275d9192UL, 0xbfd1194733f87a8eUL, 0x1d11b3cea9b55991UL, 
			0x97a5f1eb5221db6dUL, 0x4f700b535d359450UL, 0x7c3c30b959bcdb9cUL, 0x513a5defa4290dc8UL, 
			0xf80f0c5d38861962UL, 0x0e6ea957b9834599UL, 0x85af0396d632a155UL, 0x059c1bf277dda97dUL, 
			0xafd4e9a536f40bf4UL, 0x5c26fe304b503b88UL, 0x345c40f732fc235aUL, 0x8f78df11518fd5ebUL, 
			0x86f0fc93a5d71cc6UL, 0x7324166859e5039dUL, 0xd661bd6ddaf8050eUL, 0xa096c1b0dc3d5c1dUL, 
			0xad39ed5bbaf151b5UL, 0xacfd936f83a8b786UL, 0x96b9a1b83cd87fdbUL, 0x12645d00ea65bec2UL, 
			0x4490ee8a1d85fc97UL, 0x8f1c4cc5c44784bfUL, 0xaadbe0d822872ad1UL, 0x78fc2eec4acf9b0dUL, 
			0x82ded58b8d6d9e18UL, 0x20623d25213b3783UL, 0x84084039ce4f4e4aUL, 0xa121b253c8c16138UL, 
			0xb9baca1a16009c7dUL, 0x32a9804b66bd4a06UL, 0x72924bcf07730342UL, 0xa4b88acbae6802baUL, 
			0x2e61a91a58169ab2UL, 0x297129e151df1c8bUL, 0x4e1b0507aa849ea5UL, 0xeab44ef6eef63d61UL, 
			0x76c9e89465a4ab49UL, 0xdecb396f9dd414e6UL, 0xbee9e7a81e5d1efaUL, 0x9a518c89f3917c63UL, 
			0xd0e722a2164a3691UL, 0x3d6d7b476fdce9b2UL, 0xa8713950454e05ebUL, 0x1451abed0bbb2ebbUL, 
			0x6ad734146fa752c6UL, 0x8def3771ae410716UL, 0xebf8ca8af55ad087UL, 0x94c3db16790a715aUL, 
			0x430320c57ccbb4f9UL, 0x20b37abdd3e08e20UL, 0x46c4975915345581UL, 0xeed9fa0d765ae2d2UL, 
			0x72b000f2a4605a04UL, 0x51ea1204ad923f06UL, 0x785876dad4c44daaUL, 0x2a3b8c36a65cd91eUL, 
			0xb7203f1335710f85UL, 0x233d108d81a86f3eUL, 0x2d344ca2f03c83aaUL, 0xe6b77dfb23f2b4aeUL, 
			0xfe4f40c12d7af66aUL, 0x88b014172cdeb09bUL, 0x2b7067edc929324bUL, 0x0518af69248518e9UL, 
			0x2987238d278800abUL, 0x163407bcb3f71b94UL, 0xb5b09968887433a2UL, 0x80f54cafff6b04f1UL, 
			0xe063c2e7a5505ad4UL, 0x6386d92b4210f136UL, 0x2c590581fc556090UL, 0xbf55f3266a97e789UL, 
			0x69117c6575cd50b2UL, 0x0faddb2cc7e1518cUL, 0x1c051ea56d0a6028UL, 0x7068ee71f7d4e772UL, 
			0xe5f7f01508a5262dUL, 0x509a76c9063c78faUL, 0x0f483fb5f52367dcUL, 0x222ab6f2b5929c2dUL, 
			0x7dbfa9b26952f467UL, 0xc83f7f6a48fade01UL, 0x652a58c955e7c8adUL, 0xe4c0862376330926UL, 
			0xbafa008b1a17f8afUL, 0xadb5733705c7654eUL, 0x45ea6b71c9cb5f3cUL, 0x52f0bbdcb3ce28deUL, 
			0x56e4cd580b085e33UL, 0x2e9f56c11ddfcdb1UL, 0xdc2df902bfcb1ec6UL, 0x2d4a98977e5ee74cUL, 
			0x258b7367ad1f77c0UL, 0xd251f7b4f1efa26eUL, 0xe084a40751fe668aUL, 0x4d153671a519349dUL, 
			0x9d453d0c0eb60c0bUL, 0xa4b0e9e601cb4c4dUL, 0x4951ba9f957d81b2UL, 0xcc40ec1425b878c8UL, 
			0x641a094d41f5468aUL, 0x14f144b6297052d6UL, 0xe94e45e40931fa51UL, 0xc0faae3fa8f1fc6eUL, 
			0xeefaef9bd792b198UL, 0xfd107d0914701621UL, 0x8f0da86383e2e8f0UL, 0xd691aacd14f3947aUL, 
			0xc6360b651bbf34c1UL, 0x0baf37d3d5ca791eUL, 0xa4986872084e5208UL, 0x12c394aacdef7ae2UL, 
			0xb2dc804548c6a44eUL, 0xe54848d2a2b2aaccUL, 0x99217f0f604cbf18UL, 0xb71df2fd851791e3UL, 
			0x5f3c77c84601baceUL, 0x502840e303da5980UL, 0xbdc6ae1a5e79838fUL, 0x5d4219f4ef64c37cUL, 
			0xa235fbb4d31acb8dUL, 0x20a3d4a233520c24UL, 0xbae001e9293c05c9UL, 0xfd9ae117ec02682eUL, 
			0xd130390ed2ef72f4UL, 0x63f6752b22a068abUL, 0x4672e28089ca6141UL, 0x40aca25afadb8430UL, 
			0x15bd4ab11ed8f29bUL, 0x5984a48a869bd59fUL, 0x7f3c7ff87f467aa5UL, 0xae9c7c2100507a3bUL, 
			0x74d248e8a5973782UL, 0x107e356dd16cab65UL, 0x356b309039b40e24UL, 0x5765a93b0ecfa881UL, 
			0x45048a829fcbf1f7UL, 0x76741b561ad6f6e9UL, 0xa603be3c59a044e6UL, 0xe6acf77d6f52d9ebUL, 
			0xb6d955fc9fa3b329UL, 0xd4f16f138bbb0ba1UL, 0x3c699ee2b0d84624UL, 0xfc051ebe859824a5UL, 
			0xb7fc0d6673b58077UL, 0x1838b599fa157480UL, 0x601c8db5a32b7747UL, 0xbfe4d96e8c295eb6UL, 
			0x62e9782f09170624UL, 0x83eaa3b64445dffaUL, 0xa975e9c083b5a384UL, 0x8f070dcf22e9816fUL, 
			0x3848dcfa6574428dUL, 0x42de58ec30123c21UL, 0x2db97a39a2bbbb8bUL, 0xf681bb9c863bdbeaUL, 
			0x38109f6c072e3d3dUL, 0x747e498dd77b0174UL, 0xedf0d80400128dacUL, 0xeb094a828f70f034UL, 
			0xbc4c5f787d5b3437UL, 0x455333510250b2ffUL, 0x293da9bf18da1179UL, 0xb06b76798653e72bUL, 
			0xf20cd136bad7b965UL, 0xb61dae74ff4f453bUL, 0xf3b8d94b7a489b0aUL, 0x8992f1c1a50e79e9UL, 
			0x2a0f15a955c43a92UL, 0xb0e0e58c2ff30423UL, 0x3d4c07af34a12cf3UL, 0xfe7c418211fe5c0bUL, 
			0x0ec925472f1ada47UL, 0x2c74d376dc9c170cUL, 0x1cfd80f6a050bdc7UL, 0x80b1db42ed31c947UL, 
			0x314908e4e3fc3fa8UL, 0xa625afe2f8e4e582UL, 0xa6e0d45030d5a198UL, 0x2819b4a4a8df4268UL, 
			0x8e7d20daa045bbcfUL, 0x39f543de98e91955UL, 0xcd6a81d80a6693e7UL, 0x4642603fa36772a7UL, 
			0xfcd545b830345a13UL, 0x9ba69f301a068a99UL, 0x22286fa6ec405b6dUL, 0xfd90e22bd9cfe82bUL, 
			0xf801eb20baf05782UL, 0xa12bd10144c4c9b4UL, 0x838fabfd399a6a6eUL, 0x3c3411e54bd721e6UL, 
			0x6c7c5d0df8032a94UL, 0x95159817688e8c79UL, 0xac8c9c361f391ea9UL, 0xfb2ccb154d511789UL, 
			0xa11f65618669ea86UL, 0xc135b86bfe7c3eedUL, 0x50dc6a745860730aUL, 0xc7b7a7f06e029c3dUL, 
			0xd73ae51cbdcadc18UL, 0x38b7426f0cdf3a8aUL, 0xceb1a30079fc4bf0UL, 0x0251a96726f04265UL, 
			0xca650b2a7fcbf266UL, 0x88155951beb5b9f6UL, 0xbb11b2badcdb8cf2UL, 0xfc794ec7f6537e00UL, 
			0x3d15fb6d5aee64c5UL, 0xc83b9f439f96a251UL, 0xfb0a8b870d4922d0UL, 0x7f68a52242414937UL, 
			0x94f28792fbb7a02fUL, 0xe029eef4506a68a1UL, 0xb71317dabe669fdfUL, 0x1bf649d1875d1b30UL, 
			0xf2d6c47a523d5a34UL, 0xfb3097968126637dUL, 0xef00f121f880ae49UL, 0x85fcea75124e1abeUL, 
			0x0f22a9b12d0ae2e5UL, 0x828c6753d3fe7fbaUL, 0x3bfb15e3093f5d3dUL, 0xc65d140798c615feUL, 
			0x4590baea3344a6ceUL, 0x178017f904606882UL, 0xe8268cd9bd4efd5dUL, 0x3291493c38b1827bUL, 
			0x312bd852de6be73eUL, 0x7d55fa7aa5fdd64fUL, 0x02f60c05559859c1UL, 0x63fa79f5b752986eUL, 
			0x54a21bcb15f815e7UL, 0x9f2dc1461a782e11UL, 0x81660c100e5aa2d5UL, 0xfe0772e67487d71eUL, 
			0xaa8c32597985ff5bUL, 0xc6ef60b226ab4993UL, 0xc0f67f324f8b3d55UL, 0xb625cdb222af9229UL, 
			0xc59113890441aa16UL, 0xd441be564beda4c4UL, 0xd65c855e28ac7a59UL, 0x7646968ce3023230UL, 
			0x4069f3ff906ae8e8UL, 0x544fe87048d5af4bUL, 0xc4c5f56ceba7b391UL, 0xbb238ebf5e0dcdc2UL, 
			0xa6a79ca669cbabe5UL, 0x8fd5d8cf2313c52eUL, 0x6db1ecfc29d0bcf2UL, 0x78de7c942ea9547eUL, 
			0xcb060f43624ef358UL, 0x1b8fb495cf75c7feUL, 0xae5660af8440e817UL, 0xa6d54d3149571913UL, 
			0xa12783c9457a9bf7UL, 0xd79cbd349339127dUL, 0x249819b75a19db5fUL, 0x2cc588e4c801bc47UL, 
			0x5291a317c3b29620UL, 0xd21d83edd0b6751fUL, 0x1e0fa94345ee1594UL, 0xae08ceafc1a3f6ffUL, 
			0x9d40d1f6cb3ce8d2UL, 0x2e7952bab80cee1aUL, 0x088e8fae08d62f33UL, 0x1a644aa9d31cf7d9UL, 
			0x28bb044a64e8658aUL, 0x5911c28cdcdd2db6UL, 0x8a0524649887e082UL, 0xc42da29c1f3aeea7UL, 
			0x900cb9c7ffa9d86aUL, 0x659b1ef1f6b3a755UL, 0x896bf9742f10c10fUL, 0x827513f1b7df1ed9UL, 
			0xa93985e815199426UL, 0x81c2302366aaaeb7UL, 0xfb583da7ce6198eaUL, 0xe9df3c14542f674cUL, 
			0x2202c09da5d8dac4UL, 0x2cbf8c1abad81fa1UL, 0x9b205a03e10ecc99UL, 0xe1dc8ee9f2a10276UL, 
			0x7ccbc45bb212f331UL, 0xc3b2d84e73dce51cUL, 0x148616ec7228a490UL, 0xf20a40aa4bc5d3a8UL, 
			0xc8367552f23f6cf1UL, 0x5b9a6c7934c0e7dcUL, 0x5777af743fcf6af8UL, 0x0ce0ab79a0c25317UL, 
			0x19fa4d1b831e7978UL, 0xfef7e7d8ed291b87UL, 0x4330e53b60b94290UL, 0x097e81a65f53c8baUL, 
			0xdf005b0932a073d5UL, 0xc7cd1767481fb2dbUL, 0x6a5446ba0e330e9cUL, 0x676f9bfae2c75eddUL, 
			0x665f5b4db844f72dUL, 0x862cc9cc0a561ebaUL, 0x43b46c9d6e9e8202UL, 0x8ec54510f85b9ae2UL, 
			0x6111a5b78f7a5f81UL, 0x0d24a24f8c0c052fUL, 0x76a770821dcbb435UL, 0x89fce6deb9462be5UL, 
			0xb24d52ccbd1ebd37UL, 0x885683a87692edd2UL, 0x4a8a39eb795c5830UL, 0x2df21f00897ce6f8UL, 
			0x63d1180e6ad55129UL, 0x064166a11b0acd2bUL, 0xb2abbd7bba93b829UL, 0x8b66bab8fb193ecdUL, 
			0xdef86b0fa3530fbbUL, 0x2a1a56f4915dd075UL, 0xce08ddd1f8002a04UL, 0x8abf0440e098732fUL, 
			0x2c253d7efef7fe97UL, 0x6c9ca49e3243986fUL, 0x921119ad39dba78fUL, 0x86067b76b3b7529eUL, 
			0x016676d87290c70dUL, 0x85b24b75e8a39400UL, 0xaf79b3d4b98480abUL, 0xa19599f98e2c0e17UL, 
			0x85a36293f1cb32adUL, 0xd131d918707a82d9UL, 0x3332cd0a47ca1f4fUL, 0x11e1cb47fec8f025UL, 
			0x70eb01a936cec23fUL, 0xb0762473f6139a28UL, 0x71abdc85f0df0c06UL, 0xeb0d97db8114342cUL, 
			0xdce0f5c079f74ea9UL, 0x4fbb95ab304ddf59UL, 0x8a3a16dde4055900UL, 0xa48828b12f9ad644UL, 
			0x16b6bbd539818a77UL, 0x5478db2fd006cc1aUL, 0x7c093d8661ad6ea8UL, 0xa063276447236af7UL, 
			0x7b9163425f0d24c6UL, 0xb40150ebbe96c346UL, 0xd2e5b2642f8a06c6UL, 0xd2d98be9ff02c2beUL, 
			0x8ec7d9af32a1050eUL, 0xb093111c73d2fa0eUL, 0xe836a3a98bffa69dUL, 0x41cb1613374ede52UL, 
			0x02e63facb41387caUL, 0x98045ee18f0b5efcUL, 0x9a11ec938cc39af6UL, 0x4bbd79c2f1c15851UL,
			0xd250957d92528ffaUL, 0xc52c1aee1998a055UL, 0x9f13031bfb9a5398UL, 0xfa5d41140432fb3bUL, 
			0xc40e78210100f2daUL, 0x2069958698560eaaUL, 0x27c7f74db955009aUL, 0x2b6cfcade683a623UL, 
			0x0cfcccaf60a45085UL, 0xe526138b84763398UL, 0xac7a4ed7372dadaaUL, 0x50aebd2edb08b97bUL, 
			0x35fbafd92fd27d78UL, 0x1d2adb493ac670efUL, 0xd37b9e1efdcd9e9bUL, 0x0a9ed1c8c0d81082UL, 
			0xf83e71a4bd33ade4UL, 0x6cc8bea248f92d2aUL, 0xf90bdf15bed109c9UL, 0x21375635f75b16a2UL, 
			0x9dc790a428405db7UL, 0xf5e4619ca0588dacUL, 0x0be34c519dc9c791UL, 0xd505ce3bb00ce015UL, 
			0xa762dd023969e1b0UL, 0x4bf758838dd7fecbUL, 0x3ba5595ec0983e03UL, 0x38e57418030988c6UL, 
			0x5af961ff15999a6dUL, 0x5bb817278fc1591dUL, 0x32c0844be0ac1c07UL, 0xe89ee113b0abd461UL, 
			0x77cc466cb9578a68UL, 0xbeca081857148cdcUL, 0xa39fb422f4bcaf00UL, 0x7262ad3326134c2cUL, 
			0xa3ad019376c0dff5UL, 0x5daf6e919ce29bb1UL, 0x56db983c6889c3d7UL, 0x8ee5d79d50743d5dUL, 
			0xe075253d6d0ff40aUL, 0x833a9126505198c3UL, 0x5ed93e0997403d9cUL, 0xf98cdeeb7291d5ecUL, 
			0x14be23ac85ff7662UL, 0xcfc42e6f21fb51caUL, 0x8dafdd2fa3148eedUL, 0x8614d927815c7f94UL, 
			0xa50fd546b3e729aaUL, 0x5a0816b4b144f0dfUL, 0xa7fb4b4474caf828UL, 0xa73ecdcf2799c297UL, 
			0xd75b8a8678b83130UL, 0xc7e81e667ef1d760UL, 0xa937d2ebf9553c00UL, 0xd690048c94af34b5UL, 
			0x910d77767d475af4UL, 0x16dcc38bf6c20dccUL, 0x5e08b4bdb53eb36dUL, 0x653b95f096f33bffUL, 
			0x9268e34c87074e4cUL, 0x79c94b50a8160ec4UL, 0x2395ae963c212bb0UL, 0x70fc05c4c1665b24UL, 
			0xaa3e5aed53f8e988UL, 0xc57eb143c4a36b29UL, 0xc6228397e8421c37UL, 0x0e16a47861afe490UL, 
			0xdac29922c5f51f6aUL, 0xa6e857b9058b9d3fUL, 0x180727603b8f4205UL, 0x51d82eafa32013a4UL, 
			0x8e4e02984cb16064UL, 0xe1759ac38b6397faUL, 0xd387fe83e3fb5f6dUL, 0x333647c6dd627815UL, 
			0xbaa17f362e12b693UL, 0xb7358ae3ad795953UL, 0x44db68392ce101b4UL, 0xb68b886d4fd0e888UL, 
			0xe364d4ea5d8ffa06UL, 0xc1a05459855ba8c6UL, 0xe10fb0b5e1593783UL, 0x5136820f86f0ceecUL, 
			0x7ac24ccd46211ca2UL, 0x3a47ca13b8113fe4UL, 0x9ad8954dd0a19b93UL, 0xbbd81f341dcaca81UL, 
			0xc14012ccd915f2b4UL, 0x56a9fe2fa9be3778UL, 0xb11f3609f4390cd7UL, 0x0689fbcc4619bfa2UL, 
			0xa2aaab28f23da2b6UL, 0xbf1a27b0bf66c3d3UL, 0x379e247eb22861d1UL, 0xaf8942abfda473eaUL, 
			0xd246163022ec0867UL, 0xd8861a6860c8c07bUL, 0x2c9df4fa2355139bUL, 0x96446ff25c036b3dUL, 
			0x3087774e14ad8820UL, 0xec1c511dd7e0495aUL, 0x8338dda9b41af2f2UL, 0x2f260e714c25852cUL, 
			0x3cc21bcf9a24faeeUL, 0xf1f9518f3bd54b53UL, 0xa291dbaea8a8273fUL, 0x0267c2f1a40aaf67UL, 
			0xba20a4f33c50715aUL, 0xb42a1149d9ba9f0dUL, 0x3276a4a3c558e967UL, 0xefdc61a388eeb99cUL, 
			0xf51385eecf5b9639UL, 0xd7fc43a9de2af9b9UL, 0xf70319210c9e9fe4UL, 0x9e04e00c526c7a48UL, 
			0x0bceb54df534a153UL, 0xebd27eebcdb31499UL, 0xdf9734a7a165602aUL, 0x11d38b71691c1082UL, 
			0x208c3b740805f208UL, 0x40b33a04243212c5UL, 0xd64376cdd475256dUL, 0xd39dc13d03c5c73bUL, 
			0x2c4a7340a0ffd9efUL, 0x7b467778c3cd41e0UL, 0x1170327114ebafa6UL, 0xe0c6eca3f1f152a0UL, 
			0xe1eed4f4ea9fff7eUL, 0xc7b16e41c4298d4fUL, 0xa8dcbc0df444a60bUL, 0x30162e2851737a32UL, 
			0x80304959137fc61eUL, 0xf8f30aea32042e7bUL, 0x0f7b4b50dec30d14UL, 0xcab8cd63a8c91c9cUL, 
			0xfd1bf7a802a24a84UL, 0xc02f15fe69acc8c4UL, 0x4ab2e3fbd7765d86UL, 0x22afefdf86ac97feUL, 
			0x12091e56778e3feaUL, 0x5c354b0c26478d12UL, 0xd42a6787d85b6667UL, 0x76cce66faae7f816UL, 
			0xa3526cb10f44c4f8UL, 0xb39bdccc7b169a4aUL, 0x22fcc4973becb79aUL, 0x254072c46bc51375UL, 
			0xad119f7d678656aeUL, 0x343342ab1df002fdUL, 0xdf5d15bfcf42d38eUL, 0xe4dc72c71541f63dUL, 
			0x8fbee4e6623df3a9UL, 0x76336356ab80e7e6UL, 0xe4004932156cd9fbUL, 0x28f2cd74902f1921UL, 
			0x32a399f775688b37UL, 0x9a1cba9e5645b3c5UL, 0xf90896929c3944baUL, 0xb9fcbcff98ed4156UL, 
			0x426c9fcfe3800318UL, 0x6827f032680ecd0eUL, 0xca5306f69ce2fe81UL, 0x5fe99bc8f3849badUL, 
			0x20421a0c7f7fa879UL, 0xe34a29f4b2182f1eUL, 0xd45f9839b1dbc211UL, 0xb156be71529ebc81UL, 
			0xb6010fdd3a4c45e9UL, 0x34a2011baa357edeUL, 0xa4c265cdde8d2841UL, 0x9cfdfca79ce41876UL, 
			0xf661670ce0d70377UL, 0x59778a0ed5289bc0UL, 0x5e812798cdad2d53UL, 0x15f5fe250a87cdccUL, 
			0xccba8a703b6c6676UL, 0x29fb1d889a8cf74aUL, 0x0c99fd65b69b41a4UL, 0xf2606024ae3e5742UL, 
			0x7b63c33d212c3398UL, 0xd14ae613b240f452UL, 0xf6eae282b299ea88UL, 0x61e39d31c64e482dUL, 
			0x4ff9a9de586a1ff7UL, 0x0c278d739e22f994UL, 0xb47dc385c21ae3d1UL, 0xea55b8ef7c6ac697UL, 
			0x343a5da07853b366UL, 0x36bb487a22ef14b9UL, 0x2cbde2ad035c2e85UL, 0xfe4c30e3c5f222d0UL, 
			0x521133d19799febdUL, 0x50101d039b403fa2UL, 0xa35fc2e476f3d4dcUL, 0xc5e82d42d4f992d7UL, 
			0x574c111b875ba111UL, 0xb26738a2879b666aUL, 0x913c392801eba00aUL, 0x98cd0d1ffb22b810UL, 
			0xa02ca435b6434f90UL, 0x5b3f46c73e31694fUL, 0xbb05debc9101fb5aUL, 0xfbee623c02cb2c4bUL, 
			0xe0dc655757657438UL, 0x31111af2a1015dcbUL, 0xd69ee72da3245304UL, 0x6a02b96f289056faUL, 
			0x7206fb6799a096a9UL, 0x7c76eb57435bbf01UL, 0xa2d1f7d20800d7c6UL, 0x67d342882a1a5740UL, 
			0xa72934b68282e32dUL, 0xf1927136f1fe1866UL, 0x05b909a0461d38b8UL, 0x6df72469969e96cdUL, 
			0x7bc73f1387ce2276UL, 0xbf5fd802d2992709UL, 0xd621395cd895e6a6UL, 0xd8cce54a6d7a3c4cUL, 
			0x30c31c5a73a37b81UL, 0x7ee9befa7987fac9UL, 0x95a69a6ecd33c5e7UL, 0xf1ca3ebb9491b32dUL, 
			0xfea3be816dfb73edUL, 0x0938c87cf3b8d467UL, 0xf5223806cf1f8c0fUL, 0x13969e62ddd64a8eUL, 
			0xc819f01ab35d1345UL, 0x07b1734296e2f571UL, 0xf1f1fb2d0da1e589UL, 0xf27fb7eab2deb492UL, 
			0xa20865f804e85be0UL, 0x30a818eabc982b5bUL, 0x986aec33c9ad21fcUL, 0x0bba055c431c4e9dUL, 
			0x58e9e4974d191190UL, 0xc0cc91ebd5e6467fUL, 0x510b48161f9cd213UL, 0x927952c9e7883099UL, 
			0x1ecf75e707dc8eb7UL, 0x0cff3d2dc4c1fc49UL, 0xf6e529ea1d1a2c72UL, 0x0f67fe59daf4f015UL, 
			0x4c2aca8fd09d5743UL, 0x0b0d5bbf67eddab3UL, 0x515b081326dfe9fcUL, 0x49da30a1758711f0UL, 
			0x5f98d05ea2fe5f98UL, 0x0ee6d78eb172175cUL, 0x4721a9b669b50961UL, 0x8fe061786974a7beUL, 
			0xcce44fa37a9c9c3dUL, 0x71592dbf01c856cfUL, 0xbcc8c576dbb9f73dUL, 0xe36e5cba0f77940bUL, 
			0x01c9a7ea4a7d0b9eUL, 0xfb44311372fd4edfUL, 0x6723202a7d3782fbUL, 0xa0965e0238861dfeUL, 
			0x993e557ad32e63e5UL, 0x735c8711c21e4d90UL, 0x65e65b1bda5fe2b2UL, 0x48e282c6f2feff29UL, 
			0x615ac3519f4a9270UL, 0xb3ce8bd5a02774c3UL, 0x68d91f923253f553UL, 0xda93bb3b437e44f3UL, 
			0xb0a35750312e550fUL, 0xe673cb8b881dcf90UL, 0xc2df929cf98670e0UL, 0x672c6cb158e56258UL, 
			0xe77db62eafd0c69cUL, 0x96fa4342c00f07deUL, 0xed998c00a0dbb086UL, 0x854d8999e2520673UL, 
			0xf75f213a5b06f86cUL, 0xeb7fa87fa48a75ceUL, 0x16003df286469d6fUL, 0x1f56ac2594cb511bUL, 
			0x3701db9d3c7dc7dfUL, 0x9e56b4d9fa5a6902UL, 0x1fd8c571f9e45af7UL, 0xb48ea6bb7bbc1855UL, 
			0x83ee46dcfd8de980UL, 0x2fc215ec08bb4cb2UL, 0xc603871894f9b30fUL, 0x069185eefd5aeca2UL, 
			0x0d10db5621ce9352UL, 0x6b9f1d9c41dac113UL, 0x4cf9c8634fa79b34UL, 0xfffe978fcbe940ccUL, 
			0xce9c8e4778182185UL, 0x9e093ee69902f73fUL, 0x629b7b437cbcb261UL, 0x20e98f12eaa30fa3UL, 
			0xc11f61a8efbe696dUL, 0xb175741337c1e70aUL, 0xf0c50d7edf3a837dUL, 0x990a3ae96e142388UL, 
			0x9b2180f1ddd22913UL, 0x7afe0be394b875d6UL, 0x09129e3539f533e1UL, 0x92601e7895c35228UL, 
			0xa971944f05f9676bUL, 0x5e3a1dc1af7165a3UL, 0x0ef7a5b000c5d982UL, 0xccf8de9ac9f7cd7fUL, 
			0x3a925c1d8585b9c4UL, 0x6f91c87fca916247UL, 0xf60cc35948911bf7UL, 0xc6a52b1cd1097886UL, 
			0xfc0766ec28cdd7f9UL, 0xf3546fc1c1d7c3bfUL, 0x37abe8aaaed09d38UL, 0xcc39abf679f0d604UL, 
			0x1aa99a0ff54aa304UL, 0x3143b51014ddec4aUL, 0xfcc4f2efebd30eb8UL, 0xea0805bf3ccf62c5UL, 
			0x481904b4a5d2ad93UL, 0x5d3831a251ea5b99UL, 0xd432baec56c9dad8UL, 0x595466f85dde1718UL, 
			0xe54c3ef3879520b7UL, 0x1a34b979e722aee0UL, 0x3ad47a20be93be9dUL, 0x13eb8b520f94c0f5UL, 
			0x3cff473d44944e63UL, 0xd001bf7ca85ad698UL, 0x505e80ec7fcdf82dUL, 0xdb6e502a526db116UL, 
			0xdbb826396903761eUL, 0xe8280718e8fdd845UL, 0x61a7fea97f4b7e37UL, 0x76c93273170b8103UL, 
			0x033e094d99730b11UL, 0xd13da831a3ced319UL, 0x2893a1eb1be998caUL, 0x58261822bd5f9e61UL, 
			0xc9e5af6957ba518dUL, 0x1ffc3a80147c67c9UL, 0x885d5e30bff0de8fUL, 0x7774d36a4e9fadc2UL, 
			0xc96081f297f9fbbfUL, 0x1041127c16251cfbUL, 0xc3f4d3448761e58dUL, 0x76138c339ea24e8cUL, 
			0xc4df5ca55483f705UL, 0x37187365c6c6ffd6UL, 0x2d2645aed42de4f4UL, 0x70771a1c195438eaUL, 
			0x17739ccad391d262UL, 0x7ff09a9c06878239UL, 0xcebaa4070aad68d8UL, 0x37f41fe882342affUL, 
			0x066bb2fb5e21fa35UL, 0x00efc4fe641e4672UL, 0x188cc8ae09eca7dcUL, 0x715c3e7a201e34e3UL, 
			0x62a543b8214b0d12UL, 0xaeddc97afe3ee519UL, 0x232986ff87edda72UL, 0x6215f2a2e4710b98UL, 
			0x10cc9dab6b9aecd8UL, 0x46d4a027198335dbUL, 0x5940a10053027e72UL, 0x08b1323eddc8d3f9UL, 
			0x010a4a818868ea55UL, 0xf12a3fd015520e45UL, 0xbf2214e979b9fff2UL, 0xd93b5651c8b2080cUL, 
			0xf6045f2e3bab82c0UL, 0xb67d202016f2c1a5UL, 0x5dc9d568a23e9aeaUL, 0xe20630a1bd8a4f5aUL, 
			0xd8323fbe328b3b08UL, 0x66648446b6abbe14UL, 0x5ca93dd3ba85e477UL, 0x2851e5153f3e0313UL, 
			0x519bc6f30d4f4c7aUL, 0xe9e4f5375de9b4daUL, 0x8f09d13264ad900aUL, 0x7376645bc9162a93UL, 
			0xb1eea554645c9206UL, 0x051f562d508b8229UL, 0xa0213e9bc4fda425UL, 0xe8718ee74d2fdc36UL, 
			0x328a54559f42640dUL, 0x15aab31d2bcf5013UL, 0x6ba3fcebfd79f3c8UL, 0x6983184c58993145UL, 
			0x3756187c01cab36fUL, 0x2f061dd87e621d0fUL, 0x192df4e42d0daea8UL, 0xc33f900646e4231bUL, 
			0x0bae9bb3ae396249UL, 0x789ac5396375141fUL, 0xe9cd008e05218048UL, 0xff843b0ff7fd5f02UL, 
			0x8aa391cdb9d1652eUL, 0x5cd924acfa8d5e3cUL, 0x395a15e5605187ecUL, 0xfc574671f8544c84UL, 
			0x4710ecc39c11c237UL, 0x292a1541f0f44f1cUL, 0xfb7efa37bce7cf12UL, 0x51149eb314285956UL, 
			0x444fdae060b5651cUL, 0x2961d30fa2ed1b6cUL, 0x28bb7fe3d95a9eabUL, 0x3b96647332c54534UL, 
			0xaa46f1458222f321UL, 0x78d28ffba4c6499fUL, 0x63866740a2eaf8bfUL, 0x2f62c4705fe7b58dUL, 
			0xb9ffb67de052c337UL, 0x5e9f7fd9b44b1135UL, 0xaa4c0e1e5fec0a77UL, 0x3cdf15706abeae74UL, 
			0x3f848840f18730a0UL, 0x29e6f6ec2f43b841UL, 0xbb74922c9a235306UL, 0xbf983f4bb6f9a80bUL, 
			0xf694316e0f615e2bUL, 0x2641c421ee96fd45UL, 0x833317004c9b3becUL, 0xb3561a3891e2df9fUL, 
			0x1aaad371eff9bc19UL, 0x767805a6babdf772UL, 0x32ca5cca13636738UL, 0x6cc1da766c8f1350UL, 
			0x674266adc88eba73UL, 0xcf81db444578904bUL, 0x625b5cf4f5979174UL, 0xa86f2ecdfc3015d5UL, 
			0x4951104d3181bb60UL, 0xebd56ffa0a6cf1ddUL, 0xff14ebe4eaf8a6d0UL, 0x25b28eee34314fa4UL, 
			0x3651bc1187504dfcUL, 0x2ee80f2dd11dc3fdUL, 0xcea62862e13e1d6eUL, 0x5b335e53c7fc289aUL, 
			0x46a7d9f68f8c72b9UL, 0x8d15f4e4254e819fUL, 0xdddb46d576946646UL, 0x398c625aebaec5ccUL, 
			0xb367d95003f35629UL, 0x5086accbaf588bf8UL, 0x8abe41f6806bd82fUL, 0x2211f3b4bed97335UL, 
			0x26f85c38a2da9ff7UL, 0x2f2977002e706021UL, 0xe2140fa1284fe456UL, 0x6594e3c821c40616UL, 
			0xcd8aee4e820dddfcUL, 0xa0db21c56a9574f5UL, 0x1b06b222374d938fUL, 0x62c547dc6fe8c42dUL, 
			0x51dd9c755ec9c54cUL, 0x9d9929122dc93a29UL, 0x3836967efb74703dUL, 0x63f56071a9cf429aUL, 
			0x65ab67984964e111UL, 0x408d3e4c57687ac2UL, 0x85de8ba2da0be9a4UL, 0x47d0fc38921801b5UL, 
			0x4441dc69316e2b8aUL, 0xf7b3764e9ca49269UL, 0xe88dd76ef5465b69UL, 0x2f9553946fd3c54aUL, 
			0x927247ce81ed50b4UL, 0x01715ba6a8716579UL, 0x3ee9544ef123e139UL, 0x221294691247bbacUL, 
			0x59da5d70a92d5ad0UL, 0xb99fb58b5daeb231UL, 0xa47760b4ea5ca2c4UL, 0x74deb4795382185cUL, 
			0x7333fd7f75b6d3d6UL, 0x74901dcb6ced683eUL, 0x92865c065f68297eUL, 0xb77eecbbac08c9baUL, 
			0xa885255317624b5bUL, 0xeca85d015cda3762UL, 0xa073370e1afc8b80UL, 0x885a8c4d595a4b6aUL, 
			0xe7404d45b75662bfUL, 0xdf4f4851da8ffaa3UL, 0xc56967403be3a3efUL, 0x752f4beb50d15a32UL, 
			0xabf6b5aaa97b4bc5UL, 0xf5e3edc15b000786UL, 0xa6224fe86401fc55UL, 0x4b1ce225eb369f83UL, 
			0x33c7a63c27c90079UL, 0x648003171ed2f9e7UL, 0x9cce4374ba77a219UL, 0x40d5af040c4c4baeUL, 
			0x2d13f3aba1214dd8UL, 0xe1562b8cc8a3f3a9UL, 0x8343d04e733a6bc3UL, 0xf2aae7ba85a5b4c3UL, 
			0xeb22c661dd84c690UL, 0x0c3f6e7db3459ca7UL, 0xfad54fb0f0253122UL, 0x55c0237fb62ec114UL, 
			0x19e1b8ad7e1dc27fUL, 0x390b8aa1393cab1fUL, 0x09da8c1eb971381eUL, 0x357e2db7a96c7d21UL, 
			0x899d640ef80ab720UL, 0x8b3111e840f3fb11UL, 0x94ed06da226a5bafUL, 0xca559e6526e7e0d1UL, 
			0xa44d6c2c716303cfUL, 0xde1778fd665a1cadUL, 0xb1a70eaae14da92cUL, 0xfd146621f15b1e32UL, 
			0x26b1883de072a8edUL, 0x81e4844694c570a1UL, 0x33d817ee21263dd9UL, 0xa1420ea4bee55815UL, 
			0x5ec680367c2c1a28UL, 0x85268e6666636999UL, 0x0f4d608b6764aeeeUL, 0x429b838c2e6d86c7UL, 
			0x072d8e16ee6c0aeaUL, 0x0f7ff4ea7c14b063UL, 0x445d1b2553b8bdc0UL, 0xa2551665e673ab87UL, 
			0x1e9e77454dd4d77fUL, 0x4768348a5c6101d7UL, 0x157dc5e26e7bdac5UL, 0x86f1698027583a23UL, 
			0x01ef56c47abf83d2UL, 0x803edf8c5e70398fUL, 0x8b047ba2242a4a03UL, 0xd075ffef045e22bfUL, 
			0x87ff4e6c8b1dd0d8UL, 0x70c974bda78b2b27UL, 0x7531068a245e77a5UL, 0x6c7b8b9d679b3e61UL, 
			0xd30e157663addcf4UL, 0x75de11fae8c6ae9dUL, 0x1bb66c9c7410773aUL, 0x1c9c34f02f775d17UL, 
			0x02910dda786d8454UL, 0x1953ba87596351c6UL, 0x459ee618ed56c461UL, 0xc385fb6be3d9ed02UL, 
			0xcd834848a772dc6bUL, 0xc4d23552c3050b5dUL, 0x18ed80132e35d3bcUL, 0x4f7cc899afa2e410UL, 
			0xd463969833e4e5aaUL, 0xa570625ff9731a13UL, 0x3a4fa4810b59e360UL, 0xd6f763779be1460aUL, 
			0xa661f0df70c75ff3UL, 0x8f53d523b88c5876UL, 0x4eb854e6afc6aa22UL, 0x018c56217a1250c5UL, 
			0xcf04349e75db5ca0UL, 0xeb0cdfa97ee3d48cUL, 0x45aa7aa14458377eUL, 0x7f6152fde8163dc7UL, 
			0x344ce7079f27f165UL, 0xc5913223e9af5290UL, 0xfc801755368cb501UL, 0xd198d6fa369a952aUL, 
			0xc6a3b61bae76d133UL, 0x45c0e4bc4b77c140UL, 0x16fa51ed0455d6bbUL, 0x17a9a7d1f0155395UL, 
			0x87f4cbcb362ae9f9UL, 0x55c5785f00546d1cUL, 0xa1757264ececec47UL, 0xa4ca68581f2cd10fUL, 
			0x99789f8504901103UL, 0xc833672f20af47eaUL, 0x71cd60960a6b0adaUL, 0xc36097a4b78b9c66UL, 
			0x24d01feff4b61b46UL, 0x73d5ebc010db56bbUL, 0xbd347a0044f5974eUL, 0xe665fdf1edf50e78UL, 
			0x59472c0313a76dfdUL, 0x03e26f90a2320e6cUL, 0x45c3dfd7dfd12771UL, 0x0989e08785b000a4UL, 
			0x10d1180b4479dd2fUL, 0x85b7ed01b278cab1UL, 0xae97ff8edecb6c17UL, 0x4b5b5c755db0d4bcUL, 
			0xbb1ccd2cc9ab7c74UL, 0x4f0ce3d57f299e75UL, 0x0cbe7011e9d2273cUL, 0xda96db3c3e7c889fUL, 
			0x6b1ecb87307bd54cUL, 0xb7bd7ea0bb4bd56dUL, 0xe2f8f8327f60f549UL, 0xb1f69894cb4bd90aUL, 
			0x21736431498904d5UL, 0xd647b4a31d86ad75UL, 0xaf79c711c8c58cabUL, 0xb990c6a3024ec951UL, 
			0x1113c4511675f936UL, 0xa98ce9f53a6b2cb1UL, 0x5cbe269510b582cdUL, 0x20c447313aae2fecUL, 
			0x055821f71378eddfUL, 0xac1424164b599373UL, 0x0f0a1a3d8328aa31UL, 0xf5bfc04bb323572fUL, 
			0x72af1706e21926f7UL, 0x5d53aac0e7e4b7e8UL, 0xd1d2b8d1da08a7a3UL, 0x41e5172e7bd34bf7UL, 
			0x6d63c2cc8177637bUL, 0x3ad81c09e94316c0UL, 0x07876d952fd4c891UL, 0xf6eace361cc66534UL, 
			0x3e590088b8c0c8eeUL, 0x4ba4375e0dd4a5b5UL, 0xe15bc7ebd824e69bUL, 0xbf0ef0cb8edf441bUL, 
			0xa0d26fa1ad34f42fUL, 0x9ab5d4eb9189dcdbUL, 0x4c603f093691894dUL, 0x92f2ef7387140b7eUL, 
			0xa60e6cbe7b23f9d0UL, 0x121c7168be7c3a40UL, 0xfd4974957b5cb0fbUL, 0x0f4d3582175b53b6UL, 
			0xb6309dabf32edeccUL, 0x84b64a66f7dd2fa0UL, 0xeb4a2d95cd081562UL, 0x891663e4eb2efe1fUL, 
			0xbefd50de4260253bUL, 0x590a0deb1ecf0abaUL, 0x6ac70b0b8e95bbbcUL, 0x75f035eb2ac34d09UL, 
			0x516094dc980e5f31UL, 0x9f4e57e5ec14d284UL, 0xb9baea73158cb2d5UL, 0x0766e0f7ad9c648bUL, 
			0xf024f350fe848ac5UL, 0x2f92f6943a868699UL, 0x5ccfc4833facb23fUL, 0xa0ae6eea4536ecfaUL, 
			0x7e7c0d217c777642UL, 0x6d9d72c98f1603a5UL, 0x5f0ae97a15a149ecUL, 0x4038c137b2603d66UL, 
			0x9b7fc524ab71bddfUL, 0x1623b367076dec15UL, 0x38f9d1dd2d8b4f97UL, 0x259021ff6ea9f33eUL, 
			0xe2a7cb5cc85278c5UL, 0x2fb582ad3771bccaUL, 0xe090521101fe5043UL, 0xa157083ea2069d17UL, 
			0x9bd7dc81b9d28d25UL, 0x37ec33562ded70c5UL, 0x8b8c511f74ac948eUL, 0xd5c1fe9448c70a52UL, 
			0x6737e4bb5b76b3fcUL, 0xc023ba5f77acd9d6UL, 0xabed845a6d2eb0c1UL, 0xc9ba6a880054795dUL, 
			0x7c065721f562420dUL, 0x10f117b162838a08UL, 0x5b59d9a89ac497c4UL, 0x67960abb26f8bb6aUL, 
			0x35523a99547e409aUL, 0x75fc43b018cb572fUL, 0x61927955dc5b975dUL, 0x338c9dcfcf969022UL, 
			0xcf4f0caa8e5a5567UL, 0x11b90fc8543dd8a1UL, 0xfc77e260e6764e34UL, 0x141c1a741f5975dfUL, 
			0x922269c18403399aUL, 0xdb266b1fcbc9f4a5UL, 0x51a4525a6220587fUL, 0x1a0ef4348d18f6fcUL, 
			0x5f5aff607265641eUL, 0x74941d3e1f9ec98dUL, 0xe52cf87ba5dcc32eUL, 0x5e8e3580cf8b494aUL, 
			0x197a661755b21a98UL, 0x2933e04e818a5abaUL, 0x03618d07408d8037UL, 0x888a0363f7f678d8UL, 
			0x1de98c5107d51ec0UL, 0xa6f271f8dc3feb55UL, 0x1dcf0ff1ac9c1a1cUL, 0x0fc2a5c902b8a9a5UL, 
			0x4c61a632155b1bdeUL, 0x05793c783495c0d8UL, 0xb34f12b761a6b584UL, 0x9c6a26206cf2e1ceUL, 
			0xb505f77b423bf774UL, 0xbcb151c541ca8fdaUL, 0x9c9de85caac515a6UL, 0x06cf65e75b4b7fdfUL, 
			0x84c236e3f9697423UL, 0xc8aa127ab1d62787UL, 0x418faa011b899cf4UL, 0x5066821efa70fc1eUL, 
			0x40b598df2c55569eUL, 0x4b9c411688fd38ceUL, 0xe32895665103ed99UL, 0xe2e51b6d2d7c4f27UL, 
			0xd492e4187a0a04f6UL, 0x8fa31388f50e42caUL, 0x1e1deb294b243788UL, 0xcd9dc9f009c86a9aUL, 
			0xa8a44e15438a39dfUL, 0xd323ce97dfb10040UL, 0x0f2dc569399b2b86UL, 0x0c04c25ad378cd07UL, 
			0x628e20d6e774dfb6UL, 0x86907e8f7e502ab6UL, 0x32378ff3b0af4db3UL, 0x340b946af9c1a01fUL, 
			0x48abb0f9f07361b7UL, 0x08a591341e76459dUL, 0x10379af4c4610bb3UL, 0x06d2c77c5ac3e9c6UL, 
			0x4bf01f0361a42ecbUL, 0x14aee8753e8221cbUL, 0x842636096452c704UL, 0x4640ecadbd0841ddUL, 
			0xb87d2f455ae4a15dUL, 0x9c39afc400288104UL, 0x9954f9aff951b7a8UL, 0x4de6c62ca53afc6eUL, 
			0xf5c1f06c33abdd58UL, 0xed67cb7cc7b2b678UL, 0x2fe776fc8468f228UL, 0x5776bda9383c2255UL, 
			0x687ee749551a08fdUL, 0x945a95cbb7bd0d8fUL, 0x9edaee011d605532UL, 0x12e7f95a234bdb26UL, 
			0xabe28ecd345dcd18UL, 0x0a99451aac7094e4UL, 0x3bc18f8ba5a0f7b8UL, 0x661179109bf396e8UL, 
			0xe214e6bf1a81ce7eUL, 0x2a8e7a8446e307deUL, 0x0949f090c19da3adUL, 0x61fe37ffbdeff223UL, 
			0x7420fe470c52fcc1UL, 0x7d48cc87f98d5d03UL, 0x2de710629d6395faUL, 0x559cc8616f0cd757UL, 
			0x8122687516463247UL, 0x7699c34aa582411eUL, 0x1fdd748f98c7a4dcUL, 0x45a85c4df668a041UL, 
			0xcd08b598c983a3fdUL, 0x942049b3b58f4313UL, 0x7138d0f22c968ba8UL, 0x696a264f9d71c397UL, 
			0x4c7032c8f07ecf0fUL, 0x05dc3ee3be1b9bcbUL, 0x5dfd4c39847a8999UL, 0x6de31613530f0359UL, 
			0x69914df0a21db600UL, 0x2ebb133162705cd5UL, 0xcab62a6a5e55baacUL, 0x9c4c7f9ed75da310UL, 
			0xc51d2ff5fed79167UL, 0x2acfbaae9ffb4062UL, 0xdd33a5050738a3b1UL, 0x9b8c8a33bba6794aUL, 
			0x4210a364e81afe3bUL, 0x5b92c0a3630f3c6cUL, 0x3fc6d51841315a49UL, 0xf5b39bec8d6828a2UL, 
			0x0fd66fbf21f9d3c4UL, 0xefd39e584376a9dfUL, 0x2244fda97a9a148bUL, 0xa7a8c8847ad2e467UL, 
			0x845329478b11cb96UL, 0x8702cddb52fef820UL, 0xd8a5c17fd04a5ab6UL, 0x72fe63fd2736cfdcUL, 
			0x820ebf4fae3b46dcUL, 0x5195a7ed58dc7653UL, 0x32032a24d25dac35UL, 0x807a4bee71a09b63UL, 
			0xf119029bcc29f4cbUL, 0x31450be1497e7badUL, 0x4e0e1d5889b5f9d9UL, 0xfd7fc4500bc4c835UL, 
			0x152b00b4987365ddUL, 0xfeb53420b524bffbUL, 0x3ecd275a94150051UL, 0x75734745cc79da10UL, 
			0xb5adc4209360084fUL, 0xbe05e988c995ea3eUL, 0x2b8c79e6a66f092eUL, 0x53e2a81ccff23a80UL, 
			0xde3a4daaf23c0fcbUL, 0xe7c76d401fbb8e79UL, 0x7d8f45471fe14a41UL, 0x51ec14faaa7e921fUL, 
			0x2d3b4d6f68c713c3UL, 0x0a6b12760951c66eUL, 0x9d1fa87abf7f9769UL, 0x75fb73e18a308af9UL, 
			0x81f1288852d90b3cUL, 0xaeb00cd0d12b701dUL, 0xe4b2c990a99bba1aUL, 0x0c89e64d9d00fe6fUL, 
			0xd6cf4ffb72296b8fUL, 0x9491695b0b98bd7aUL, 0x3ad67ab9063ff4d2UL, 0x57e8c165499939f9UL, 
			0x23b460e8a95b3712UL, 0x707735dd9630987fUL, 0x47dc8e156075f1afUL, 0x6181a07831d2eaebUL, 
			0x70f9dd0b5a5d9dbbUL, 0xf7f54c9a01014debUL, 0xb3755368bf3bf361UL, 0xe8b3cb5a83d3ee01UL, 
			0x6e6ce85e73ecaa45UL, 0xab2133e9eeebc43fUL, 0xae3d8e93f533f578UL, 0x38fd281ef9caed0dUL, 
			0xf75d6579b3fe50caUL, 0x4616893d44323db5UL, 0xa222bddc464b488dUL, 0xd4dceaf9f7438750UL, 
			0x51c67071138ebbbfUL, 0xbea0a3886103d58dUL, 0xc39d4c573d77985fUL, 0xa8c7e6b2a5744998UL, 
			0x7191eca804d3cd1bUL, 0xbe60e694a1cd9b6eUL, 0x558bafc2d9382c4cUL, 0xd897f27ba5b6ab7fUL, 
			0x5fb64d11b3a9d660UL, 0x9bc998efe2ac525eUL, 0x1e0abe28ce655365UL, 0xd606eafafb8285d4UL, 
			0xac723e63e2335114UL, 0xdeab6ec8838a7c9fUL, 0x319a227fb15237acUL, 0x660413635c38b366UL, 
			0x6a75dd44e3bc66d1UL, 0x14d58d50f226c577UL, 0xcf8d4e0609609cf4UL, 0xaca38a522e63fd61UL, 
			0xa850385a2c2c46b7UL, 0x6bf4292f043d8813UL, 0xc9615d7d89213751UL, 0xf26bc94bbeea2cf5UL, 
			0xf382bfd743d16ad4UL, 0xabb77b92e1b953d8UL, 0xa3444df96411d232UL, 0x50e3304c178eb913UL, 
			0x27b01882d02f2b95UL, 0xc3cf0b4239ff389aUL, 0x9500ea96daef0110UL, 0x79188c2b3b5981c0UL, 
			0x9e1f1654d0b21f3cUL, 0x817a5c9726efebb6UL, 0x1fb78bb54effb5baUL, 0x8a19b17b2dccf3e5UL, 
			0x7adeefbd39a39703UL, 0x4cd099942e38d52eUL, 0x86f7d655c291b65cUL, 0x37e653fec1cf5fd0UL, 
			0xa0eb9719066fbb3fUL, 0x850070fec6142886UL, 0x88cc0d184411f125UL, 0x921256580764f579UL, 
			0x14181912097d8551UL, 0x9dc69e5990b94f04UL, 0x18419398fce03612UL, 0x85f2f46231069a44UL, 
			0xe810632e67ca9e05UL, 0xa284a4481719d3ffUL, 0x1c6ea9febee9b0f4UL, 0xc275c9fe4ff429b7UL, 
			0x06c990981454f30fUL, 0x5f59abeca8391e93UL, 0xffca06a83095bfd8UL, 0x34775990bc96f27bUL, 
			0xfb19664ef6bffa27UL, 0x4b9b4366e7a3b52aUL, 0x60fb64d31542423cUL, 0x97fde161f9605523UL, 
			0x87ead49221633ec5UL, 0x8d1d5cf9bd30f7deUL, 0xa9c34da438aae648UL, 0xefb37f76aab70b49UL, 
			0x79aec9f474dce2f4UL, 0x3cb7455a5e3ca155UL, 0xece006f4b0842557UL, 0x479842fd38b783e2UL, 
			0xff1664d2809db6b8UL, 0x97af9998839a9262UL, 0xf08a8625de66f932UL, 0x6165db7875db30ddUL, 
			0xb5df7d96b3eae54eUL, 0x0f0490e642538292UL, 0xd526d92c1a4abc89UL, 0xac38d71bcda350a1UL, 
			0xe311a29bb80f7580UL, 0xb5cfef86f2761210UL, 0x416fc8764584f622UL, 0xf4e58d9cd2c6d956UL, 
			0x936104fc92abc25bUL, 0xd91e01fd82f03b3bUL, 0x2c2fa7e481d135bdUL, 0xf4af6fb98249d3d2UL, 
			0x7eeced24dcb94a7fUL, 0xd83af05b08df934cUL, 0x13216557eb565885UL, 0x9b298775068b1b59UL, 
			0x1f551ae20518831dUL, 0xee972e82069b807cUL, 0x476c19d828ef5576UL, 0xba7f9c4d92060685UL, 
			0xd254b3815d7511a6UL, 0x337f4bba71b5f3ebUL, 0xfef296840daebf61UL, 0x59a7dee097335cf2UL, 
			0x62f8bd944fdba8f0UL, 0x7c0b22224030ba0bUL, 0x874a21bc1bebed82UL, 0x90aa6673d2e3e43fUL, 
			0x8719251fe98bf232UL, 0x1a2722bcab7bdc2fUL, 0xd5c6fb2f7c4c00ceUL, 0xaef63f820fb8787aUL, 
			0xfd61983281cf2394UL, 0xb5a46d5420d1aaf2UL, 0x163ab07d5f9931fdUL, 0xc7b0820b64001e6eUL, 
			0x000540e3d31897beUL, 0x2b140ec0d36b3854UL, 0x5fa40f2241943d84UL, 0x743a6b927ecb2a80UL, 
			0x27d1b1510a566e2bUL, 0xb7ff434cdd97acc5UL, 0xf8b672f871d4d68aUL, 0x68666f3d627f2a3fUL, 
			0x6cb9974ee7363cccUL, 0x7103f8bb495850d6UL, 0x12eae3c85533fb35UL, 0x4e7219fb53d9f955UL, 
			0x0775276ec919666aUL, 0xd269c865ef4600fcUL, 0x6fed03bc4b675f10UL, 0x40c469bec8a645aaUL, 
			0x0dd50a89b01859acUL, 0x75961c338d174fadUL, 0xae7adb0a77ea5c27UL, 0xdd4ff0d1872b0d31UL, 
			0x04ffba24628e3221UL, 0xa48a49f05d9b743fUL, 0x61f97e56eb83f967UL, 0x828e7dc83f749208UL, 
			0x8e2140a7e905c400UL, 0x7346d168df4876e8UL, 0x26200e2d2f873139UL, 0x644b3ffe6ac1710cUL, 
			0x84c26a124fb07733UL, 0x690297576bf52c27UL, 0x61e9b44cd9eaf261UL, 0x68b94b24fa582327UL, 
			0x845197e31b25bc66UL, 0x92c98a081a977f85UL, 0x4a8b431efba6a453UL, 0x1ef264ade6889720UL, 
			0x02fb13bfd74cfb84UL, 0xd0bde9857f5a31dcUL, 0xdff0ab01cec057acUL, 0x0af73befec46a9c8UL, 
			0xf5b564ebff5a4cdfUL, 0x1fc494901c405d8eUL, 0x8db542cfeb9bc188UL, 0x069e5ebde8ed9931UL, 
			0x39fe63d116b42df9UL, 0xbc22491fbc8eadc1UL, 0xb7bafdbc465e2202UL, 0x26f94408e2b8dc83UL, 
			0xb016e39260410128UL, 0xd8e43bfb45f705a2UL, 0xa79387a963a43733UL, 0xc2497d9dcdf2fc94UL, 
			0x37745d516d405518UL, 0x84c1a5150f53ae4bUL, 0x64ce566dc808afb2UL, 0x20eb47edfe1f95a2UL, 
			0x110136255f168f32UL, 0x1ccfa28cef339d50UL, 0xef29e80193b09550UL, 0x422e313d0ebd853fUL, 
			0xc96f0923c326242fUL, 0x2d7c8547d48de2aaUL, 0x4594eda9b1049fa8UL, 0x7cb4d347e1e0b862UL, 
			0x923e307c2d685ca4UL, 0x13bae0f04567857aUL, 0x9d7d4c0c4f8ac34aUL, 0x68b2fdd5eef7f756UL, 
			0x2bce5079f94c2cf5UL, 0xe391fc79710ce4deUL, 0xaef330f303f76289UL, 0xacda1c458ab4fd19UL, 
			0xcbddb611fb79a46dUL, 0x17127580393b1523UL, 0xbe85d4f4dd886ddcUL, 0x5e1351bf699d368fUL, 
			0x7969bebcd3aca76aUL, 0x448edd9bd0dd2ab3UL, 0xee5a8acb443f628aUL, 0x83226e784a2ad58eUL, 
			0xcec7ec787a905a65UL, 0xb48e30ded22199e2UL, 0x680d04281615e4bcUL, 0x94e0c3c121c0049eUL, 
			0x1e4211c22da791ebUL, 0x1cfd81df9bb7b034UL, 0x2a2214cdcdb2616cUL, 0x98ce7f084523884eUL, 
			0x91f219ec58db562fUL, 0x90434efcec9de461UL, 0xafc7a2949152b1c5UL, 0x5ac50dc1335b6d71UL, 
			0xbfc0012827e61734UL, 0x6916a0c62627be52UL, 0xa742678c458726aaUL, 0xce780d86b9db2dc5UL, 
			0xa4289be42bbf515aUL, 0x6844b0db81ce6830UL, 0xe1ae4b15ead40a5bUL, 0xa1e883302bdc2e6eUL, 
			0x541a76b8351fdb6dUL, 0x40cbc8d710b62c25UL, 0x80f360e7bfba8732UL, 0xb10373984d0c174fUL, 
			0x7b6577153dd916a8UL, 0xf9b27753254739adUL, 0x876e8cd9fc0d30a9UL, 0x9c121215ef2f712aUL, 
			0x87fb9fd3b34adc3fUL, 0x3dfc9962f4ad0a74UL, 0x3ffeab8a80710f12UL, 0xa9bde25a31685d09UL, 
			0x4f33f48b0a451cd5UL, 0x8fcd55d48e8c5401UL, 0x2877d3ad65e89464UL, 0x19cdb20dccdc7315UL, 
			0x9531d338b74289a4UL, 0x8a88f142de94ab25UL, 0xee141d5dbb070ac5UL, 0x9e8a427e5dbcc7a6UL, 
			0xa3e24fac01a16e76UL, 0x3c29ea3cdac767f4UL, 0xa9f09e821f99e15aUL, 0xf1c6443e4e6a6669UL, 
			0xd25e44979525617eUL, 0x47f0ebca75a6243aUL, 0x632ca64fcc4cfac4UL, 0x4e16bea56739601cUL, 
			0x94bb41c1dc977177UL, 0x8cf1b42d279f3536UL, 0x7517434f1e4094a6UL, 0xa56911b4871b03b2UL, 
			0xc3bce4379017ffb3UL, 0xaca6297521943945UL, 0xb9944c5ade58d668UL, 0xe61c25a5520328e5UL, 
			0x654f590d020aebe0UL, 0xa064f5982642fb6eUL, 0x73d784ede7f8ca93UL, 0xcac2a568eba4052aUL, 
			0x5eaa89b0a7b90926UL, 0x90af96b417d450d7UL, 0x8f72c0723bacecd2UL, 0x0a11da826b478f0dUL, 
			0xce4ed52ba19b91eeUL, 0xabe2e6bfe79a8347UL, 0x2f6c626e9cbcbe28UL, 0x77da98dc6fafb9e7UL, 
			0x19030ddcd353dbaaUL, 0x7e58b3c2e207577dUL, 0xc57b287507d3aea5UL, 0x6163079c9b5382dfUL, 
			0x4f683d48fca0dec4UL, 0x047e36584f427807UL, 0x164fe428c8d4472cUL, 0xb0e80f26119d9c14UL, 
			0xbc220549b7d64ecaUL, 0x46b3bed462615a6fUL, 0xfa5a7ca02917fa86UL, 0x4858a63ee9b6f5b2UL, 
			0x0faa046884814293UL, 0x6963c8704e555f52UL, 0xac390a82bb68f77dUL, 0xbc693bbbd3d5cfc0UL, 
			0x61ef09d74f0082bbUL, 0xf0a120f7e9e7cd0bUL, 0xb201b09d75b246d6UL, 0xb6d51c1f12cdb867UL, 
			0x3662755ede3332b9UL, 0xbdb01f8cd7a94a9fUL, 0x1f9b4d7ec7a9dc72UL, 0x7e4ed7e29a0aa5f3UL, 
			0xc92e7f50f17b9a76UL, 0x97d724f90555afcfUL, 0xd693e60a8c047118UL, 0x87f6598e43e7a422UL, 
			0x15f7ec79b19dd145UL, 0x78500532d20cc7b6UL, 0xbaec4392eb5279d3UL, 0x41551e820be491e6UL, 
			0x52cf1e0da07fe52dUL, 0x518ba6b7b19c824cUL, 0x715fbb28810e8066UL, 0x5aacd30191e9ac4dUL, 
			0x8be6b99daa544b8aUL, 0x01cfdd05f1675f5eUL, 0x2a040832f10dbaa9UL, 0x92325a1fdd2757a4UL, 
			0x2cb6529f2a102717UL, 0xa51fcbbe283a0bb0UL, 0xc3a698f1faec97c1UL, 0xe831e49a51969d9eUL, 
			0x727978a0ee70387eUL, 0x1a31100374a1b424UL, 0x1490aabd6750df2cUL, 0x4289c58c3556e2e5UL, 
			0x95fa5d887b793750UL, 0xa3dfb9a150b0e36aUL, 0x5386ba592d1b7707UL, 0x3ff8d556913579a1UL, 
			0x363e9bd9aef2c163UL, 0x6b7ccc7c55a438d0UL, 0xb3aba1ca8dc7337eUL, 0x0087a02926d6b0e4UL, 
			0x9ac1fa661cfc93efUL, 0xa3bcd2432ea8a725UL, 0x9e68be49436141b7UL, 0xc356035644d6269fUL, 
			0x0af737acf9c3bb75UL, 0xcb437c34f985afc4UL, 0xc8f86ff378be5070UL, 0x0ce11a39fb1a7916UL, 
			0x4907acbedba932c3UL, 0xd98d42cd1f64d484UL, 0xdc318554542663f2UL, 0xae50d1a93ffe935eUL, 
			0x26d5351b8f6b742eUL, 0x77905b908e9ac080UL, 0x73c50e093d71c1edUL, 0x1853fe12d0b56486UL, 
			0x022a4fffbe79886cUL, 0xf5f9263109927733UL, 0x4aca25a6fbcd6934UL, 0x9f67db00b115f724UL, 
			0xec318feb4f966477UL, 0x77d4d7e9caf8214aUL, 0xcc0acf07404c9deaUL, 0x589e156011147825UL, 
			0x3afdfae81562d02dUL, 0x8e3fd202ba1fc860UL, 0xa6f694572db5d5baUL, 0xd81101d2d1b5715eUL, 
			0x2922a46cf4c6eddcUL, 0xa496ed98ab317b6cUL, 0x30ef63a9a9a58597UL, 0xc45b6bb441fa778bUL, 
			0x760becbf778ff708UL, 0x1755479ba6481076UL, 0x2b80ddf3db0f537dUL, 0x313c9a701e277ddbUL, 
			0x72aad7b13c96458cUL, 0xf8216c53642c763aUL, 0xa0efcd8505d84cd2UL, 0xe6c146231bcec8acUL, 
			0x183efeeb966b784dUL, 0x23c9851f5562c61dUL, 0x5a5e061d83e79faaUL, 0x43c9eb9c491bf77eUL, 
			0x96f8d0cfaf4e3de9UL, 0xa2e23e91999494c1UL, 0x8fcba7087c0612efUL, 0x5d1e393924bd4f5eUL, 
			0xcfb79ab6fd16b94cUL, 0x8fc90e7e610f6d9eUL, 0x7020c0f4bc02820cUL, 0x2e2c8ef3b4f39b7eUL, 
			0x044040f8e1de79a0UL, 0x74d00abe5d955cb6UL, 0x5dade813dc9c2b0bUL, 0xd4f9ea053bc4b6a8UL, 
			0xd792acf381e0daf2UL, 0x4929975c75a8ce23UL, 0x0f1616a889a90bfdUL, 0xf4ee7c695558fa11UL, 
			0x02d0f60ba87d55abUL, 0x13694885f6feeab8UL, 0x90829f3349daa0beUL, 0xab38817bdc10915eUL, 
			0x7bf5c33f124dda00UL, 0x9a37d09588037703UL, 0xf935def431a79bccUL, 0xa27462bca56dfc2cUL, 
			0x692e0a522d4ec8c8UL, 0x19598ec7951c76a3UL, 0xfb41928b07f8d395UL, 0x7696bbedec45ceacUL, 
			0x8c708914370bb987UL, 0x731c2856abc69cc8UL, 0x8384c7d8333adfd0UL, 0x0370d52a335a3aa9UL, 
			0x8a3c1e70b6237bdeUL, 0xad9351f465a30725UL, 0xe1f2659653b1c72bUL, 0xed37cf1ff4153e80UL, 
			0xd3f2dc6cc2f6454aUL, 0x40cd3ecb87767990UL, 0x5981895ed3b32b58UL, 0x56b1b3b42c78b8f0UL, 
			0x449a94794a27ba15UL, 0xd56f8db91c2c00a2UL, 0xff7dc513e72c86cbUL, 0x8623097156114b90UL, 
			0xb756b69ffd5a4fbbUL, 0xf8ef2aa395eb36a4UL, 0xdfc10221321b97a4UL, 0xf3031796d4d81b3eUL, 
			0x105e0422b4283c8bUL, 0xab08beebab4f8b55UL, 0xe08b95c70a74fc9fUL, 0xe082dff6a5103960UL, 
			0xb657967436ccf86aUL, 0xcf17b18b87f41b88UL, 0x2f49484592e0b0a8UL, 0x3dcfc02627a6a1d9UL, 
			0xbec9a89bf7f3bc36UL, 0xbc8dcdd5b772a10eUL, 0x4db2584aa3f670cbUL, 0xdb46b736f7124c3cUL, 
			0x8f98d589d0eeeb30UL, 0xe62752cbb12d1c52UL, 0xa558184d6d4e0225UL, 0xd2310e51d9c28837UL, 
			0x89d73732d8856735UL, 0x0ddd49ddc44960c8UL, 0xb01aaa5023192b11UL, 0xab4f197bc70647f4UL, 
			0xf579aef93dca5149UL, 0xe11c8fd7dd056732UL, 0x657bae6c7ff7d322UL, 0x16408aedae6d37dbUL, 
			0x86d23d010b970253UL, 0x66896bf8b0b8a2d4UL, 0x4ca1f65d3d0337edUL, 0x6438c508cea5c0d0UL, 
			0x5a5ffc1a990e7f82UL, 0xc6b05571f89d7fb8UL, 0x12fbfa68b619641dUL, 0x8bd64f51a6891dbcUL, 
			0x25c54631c2e40785UL, 0xd9bcbaed27c5522aUL, 0xefa794520a1d267eUL, 0x8d03e0db6446a829UL, 
			0x51a0d62ab9bed2e0UL, 0x99355dc0d29e9227UL, 0x3bf04b5402bbeb82UL, 0x951d96e133adf43fUL, 
			0x314a2b7d00a70f9aUL, 0x1f3a5e2c757c6265UL, 0xb1464115a266fd2eUL, 0x5d18644df957b33bUL, 
			0x2126789094211e59UL, 0xb105672c9f4ce8f3UL, 0x3c854148d772b078UL, 0xfba744323fcd2484UL, 
			0xcd27a3a794da33d3UL, 0x13d167dda62e6432UL, 0x4d95f78ee6ec8b0eUL, 0xd02f0c25eb3904f0UL, 
			0x65d693427922fd64UL, 0x825e2ef8a84d5bb8UL, 0x2fa0d1e010f7c2f6UL, 0xfb45a13c79bd1245UL, 
			0x410f04ae571508a7UL, 0x7e757ed92546dd43UL, 0x0cbf6645c9058e19UL, 0x58ccc6145ab38258UL, 
			0x9a5f0758d74c2984UL, 0x346550e159175131UL, 0x2db771ea83b30db2UL, 0x99009cb994d98945UL, 
			0xa9287027033a11d3UL, 0xf3f52cec19533e7aUL, 0xd38e46707f0d6f67UL, 0xf9e6a1efb88c611eUL, 
			0x8a452dc08f3a5723UL, 0x0707e38e64e28d4bUL, 0x48a1299f3b968e46UL, 0x3d144edf1fe7c1d5UL, 
			0x054c7acc7a56a8e6UL, 0x86d7d14788d74533UL, 0x3146834056341590UL, 0x4386724581f4b9a7UL, 
			0xf927a52809a1b3b8UL, 0x50727630531efe0fUL, 0x2428e60f11cb4239UL, 0x9c171f8472ab2e8dUL, 
			0x1e8bfd2fb256fb82UL, 0x1d847d7cad1b9837UL, 0x3c2b16ae4c5b0874UL, 0x481c0c36da63f07bUL, 
			0xa9fcc827d881aaa7UL, 0x8cc6920efecf6f75UL, 0x91fcdc81707caeabUL, 0x17cafe319b43804cUL, 
			0xc12bb9a4059fce83UL, 0x6d3caf4f2cc38b82UL, 0xe9152d97218a8270UL, 0x4050053a83d9c9e6UL, 
			0x1eb182070bf3c6feUL, 0x2e4bb6f282844709UL, 0x1eff8e58fb10c1bdUL, 0xec42ee893017c59cUL, 
			0xdd3ebadccf95198cUL, 0x2e7af66133297b85UL, 0x3c5fa91811538babUL, 0x4a04f709ba355a8fUL, 
			0x5f9bc845078f0350UL, 0x79bfecf33364d784UL, 0x67686262a16c0549UL, 0x92c469a1570ed99aUL, 
			0xc5d48067f8ed1e8fUL, 0x62dd57afcbf44131UL, 0x6fbd890a91f62cd7UL, 0x38ea40f5c9f8857eUL, 
			0x5335eefb6851721dUL, 0x14cbdfe6c99ad530UL, 0x1b946f1a69062dd3UL, 0x581f872d459e25abUL, 
			0x49d50b6aca39df94UL, 0x94a1abdb2564b966UL, 0xca4aabdfd28d5101UL, 0xcdfb7541cb8482d6UL, 
			0x8914f53e06f0fd21UL, 0x7d641eca290986b7UL, 0x62aad688e3b789b7UL, 0x16e6f3dc538f4972UL, 
			0xdcd33f8d7c298e5eUL, 0x7c4053042e4bfa94UL, 0x67e8a871571ccf7eUL, 0xb63fc7dd4025bccaUL, 
			0xe6a8a33eb82fb823UL, 0xad6f01fa2a4686aaUL, 0xa41fec442ad7490bUL, 0x3b65af80d7ce9435UL, 
			0x92ad96bcc43ed40bUL, 0xa468f2d7e6def47fUL, 0x395cf9af1e71876eUL, 0xf1925d40d35b9db9UL, 
			0x5b2e75ec370408d6UL, 0x66375d88ec828dc1UL, 0xc0f5c3d2911f6377UL, 0xf3a426aee2d41e16UL, 
			0x58529cc18da7f095UL, 0x73be6e82487c14a8UL, 0xa269a15c770ab97cUL, 0x43d1c83a7dec7974UL, 
			0x10d1574622d6b06dUL, 0xd1edfb6121a6b64cUL, 0xdc4428616af42188UL, 0x2705e671067145a5UL, 
			0x555e1e3d4b07c6efUL, 0x9c0852ea278e0637UL, 0xb19455126009e27bUL, 0x141078aa5fbb6a78UL, 
			0xc22b236b7d3206ddUL, 0x928023383e152e02UL, 0x3d0bb396a09e51daUL, 0x05b091f827795425UL, 
			0x1438e95b8b8bf644UL, 0x7172857934720d3aUL, 0xaad6078009d678a0UL, 0x8e24e8d82933b7abUL, 
			0xad10a0c792dbf991UL, 0x472bf90febc69f33UL, 0x99cb85274344376fUL, 0xd31c5626caebdba0UL, 
			0xdb92343cf2c73c0bUL, 0x4d55679a3009cd59UL, 0x6af458807162fe4aUL, 0x240a1dc733af4e9dUL, 
			0x87974caa87386377UL, 0xdede7b1827a3fbc4UL, 0x683b1861bd29027eUL, 0xbd319cc6187f730cUL, 
			0x0def6264ed54e23fUL, 0x433d8d74babe7686UL, 0x27fe348c011bb9f6UL, 0x5bd77db89d094b64UL, 
			0x9d7618fa0176a445UL, 0x5cf0d5f107977dfaUL, 0xe61c69d1846e4405UL, 0x69654e1f2c74ab00UL, 
			0xc1cf1065d5afa5c8UL, 0xbe8e198d70f18ad7UL, 0xaf31208dad7602b2UL, 0xac54462731858b7eUL, 
			0xa538cdf1b60acfbaUL, 0xdce8fbc41402aff5UL, 0xe78950b54879dd37UL, 0x3c41c73fdabd05feUL, 
			0x9ab22c094dc3f4dcUL, 0x67b0d25097c7bf0cUL, 0x2a142bbdeb845a5bUL, 0xa0d290e60b669575UL, 
			0x470fac1cfef48605UL, 0x16d289658f80fc3aUL, 0x32a6d89295e59a53UL, 0x5514149f19bca724UL, 
			0x19a800eecfd75abcUL, 0xa82ce74fe0b2448eUL, 0x7eac71c98020e6e6UL, 0xc898976815624a3eUL, 
			0x037f0f7083aba7c8UL, 0x217f383f5109718aUL, 0x60baf43dcbeb1e25UL, 0xc7d24c910eeda8faUL, 
			0x1c09b0137cd86724UL, 0xb40406527ebb60a2UL, 0x5206eb115873bfccUL, 0x7a7fc522880c020bUL, 
			0x74a010a72d2ec3aaUL, 0xfc15df0113013273UL, 0x0ff15c10abd3c255UL, 0x9d57dafe82f64a0cUL, 
			0x012ae0618056b429UL, 0x98e930446e2e7c41UL, 0x4a819d4330f973deUL, 0xa3097cc0e291298cUL, 
			0xa8a03303c372de6eUL, 0x9e9c51a7e2a41bbcUL, 0x514b997807b6034fUL, 0x75fdb08c4f9128a8UL, 
			0x6da2b75bf8f4b6e2UL, 0x04d99ed97cc47097UL, 0xfe55a0958621d5e0UL, 0x44be7ecddcbd4e9aUL, 
			0xc35526fc49451fa4UL, 0xfab1f32801cd5958UL, 0x32ac527307ab8f2cUL, 0x15616b8fc91b68c2UL, 
			0xce4dc0f37b3699d9UL, 0x323b7dca7f8480e0UL, 0x7dd812e895134e33UL, 0xb6bede5bab267476UL, 
			0x5e6151ca30716b26UL, 0x08091d99ddf55696UL, 0x9c2802a0cca2ab39UL, 0x16cf3b2fa6982a73UL, 
			0xb6c7bae62a5a51bcUL, 0x1902000af8520a19UL, 0x178983db5171c0bfUL, 0x5423ad557e239512UL, 
			0x036e8ee9ddf60e71UL, 0x8ef2d8ee59625937UL, 0xd5841ee1f23dd24bUL, 0xfcae85c1cf42612fUL, 
			0xfcb631545e5e7ecaUL, 0x3013d345638ccd19UL, 0x8d2d067e213b9a05UL, 0x8618ea28a9016a3fUL, 
			0x3173236e9025c5efUL, 0x2398ef084d08beb5UL, 0x581b4ccbdeb4b5c3UL, 0x6174b9afe5fb5d6aUL, 
			0x82bd1d041700a93dUL, 0xa85c19103bd18668UL, 0x1535851357c122d2UL, 0xf9b2c6e2fd739e2eUL, 
			0xe1bc3c187b106a2cUL, 0xec4617811c292530UL, 0x7ffdd85f5ddb823bUL, 0xd6bcf695d90bb1dcUL, 
			0xfe3154c73c09a933UL, 0x9e4c97e93948ad3cUL, 0x5036132e352ce9b8UL, 0xc4c93700e51129e5UL, 
			0xce2d9618eab3f957UL, 0x03b8141a5150ce25UL, 0xf86854c29331f096UL, 0xffaf5e194d9e1637UL, 
			0x6b146bd84ab70ca4UL, 0x4b5e7b4c9a2bc957UL, 0xdb35e17cf2e44e76UL, 0xc190c9a7a77107b9UL, 
			0x3d0e5651a9e786d6UL, 0xc8ec0dda5e00e54cUL, 0xbc4012a73d42b366UL, 0x176ecf7f33a4204cUL, 
			0xefb89579c5fc2158UL, 0x36de6294b5ffc07dUL, 0xe3cbd0da63a195b4UL, 0xb6b758c081e5aeaaUL, 
			0xc516b746b59e4f47UL, 0xd467bb18cf2ccaf6UL, 0x92686c2a4b2e5a9cUL, 0x90e30436151b62c7UL, 
			0xb602aba6c21c046dUL, 0xcccac461c82a5910UL, 0xa11c4333d8056dbdUL, 0xbb0fef4824969dfeUL, 
			0xa65b1ba832624057UL, 0xc78e025fbacfcef0UL, 0x9b2aeab09a3e548eUL, 0x8457aa84ed4ec792UL, 
			0x59c8acc20307f1dbUL, 0x2e7c93f70007c806UL, 0x4c11a423e0112b5aUL, 0x2ac64cf3c3dbbda6UL, 
			0xa2e046d17635164bUL, 0x26df2e197b3d560aUL, 0x711cd18795156bfaUL, 0x1815a3fcb73d22acUL, 
			0x890bfe8b8d1e04b0UL, 0xe18b6a6aacd662caUL, 0xb876823702ee35ebUL, 0x21c636f4d393ae89UL, 
			0x0f833437cf1a5b4bUL, 0xe601c21c48a9e674UL, 0xf5ecc5b2f718d855UL, 0x0df2faaaf155b8b0UL, 
			0x06835684ce6ec3ccUL, 0xaebb8bac8d28aeefUL, 0xb0dfb2382a51d2cfUL, 0x2ad8ad8371bb1354UL, 
			0xd72e852d3ae51f23UL, 0xbf8bed80ae9bb78dUL, 0x8d0a72059e7f232bUL, 0x3217e260cccc2ac5UL, 
			0x51c9b126c667309cUL, 0x75b2843490d8a173UL, 0x7bdb1b676a0ead1cUL, 0xdf4689300cb723eeUL, 
			0xfaefcee44317ca95UL, 0x7fa4e798c65d8a80UL, 0x23e28191bf44049cUL, 0xa2c4c6b581224ce9UL, 
			0xda4d2c4f3f7cf4b1UL, 0x40a7750ebe47f89eUL, 0x9900eb5726264f1cUL, 0xd3ddc8632390f00fUL, 
			0x9341c0b2872982a1UL, 0x967e92227abdc477UL, 0xd6cc48e86c848ef9UL, 0x6efeae65375e32eeUL, 
			0x724b8df35c79fab7UL, 0xc8450b086f70dde7UL, 0xffceb617558448b9UL, 0x369e4a42851fb61dUL, 
			0x0e47733dc902d36cUL, 0x9dce72bfef4e67adUL, 0xc2ed697686a5e682UL, 0xf42906298d9e20e2UL, 
			0x46d5bef42b017906UL, 0xb71ec165075d2f2cUL, 0xe6c941086d86d255UL, 0xac903d5eb6102b9aUL, 
			0xac20411a1305e4ddUL, 0xd36732eb0fa6e574UL, 0x2e096e469f35999fUL, 0x4bdec769809ee3b3UL, 
			0x6582674c93b4d941UL, 0x3a2fa1454c0a9e96UL, 0x4606ef6ca7b61003UL, 0xc6dd2bbe0e27e94cUL, 
			0x4d62a065d807e291UL, 0x1583539b103af486UL, 0xb1be64b279387b87UL, 0x73bdc6459bf33618UL, 
			0x530b893fd6005bb4UL, 0x6c032ec608a8387dUL, 0xda7e0367c5c1a629UL, 0x0b071346a4f31c6dUL, 
			0x39a69b7071eeeb72UL, 0xe9eb5bc64c4b4b3cUL, 0xd2a0fe8bc0b3ddc7UL, 0x1ab94cbfe2abe783UL, 
			0x208d60b03906c23cUL, 0xc4f21f34bca907bbUL, 0x9c42e9dff82e2041UL, 0xf380413b2ebba993UL, 
			0x2ec46bdfc1386b7bUL, 0x53cabfa200e5de73UL, 0xb4e65ff155f4a26cUL, 0x9ffdd3e2ecc0df3eUL, 
			0xebadc3e671462054UL, 0x606699387b6337a5UL, 0x041e26a3a1d339aeUL, 0xb44086c33bcda1d6UL, 
			0x25ea5e8923a354d2UL, 0xe3a1c903470150eaUL, 0x572851f63c5631a4UL, 0x2be1c12b4846647bUL, 
			0x5fc293cd3a753dffUL, 0x1755fbffe785ad94UL, 0xa6740cb750e1c3e3UL, 0x6314ea9f5e695462UL, 
			0x4be508a44791d47cUL, 0x29873157fba65e61UL, 0xcefc72a7fa5e897dUL, 0x57db661825aee39cUL, 
			0xc8d926cd12ca9ea2UL, 0xd087dc0d6ac07dacUL, 0x7d25976daa3449d6UL, 0x9ad9cae4c3e1c3acUL, 
			0xefcb3a5eaad3fdcaUL, 0x1477219ed35096bfUL, 0x6ddda2bbaf585b37UL, 0x42a21822efda6a81UL, 
			0xcff918a0788b8019UL, 0xbfaf3fd51790ed67UL, 0xf51557f183b31e71UL, 0x7b4eb346214fc246UL, 
			0x2b2ca06e4a869963UL, 0x0886361868a595b8UL, 0xdffade205fa1d0c0UL, 0x6bdca9ff2401d86bUL, 
			0x6518ee0408f63e72UL, 0x83a0c7611052c8ddUL, 0xa76d08c704805adcUL, 0x6b9196351fae3005UL, 
			0xdb4c89fbec24ef4aUL, 0xc06b406ae17c6fd1UL, 0x4ac71b9d1ea8c5eaUL, 0x3caa4f4cc1ebdb4dUL, 
			0xb49630ef6cb7eb36UL, 0x96fcdbd0e3933b36UL, 0x9b4e32e510c75f20UL, 0xaa58ac053cd2753aUL, 
			0xa822e062abaeccf7UL, 0x8752639015fa0dc8UL, 0x259aecea99a9ed97UL, 0x2955ff70df26ff0bUL, 
			0x7fcb3d8c0e7ed31bUL, 0xf7b3fb5d7180250dUL, 0x9a8d82ac800b3587UL, 0x5641d7a72992bc52UL, 
			0x1e2e3c6c7ec2cabdUL, 0xbd2e46428f070319UL, 0x2617f7a4393e049cUL, 0xcc36a5548f437f06UL, 
			0x5aa17a25ab2fc85bUL, 0xa5c35dc1eb585d90UL, 0xf7a643f82be48de8UL, 0xaadf65d0a4958e21UL, 
			0xe679d17362fb75bdUL, 0xd8250b32b75c29b0UL, 0xf85f32d85bb8caa9UL, 0xe557cbf5813ea13cUL, 
			0x371c03e1b8127f9fUL, 0xf3508edc20f26ae5UL, 0x6d5694d991cdcf6bUL, 0x0c8fdb4395fcfa6fUL, 
			0x951956ed50ac03aaUL, 0xfd23d259f07615bbUL, 0xd2e265ca652d4e39UL, 0x3d5980911e9982e0UL, 
			0x9508ff4151124836UL, 0xf0659bed3a176582UL, 0x2cd2535b9ad228bdUL, 0xa3fd5ca927a3a422UL, 
			0x6bc670d7ff8ca0f6UL, 0xc0e4b581faf15443UL, 0xc0d49e6cc505518aUL, 0xe7dc0c4792ce1709UL, 
			0x38b3eceef37cbd05UL, 0xf8a1e5e39427c088UL, 0x56e8fbb0a7b17036UL, 0x7db0564e81d5f50bUL, 
			0xe1da41e69cf8b76eUL, 0xb51cab0836954908UL, 0xc77ca25f1387fce5UL, 0x31b79e1e3706fc08UL, 
			0x78d35b9687605345UL, 0x279bfbddbf0623f3UL, 0x4c9179d3e3dde434UL, 0x6ed9d8a9cc6442dbUL, 
			0x3c141b0ba4c01ddeUL, 0x8cbc2ecab623f8c6UL, 0x3a16c7ac0d8b5fcaUL, 0x0e13818ce15e3f22UL, 
			0x11c7bc28ca612752UL, 0x1dad41a606376982UL, 0xb1a85efb08e586c5UL, 0xdf84bd0d6f1b1f0cUL, 
			0xd49a987f73d75871UL, 0x5b43ae211a56464aUL, 0x023b166c2914d29fUL, 0xf6e58887e24e9bd6UL, 
			0x29874865031fa82aUL, 0xdd49d90b9def0ae2UL, 0x002d8cf96a1d9c95UL, 0xfe6e4f309c60c7d7UL, 
			0x1ced62ea17fa6503UL, 0x21edd79740e569f4UL, 0x8393a776e8715a9cUL, 0x05b8bd2dfbd182a8UL, 
			0xbd71f44945294523UL, 0x10449636a4883db6UL, 0xdc15e2b31d66a473UL, 0x8d033594054026e1UL, 
			0xfa857064278ffaddUL, 0x2af83995da7b7190UL, 0x79b767e74eac792bUL, 0x7ad722a8fd2d0e11UL, 
			0xecca0fe6e07e4585UL, 0x1c6dddd8fb0de242UL, 0x8e02e6aab96c07f6UL, 0xd9de1062228a2e3fUL, 
			0xf043b02ef545eaadUL, 0x3961be7ce40a609aUL, 0x60e184f63b00e4a6UL, 0x4e26be22dbc5eb0aUL, 
			0xb9756102cef85c37UL, 0xb65b804b5c46c4e9UL, 0xbd2b7a8bbfc69e64UL, 0x2afad20ba17d6ae3UL, 
			0xd570853a0b8da0e0UL, 0x560b97b8617fb372UL, 0x9c88c825c0fa8e9eUL, 0x5d4ab1c1910d86b7UL, 
			0xabf00da17172991cUL, 0x6be6cbee31d3c846UL, 0x94c9a729bab8dba4UL, 0xc959926711eb6187UL, 
			0x954aed8701cc5eadUL, 0x0667d86d72d51929UL, 0xfb1d9174338fa7e8UL, 0xd1113bec9a5b9519UL, 
			0x795a1fbe2512bdd6UL, 0xf407b035d5534905UL, 0xc7c3039b95cbbdc9UL, 0x15a3d5fe4a92d08cUL, 
			0x8ff0c0d583689126UL, 0xe0e69a759b385499UL, 0x58fa30696d231a55UL, 0x50c9b12f77dd9ad7UL, 
			0xfa4d9e5a634fb04fUL, 0xc562ef03b405b388UL, 0x43c5047f23cf32a5UL, 0xf887fd1115f85dbeUL, 
			0x680fcf395a7dc16cUL, 0x37426186955e30d9UL, 0x6d16dbd6ad8f50e0UL, 0x0a691c0bcdd3c5d1UL, 
			0xda93deb5ab1f155bUL, 0xcadf39f6388a7b68UL, 0x699b1a8bc38df7bdUL, 0x2146d500ae56eb2cUL, 
			0x4409eea8d158cf79UL, 0xf8c1c45c4c7448fbUL, 0xaabd0e8d2278a21dUL, 0x87cfe2cea4fcb9d0UL, 
			0x28ed5db8d8d6e981UL, 0x0226d35212b37338UL, 0x48800493ecf4e4a4UL, 0x1a122b358c1c1683UL, 
			0x9babaca16100c9d7UL, 0x239a08b466dba460UL, 0x2729b4fc70373024UL, 0x4a8ba8bcea8620abUL, 
			0xe2169aa18561a92bUL, 0x9217921e15fdc1b8UL, 0xe4b15070aa48e95aUL, 0xae4be46fee6fd316UL, 
			0x679c8e49564aba94UL, 0xedbc93f6d94d416eUL, 0xeb9e7e8ae1d5e1afUL, 0xa915c8983f19c736UL, 
			0x0d7e222a61a46319UL, 0xd3d6b774f6cd9e2bUL, 0x8a17930554e450beUL, 0x4115badeb0bbe2bbUL, 
			0xa67d4341f67a256cUL, 0xd8fe7317ea147061UL, 0xbe8faca85fa50d78UL, 0x493cbd6197a017a5UL, 
			0x3430025cc7bc4b9fUL, 0x023ba7db3d0ee802UL, 0x644c799551435518UL, 0xee9dafd067a52e2dUL, 
			0x270b002f4a06a540UL, 0x15ae2140da29f360UL, 0x878567ad4d4cd4aaUL, 0xa2b3c8636ac59de1UL, 
			0x7b02f3315317f058UL, 0x32d301d8188af6e3UL, 0xd243c42a0fc338aaUL, 0x6e7bd7bf322f4beaUL, 
			0xeff4041cd2a76fa6UL, 0x880b4171c2ed0bb9UL, 0xb20776de9c9223b4UL, 0x5081fa964258819eUL, 
			0x927832d8728800baUL, 0x614370cb3b7fb149UL, 0x5b0b99868847332aUL, 0x085fc4faffb6401fUL, 
			0x0e362c7e5a05a54dUL, 0x36689db224011f63UL, 0xc2955018cf550609UL, 0xfb553f62a6797e98UL, 
			0x9611c75657dc052bUL, 0xf0dabdc27c1e15c8UL, 0xc150e15ad6a00682UL, 0x0786ee177f4d7e27UL, 
			0x5e7f0f51805a62d2UL, 0x05a9679c60c387afUL, 0xf084f35b5f3276cdUL, 0x22a26b2f5b29c9d2UL, 
			0xd7fb9a2b96254f76UL, 0x8cf3f7a684afed10UL, 0x56a2859c557e8cdaUL, 0x4e0c683267339062UL, 
			0xabaf00b8a1718ffaUL, 0xda5b3773507c56e4UL, 0x54aeb6179cbcf5c3UL, 0x250fbbcd3bec82edUL, 
			0x654edc85b080e533UL, 0xe2f9651cd1fddc1bUL, 0xcdd29f20fbbce16cUL, 0xd2a48979e7e57ec4UL, 
			0x52b83776daf1770cUL, 0x2d157f4b1ffe2ae6UL, 0x0e484a7015ef66a8UL, 0xd45163175a9143d9UL, 
			0xd954d3c0e06bc0b0UL, 0x4a0b9e6e10bcc4d4UL, 0x9415abf959d7182bUL, 0xcc04ce41528b878cUL, 
			0x46a190d4145f64a8UL, 0x411f446b9207256dUL, 0x9ee4544e9013af15UL, 0x0cafeaf38a1fcfe6UL, 
			0xeeaffeb97d291b89UL, 0xdf01d79041076112UL, 0xf8d08a36382e8e0fUL, 0x6d19aadc413f49a7UL, 
			0x6c63b056b1fb431cUL, 0xb0fa733d5dac97e1UL, 0x4a89862780e42580UL, 0x213c49aadcfea72eUL, 
			0x2bcd0854846c4ae4UL, 0x5e84842d2a2baaccUL, 0x9912f7f006c4fb81UL, 0x7bd12fdf5871193eUL, 
			0xf5c3778c6410abecUL, 0x0582043e30ad9508UL, 0xdb6ceaa1e59738f8UL, 0xd524914ffe463cc7UL, 
			0x2a53bf4b3da1bcd8UL, 0x023a4d2a3325c042UL, 0xab0e109e92c3509cUL, 0xdfa91e71ce2086e2UL, 
			0x1d0393e02dfe274fUL, 0x366f57b2220a86baUL, 0x64272e0898ac1614UL, 0x04ca2aa5afbd4803UL, 
			0x51dba41be18d2fb9UL, 0x95484aa868b95df9UL, 0xf7c3f78ff764a75aUL, 0xeac9c7120005a7b3UL, 
			0x472d848e5a797328UL, 0x01e753d61dc6ba56UL, 0x53b60309934be042UL, 0x75569ab3e0fc8a18UL, 
			0x5440a828f9bc1f7fUL, 0x6747b165a16d6f9eUL, 0x6a30ebc3950a446eUL, 0x6eca7fd7f6259dbeUL, 
			0x6b9d55cff93a3b92UL, 0x4d1ff631b8bbb01aUL, 0xc396e92e0b8d6442UL, 0xcf50e1eb5889425aUL, 
			0x7bcfd066375b0877UL, 0x81835b99af514708UL, 0x06c1d85b3ab27774UL, 0xfb4e9de6c892e56bUL, 
			0x269e87f290716042UL, 0x38ae3a6b4454fdafUL, 0x9a579e0c385b3a48UL, 0xf870d0fc229e18f6UL, 
			0x8384cdaf564724d8UL, 0x24ed85ce0321c312UL, 0xd29ba7932abbbbb8UL, 0x6f18bbc968b3bdaeUL, 
			0x8301f9c670e2d3d3UL, 0x47e794d87db71047UL, 0xde0f8d400021d8caUL, 0xbe90a428f8070f43UL, 
			0xcbc4f587d7b54373UL, 0x5435331520052bffUL, 0x92d39afb81ad1197UL, 0x0bb6679768357eb2UL, 
			0x2fc01d63ab7d9b56UL, 0x6bd1ea47fff454b3UL, 0x3f8b9db4a784b9a0UL, 0x98291f1c5ae0979eUL, 
			0xa2f0519a554ca329UL, 0x0b0e5ec8f23f4032UL, 0xd3c470fa431ac23fUL, 0xefc7142811efc5b0UL, 
			0xe09c5274f2a1ad74UL, 0xc2473d67cdc971c0UL, 0xc1df086f0a05db7cUL, 0x081bbd24de139c74UL, 
			0x1394804e3ecff38aUL, 0x6a52fa2e8f4e5e28UL, 0x6a0e4d05035d1a89UL, 0x82914b4a8afd2486UL, 
			0xe8d702ad0a54bbfcUL, 0x935f34ed899e9155UL, 0xdca6188da066397eUL, 0x642406f33a76277aUL, 
			0xcf5d548b0343a531UL, 0xb96af903a160a899UL, 0x2282f66ace04b5d6UL, 0xdf092eb29dfc8eb2UL, 
			0x8f10be02ab0f7528UL, 0x1ab21d10444c9c4bUL, 0x38f8badf93a9a6e6UL, 0xc343115eb47d126eUL, 
			0xc6c7d5d08f30a249UL, 0x5951897186e8c897UL, 0xcac8c963f193e19aUL, 0xbfc2bc51d4157198UL, 
			0x1af156166896ae68UL, 0x1c538bb6efc7e3deUL, 0x05cda647850637a0UL, 0x7c7b7a0fe620c9d3UL, 
			0x7da35ec1dbaccd81UL, 0x837b24f6c0fda3a8UL, 0xec1b3a0097cfb40fUL, 0x20159a76620f2456UL, 
			0xac56b0a2f7bc2f66UL, 0x88519515eb5b9b6fUL, 0xbb112babcdbdc82fUL, 0xcf97e47c6f35e700UL, 
			0xd351bfd6a5ee465cUL, 0x8cb3f934f9692a15UL, 0xbfa0b878d094220dUL, 0xf7865a2224149473UL, 
			0x492f7829bf7b0af2UL, 0x0e92ee4f05a6861aUL, 0x7b3171adeb66f9fdUL, 0xb16f941d78d5b103UL, 
			0x2f6d4ca725d3a543UL, 0xbf037969186236d7UL, 0xfe001f128f08ea94UL, 0x58cfae5721e4a1ebUL, 
			0xf0229a1bd2a02e5eUL, 0x28c876353deff7abUL, 0xb3bf513e90f3d5d3UL, 0x6cd54170896c51efUL, 
			0x5409abae33446aecUL, 0x7108719f40068628UL, 0x8e62c89ddbe4dfd5UL, 0x231994c3831b28b7UL, 
			0x13b28d25edb67ee3UL, 0xd755afa75adf6df4UL, 0x206fc0505589951cUL, 0x36af975f7b2589e6UL, 
			0x452ab1bc518f517eUL, 0xf9d21c64a187e211UL, 0x1866c001e0a52a5dUL, 0xef70276e47787de1UL, 
			0xaac823959758ffb5UL, 0x6cfe062b62ba9439UL, 0x0c6ff723f4b8d355UL, 0x6b52dc2b22fa2992UL, 
			0x5c1931984014aa61UL, 0x4d14eb65b4de4a4cUL, 0x6dc558e582caa795UL, 0x676469c83e202303UL, 
			0x04963fff09a68e8eUL, 0x45f48e07845dfab4UL, 0x4c5c5fc6be7a3b19UL, 0xbb32e8fbe5d0dc2cUL, 
			0x6a7ac96a96bcba5eUL, 0xf85d8dfc32315ce2UL, 0xd61bcecf920dcb2fUL, 0x87edc749e29a45e7UL, 
			0xbc60f03426e43f85UL, 0xb1f84b59fc577cefUL, 0xea6506fa48048e71UL, 0x6a5dd41394759131UL, 
			0x1a72389c54a7b97fUL, 0x7dc9db43399321d7UL, 0x4289917ba591bdf5UL, 0xc25c884e8c10cb74UL, 
			0x25193a713c2b6902UL, 0x2dd138de0d6b57f1UL, 0xe1f09a3454ee5149UL, 0xea80ecfa1c3a6fffUL, 
			0xd9041d6fbcc38e2dUL, 0xe29725ab8bc0eea1UL, 0x80e8f8ea806df233UL, 0xa146a49a3dc17f9dUL, 
			0x82bb40a4468e56a8UL, 0x95112cc8cdddd26bUL, 0xa850424689780e28UL, 0x4cd22ac9f1a3ee7aUL, 
			0x09c09b7cff9a8da6UL, 0x56b9e11f6f3b7a55UL, 0x98b69f47f2011cf0UL, 0x2857311f7bfde19dUL, 
			0x9a93588e51914962UL, 0x182c033266aaea7bUL, 0xbf85d37aec1689aeUL, 0x9efdc34145f276c4UL, 
			0x22200cd95a8dad4cUL, 0xc2fbc8a1ab8df11aUL, 0xb902a5301ee0cc99UL, 0x1ecde89e2f1a2067UL, 
			0xc7bc4cb52b213f13UL, 0x3c2b8de437cd5ec1UL, 0x416861ce27824a09UL, 0x2fa004aab45c3d8aUL, 
			0x8c6357252ff3c61fUL, 0xb5a9c697430c7ecdUL, 0x7577fa47f3fca68fUL, 0xc00eba970a2c3571UL, 
			0x91afd4b138e19787UL, 0xef7f7e8dde92b178UL, 0x34035eb3069b2409UL, 0x90e7186af5358cabUL, 
			0xfd00b590230a375dUL, 0x7cdc717684f12bbdUL, 0xa64564abe033e0c9UL, 0x76f6b9af2e7ce5ddUL, 
			0x66f5b5d48b447fd2UL, 0x68c29ccca065e1abUL, 0x344bc6d9e6e92820UL, 0xe85c54018fb5a92eUL, 
			0x16115a7bf8a7f518UL, 0xd0422af4c8c050f2UL, 0x677a0728d1bc4b53UL, 0x98cf6eed9b64b25eUL, 
			0x2bd425ccdbe1db73UL, 0x8865388a6729de2dUL, 0xa4a893be97c58503UL, 0xd22ff10098c76e8fUL, 
			0x361d81e0a65d1592UL, 0x6014661ab1a0dcb2UL, 0x2bbadbb7ab398b92UL, 0xb866ab8bbf91e3dcUL, 
			0xed8fb6f03a35f0bbUL, 0xa2a1654f19d50d57UL, 0xec80dd1d8f00a240UL, 0xa8fb40040e8937f2UL, 
			0xc252d3e7ef7fef79UL, 0xc6c94ae9233489f6UL, 0x291191da93bd7af8UL, 0x6860b7673b7b25e9UL, 
			0x1066678d27097cd0UL, 0x582bb4578e3a4900UL, 0xfa973b4d9b4808baUL, 0x1a59999fe8c2e071UL, 
			0x583a26391fbc23daUL, 0x1d139d8107a7289dUL, 0x3323dca074acf1f4UL, 0x111ebc74ef8c0f52UL, 
			0x07be109a63ec2cf3UL, 0x0b6742376f31a982UL, 0x17bacd580ffdc060UL, 0xbed079bd184143c2UL, 
			0xcd0e5f0c977fe49aUL, 0xf4bb59ba03d4fd95UL, 0xa8a361dd4e509500UL, 0x4a88821bf2a96d44UL, 
			0x616bbb5d9318a877UL, 0x4587bdf20d60cca1UL, 0xc790d36816dae68aUL, 0x0a3672467432a67fUL, 
			0xb7193624f5d0426cUL, 0x4b1005beeb693c64UL, 0x2d5e2b46f2a8606cUL, 0x2d9db89eff202cebUL, 
			0xe87c9dfa231a50e0UL, 0x0b3911c1372dafe0UL, 0x8e633a9ab8ff6ad9UL, 0x14bc613173e4ed25UL, 
			0x206ef3ca4b3178acUL, 0x8940e51ef8b0e5cfUL, 0xa911ce39c83ca96fUL, 0xb4db972c1f1c8515UL,
			0x4a22f419f927d379UL, 0x88936ccf6dfa3222UL, 0xf060a56573f02df7UL, 0x7426ee6ea9057503UL, 
			0x8cad1696aaa9734eUL, 0x9fb2fbddfb2ca33aUL, 0x918174ef522aa3f1UL, 0x98b8743bc0db3095UL, 
			0xa878873abe3a22d8UL, 0xcb9065dedb100df2UL, 0x331ce14104943338UL, 0x2c345c954daf551aUL, 
			0x05773f4799441d12UL, 0x63954fe30b8a17c4UL, 0x451cfc64748cf5f0UL, 0xacf64d8a8d4a69d3UL, 
			0x7c061e3450043ecdUL, 0xbd8c593def749398UL, 0x75a7426c564faf8fUL, 0x910b2201752b6936UL, 
			0xf18afe3d9ae42154UL, 0x7ec6b8fa3d24d832UL, 0xa0c8e624ff8186f5UL, 0x42ac850a58aac262UL, 
			0x39b449af0fb6ca51UL, 0xe17d20d4d14c7585UL, 0x023f2c2a8dfc00a5UL, 0x02ce1d60afaddb8dUL, 
			0x2f76b7726ff3f4b3UL, 0x2d516197d68f2465UL, 0x0a8ed5eac83861a9UL, 0xccf6c06a553e46bdUL, 
			0x188be8bf5123ddb1UL, 0x538dad612e68d84cUL, 0x37fe599e78573aa0UL, 0x19b4300b9068e899UL, 
			0x3436a0fb1a874270UL, 0x273cb6f8f9c5f654UL, 0x254df80dbfd0814bUL, 0xd2c144fa2e14042cUL, 
			0xc2129404b7ae73aaUL, 0xd306fb9a262df080UL, 0x2f4c0fa1fae408fcUL, 0x78dc45c916f248cfUL, 
			0x6c509832d77b19beUL, 0x8e8c97b695762387UL, 0xd73447903e6cd4c4UL, 0xde6f4196d8271efbUL, 
			0x37a24be051cf9332UL, 0x2dab6e565eea7743UL, 0x3575eeee138d7d91UL, 0x3c0487819ff981f1UL, 
			0x4523dbdd1d560a01UL, 0x8dcc6bbc16714ab1UL, 0x310945cf72280aafUL, 0x4afea8def73e025bUL, 
			0xf4a11b1411e32e76UL, 0x684085db7984a88bUL, 0x2dae54525c0054bcUL, 0xb502fa7bf0750772UL, 
			0xfdb0c8e1d1ace8e9UL, 0x1f85ea2d3b6cae8fUL, 0x92fc3bf806959a50UL, 0x1872ae868bb52e9aUL, 
			0x3c0324c02d7fcdd3UL, 0x8c1650ee8035bf92UL, 0x8990d1fdc9e8610bUL, 0xab6e3d16b73ecafcUL, 
			0x498ff992827763b3UL, 0x3dc12f52a5d4f70bUL, 0x61a19ab507d9e2adUL, 0x2d4c97303a906e36UL, 
			0xdce9adf8e65abebcUL, 0xc213f085d0b1f376UL, 0x41dc70d0c57724b0UL, 0x0b01eb8449bd1260UL, 
			0x56371b0c996b50f3UL, 0x5203d0c43f1f2021UL, 0xe54dbf0896c6ae5eUL, 0x55ddd4b7ea4dcddbUL, 
			0xcebe43c427d37ba0UL, 0x8a3e2f22d52d3b86UL, 0xc7aa5256cf2100d6UL, 0x2b09d7abda7c88c6UL, 
			0x1988e48be6986933UL, 0x01e3806d56670ec3UL, 0xfd42f4ea4635f0f3UL, 0x5d476e04638386d5UL, 
			0x8ae9688f42697e56UL, 0x2f3c779f3c510d1bUL, 0x576b0fae7f08a146UL, 0xafd5788bef67593bUL, 
			0x33353d9974093b59UL, 0x53619a575bb08047UL, 0x0cfe9c195d96b641UL, 0x3fd9e5347e3013c7UL, 
			0x4beb6a0998cda1b9UL, 0x4bd36dbabd8a851dUL, 0x94fe7370922065f8UL, 0xfee7b97820a5b40bUL, 
			0x01d11cee643dda9aUL, 0xc86624614acfe328UL, 0xdd044f3e53697970UL, 0x9b9ca618e292d897UL, 
			0x09856783fe937cc8UL, 0x7f7627d50245e026UL, 0x36f54c3d3d319709UL, 0xa1b9867e33a731b9UL, 
			0x5a9e30780a261323UL, 0x53966fef4357f4aeUL, 0x0b1e30328d2fc1b9UL, 0xc846b03ddccf58f7UL, 
			0x7062dcc7852bff02UL, 0x4870ef3c439f7f51UL, 0x70af6698acf7fec1UL, 0xfeaac8a928b31decUL, 
			0xac8254e27e063025UL, 0xc94c15c4805e6ff5UL, 0x41fe013632bab397UL, 0x6045d61fb86a69d6UL, 
			0x98d50e1da2a97daaUL, 0xe0530eae9909628aUL, 0x40eb148e421294bbUL, 0x44f68400a2818500UL, 
			0x93e01aea377f47c8UL, 0x1be11d108486eac5UL, 0x6a19061e65c73b36UL, 0xcb88c03676792a3aUL, 
			0xccce4e73c7f77c16UL, 0x865cb6ee8f94d7e1UL, 0x384854ae7eeb35adUL, 0x0b6c9d962013190aUL, 
			0xda0fef20671b8c6aUL, 0x7073a3c90eac951dUL, 0xa515ea2c4084ae67UL, 0x8d5480bd3f8868f3UL, 
			0x75617d39a933eed4UL, 0x87926c7fb83d8e8aUL, 0xe950c5714b142bd3UL, 0x9737c74bd831fc79UL, 
			0x6fac6b211cd703c9UL, 0x2205e8ab91e4d968UL, 0x4391b1dd452bb1beUL, 0x188bc7b331cd7b6bUL, 
			0x3928b657aeee8d70UL, 0x55f848851b63f3e0UL, 0x987e81f508c153f9UL, 0x9ae91e85b2806212UL, 
			0x3667f411bbdb2c34UL, 0x0009e5346a79a47eUL, 0x4422675789e04cd7UL, 0xc849118266eb740eUL, 
			0xdc5ecbc9b4007f37UL, 0x1000bb253ad1cbcbUL, 0xcaafe90268bf457eUL, 0x99748e1af79f669dUL, 
			0x003ff1721db5d109UL, 0xf8635cfb22e05283UL, 0x7dabf9f8ff0ee35fUL, 0x5878577df4c6eb2fUL, 
			0xe8b7f780cad0ad69UL, 0xb19a790dbca48cadUL, 0x802bab78f9cc76d3UL, 0x2fc5fd807ed5f437UL, 
			0x99e368a7171d3f1aUL, 0xc3ef9e795d679c60UL, 0x472dff065549866eUL, 0x5b2c56192cf856d6UL, 
			0x56a7a44308e2efcbUL, 0x0936a563320c1c4eUL, 0x3982b48c44dd96eeUL, 0xf4787138fecd6b18UL, 
			0x76b1b8aac140a11bUL, 0x2113dca24d95fa8fUL, 0x26d19df48434902cUL, 0x627c7293a1d48882UL, 
			0x8353da1508bbbb18UL, 0x95746dd3f8d173efUL, 0xaff472bb55f6ee38UL, 0x7ababe9c3c0129e9UL, 
			0x10b0840698900df5UL, 0x43ebc0695aee7926UL, 0x73c9c9d95ff3cddbUL, 0xb0c4f60b8cede496UL, 
			0xef7f3c4d23b76177UL, 0xa194d01cf99f7ef8UL, 0x541082d98360c64eUL, 0xc22d57c813bb81f3UL, 
			0x03042a3d10205bbeUL, 0x055de31997ce6f5bUL, 0x9459c430a82c92d8UL, 0x78ea00c289799a4cUL, 
			0x26600641fffc7459UL, 0x2a6460a5fae7093aUL, 0x37298ecb107e4840UL, 0x8dc499ee4f79f142UL, 
			0x28e66561d5263661UL, 0x51bd0931d5fbb3b9UL, 0xf80f0d96a5ca33a6UL, 0xf484a765799d5a6dUL, 
			0x389e320b50e7eafaUL, 0x270be18c060fb7e5UL, 0x52ac4856f6a57325UL, 0x7cc9b809a58895e5UL, 
			0xc842b12122be1d0aUL, 0x0663697636a42586UL, 0x4cf1c4903e902eabUL, 0xb9af57bd9afb2373UL, 
			0x1ba571bffa3bff39UL, 0x1b15c120e52756a8UL, 0x3641794daaa14b89UL, 0xb049edd393612ae1UL, 
			0x3f9e0b59d9d0c491UL, 0x79f61b067c7d6bb6UL, 0xaf5faa3be46d0d52UL, 0xb17e9fbbfcfbf484UL, 
			0x11870061dc899b15UL, 0x572d49a94ff19fa7UL, 0x469f082d42fbcb3bUL, 0x4882c3e4b31808edUL, 
			0x00886320103516daUL, 0x1fcc537f11d37f8cUL, 0xfb33fcb4800281c2UL, 0x738c055ef6f05496UL, 
			0x703c56d4b57014ccUL, 0xad0d88107d5e41bfUL, 0x799d0ba78768d7a2UL, 0x6bfcf9b44b43ecd0UL, 
			0x8f6a7360542062edUL, 0xa65019ebf9c27611UL, 0x76757494a632cfd6UL, 0x771153c95c4e59f9UL, 
			0x3da2bd7eadc52ac9UL, 0x0d3d63c85df5952aUL, 0xf3b8c00f8436987dUL, 0xa352a820e86ce4f5UL, 
			0x2fcec1f4ef666afdUL, 0x8886f5c24bcbe71aUL, 0x25adeb6768f94066UL, 0xff192f81cdda0ff9UL, 
			0x678211c1a84cd15cUL, 0xa774049e86887fe8UL, 0x72cf93c46368991bUL, 0xa1bc7f234e7a7267UL, 
			0xe39387da44f120e8UL, 0xa4a52751b4c34055UL, 0x252da06b974fc876UL, 0xe34a063211d66a7fUL, 
			0x2dfa4c293c772df7UL, 0xabc14cd65911682cUL, 0xe69f3b5fb25fa6b1UL, 0xdac6bd1fbe113c57UL, 
			0x8ec7e03318f8f408UL, 0x1f249756ad8b2786UL, 0x5d828b154f517408UL, 0xccb82357a11ef5a0UL, 
			0xaf8133c3e415acf6UL, 0x7ee60069147ce745UL, 0xb09a93941109d571UL, 0x3bfc29ad0bd46c7aUL, 
			0xfc0223104c90b2cfUL, 0x1821d6698c64eaf0UL, 0xbbc525634729c952UL, 0xe9c9db897c777f9dUL, 
			0xb3208627f3e9fa16UL, 0x5c85d24a319e1080UL, 0xbf4769f90022702dUL, 0x40f55500ec1ee073UL, 
			0x50312a260c9227aaUL, 0xc01585ddd4678afbUL, 0x8749f8ff7bda1ac9UL, 0xb898b65d22c9bd21UL, 
			0xc41b5c973a4b88f1UL, 0xf370e9ea87a1ac4bUL, 0xcff8daaa354a5bd4UL, 0xd4efdff9c92ba012UL, 
			0x772693052bad78b1UL, 0xc71d371e33d21c85UL, 0x6aa4097bdbe4f7bbUL, 0x6b28329ef5862567UL, 
			0x06a544f804118741UL, 0xfb2e5f43732fb9acUL, 0x6d42861f7ec32177UL, 0x5cdb3555185d622eUL, 
			0xdccbe84474dfcad0UL, 0x998268cda558e957UL, 0x80a1dd6eff7057abUL, 0xa6f2dcc47328c93bUL, 
			0xaa654b269c80f924UL, 0xb7f468f6e3468065UL, 0xef7d80b7e135fe08UL, 0x7c71f7d58fcae887UL, 
			0x88fcd1ed1d6692dcUL, 0xffac0bcff9a1770cUL, 0xb5f510e8185956b9UL, 0x9fc7d963c037a03aUL, 
			0x8766bd37cc5fb4b6UL, 0x521e10610681c3a6UL, 0x7284ac174300d41aUL, 0xf3a30fccbe609ddfUL, 
			0xf69ad674494f9065UL, 0x1c75a0cefd521b43UL, 0xa96cf20f027006cfUL, 0xfabc6d12f0892d99UL, 
			0x361ef7e2af71b5bfUL, 0x230466873612b03cUL, 0xa1723a5aa28f49dcUL, 0x8d7c43ff81748718UL, 
			0x09f82462d2df5e83UL, 0xb6fd871386f9b1e7UL, 0x78a08f2de6f5617bUL, 0x823598664fad1bdbUL, 
			0x71abb8cd94814f78UL, 0x7e27b68681408750UL, 0x053dc33c3a44fd01UL, 0x8f053b7f1a7b4ea8UL, 
			0x6f33f7a273e03ea3UL, 0x00e25a6e6448c3e6UL, 0x7e8977c5c04cad58UL, 0xcda2a7580789b283UL, 
			0xef6eae52394430fdUL, 0x2d06093623c52ff4UL, 0x490358cb2f834d4eUL, 0x2e2bbd742c416d6fUL, 
			0xc8ec0071d2fa9152UL, 0x6e0f5f11c99c3ac3UL, 0x0e431a9c50fc54f3UL, 0x65c5d927aefa8270UL, 
			0x0771e40eeefce0b8UL, 0x46a7581d332b4dfaUL, 0x2c2ad8c7148d749aUL, 0x4cba239924b65b65UL, 
			0x4d5b9f0fb0ab1c65UL, 0xcd91ad6dc47d42edUL, 0xb13c7f3715ec1106UL, 0x1f89001165a6d0abUL, 
			0xac0fa4eff015a660UL, 0x440d36003c804f66UL, 0x90f635c56fcdf38dUL, 0x2b9d6994572cf6bdUL, 
			0x82c73fb1235624dfUL, 0x68730ade6811bf87UL, 0xd42c2a075a7c47ddUL, 0x1e1043bce7f43981UL, 
			0x8ab6d971ff75775fUL, 0x66e9681b6298657aUL, 0x8e704ee1d6b2c4d0UL, 0x1068d00cf93ce8dbUL, 
			0x8748223e20d172aeUL, 0x0d6012bb8b877b41UL, 0x9b92ec3e449ece74UL, 0x1113686f6e2d03caUL, 
			0x6018f38046f949b1UL, 0x1a73f8fba1d9df07UL, 0x835e31a3a43dbd4cUL, 0x0e776dc9de039771UL, 
			0xa5b9557c2693720bUL, 0xa7ce8c7ebc6be91aUL, 0x68dd8c3fa8c1384dUL, 0x182c031a9c6e00c6UL, 
			0xb230ed5695e4a969UL, 0x344f831c7c02cf6cUL, 0x9f9bd771d4c34910UL, 0xb269793ec615adf9UL, 
			0x6884f535b3f8cd4aUL, 0xee4a319f60d2054bUL, 0x2ae63aa029ac191fUL, 0xa6590c044d804f7dUL, 
			0xa3a3e6ddddb3c226UL, 0x73970a42692ca2e6UL, 0x599e6fcf1f577977UL, 0x450b262d895da8afUL, 
			0x7ea72c950539da8bUL, 0x541a9a9b6976823bUL, 0x2f824db93c03f3c4UL, 0xcbaa063453d7e329UL, 
			0x49070c5905d50dadUL, 0xbebedbeb553c5e6bUL, 0x2f34004352dec118UL, 0x9622c2670c00a06dUL, 
			0x25fb8074a7e8e316UL, 0xcec271042fce534fUL, 0xdfa6490eb43af3a7UL, 0x1b1eb52f8b6390f0UL, 
			0x5cc23e2eb829fba6UL, 0xa76b249a25d9df92UL, 0x369c05fe847e329aUL, 0xc61cd1c4e7984b0dUL, 
			0x03de2227f9eeb4a9UL, 0x63305465978a2062UL, 0xb03875c47f107d85UL, 0xb0dd68ed2ff602efUL, 
			0x0b2d6816a38057b1UL, 0x9ba46d4c19b467a7UL, 0x649e7ec494ac3d3fUL, 0x870afbabeec09560UL, 
			0xac35f05c3f09bfe5UL, 0x13f28f00b21e676dUL, 0xc48aacd2a69addefUL, 0x7ed507a1747729a7UL, 
			0xd036f48f5642bc93UL, 0x2f4e983374dc2808UL, 0x032262cab621d8cfUL, 0x712be61d7e28eed8UL, 
			0xea68c088f6698101UL, 0x939266ea7e77e86fUL, 0x7c13710851c78965UL, 0x2e6cf05e6d9f2b26UL, 
			0xe7e34acab252b86eUL, 0x96b047a934c568bfUL, 0x955710cf432cf53dUL, 0x0bfeb0190282ee05UL, 
			0x3be672e9d9907693UL, 0x1947d57e3b8fe7fdUL, 0xbbd1bae933cd7750UL, 0x99be8a1721c254d7UL, 
			0x577b541ac920810fUL, 0x27f71f4e55e6620cUL, 0x38ecac6728c3a113UL, 0x07426a13bc5c3e18UL, 
			0x0edddae671da0a37UL, 0x9bcb78c790ed56efUL, 0x5e19f893f0902ba5UL, 0x5df705eb5f7d35a7UL, 
			0x7ef60cb7a6bc259bUL, 0x96ee869ccbf472ebUL, 0xd0016aa8e5f508c0UL, 0x5b236d06f9c747f0UL, 
			0x63304617cf785f63UL, 0x1d12ab335451791bUL, 0x0388238060b1bd09UL, 0xb6834b18b7d06a28UL, 
			0xb9ebb43d8cd35011UL, 0x86d54ee2ed1af5e7UL, 0xb5282e7271f6fe19UL, 0x37bc94887a02624dUL, 
			0xe62a64e606d55abfUL, 0xc247b373a8b67445UL, 0x7e65cec3cd7b3a47UL, 0x995cdcce0607ee32UL, 
			0x06285661da24e87bUL, 0x9dc7a4964460847cUL, 0x8b3d99b6cc046cbcUL, 0x200c2021887d93f5UL, 
			0xe13f4b77d8d91f5bUL, 0xd26e7ec29ce6d7f4UL, 0x454be24b1efbbbe4UL, 0x08d9b325cc32888fUL, 
			0x51bf4a20a07b2f90UL, 0x2bd835873d25dd7aUL, 0xdc56eb7ad5bd4793UL, 0x96607e5c5f401209UL, 
			0x9d782d093347f17bUL, 0x9f911aac9a1ab697UL, 0xce67a63d97eecb29UL, 0xbef0cd869e8bab62UL, 
			0x83dccce9d4a44874UL, 0x35469283b2fe127aUL, 0x6ba9599104e0f7d5UL, 0xb281e847bdce8499UL, 
			0x2448f21c2f8288e6UL, 0xffff99649f830f94UL, 0x0b0bfc157e1a140dUL, 0xb27ab61f3789e3f0UL, 
			0xb531bdffeeb6c662UL, 0xe4dc08e12db3198aUL, 0xdc45d93345afce32UL, 0xea487d09fd66a251UL, 
			0xe6e84fb60cb593deUL, 0x705b1ce8fe39ffb1UL, 0xc4d14cb27be52fbdUL, 0x92f02ef7b04283e7UL, 
			0xf911ec86d4ca2e59UL, 0xa6152b3d3612bf16UL, 0x0fce2ce67096cf0cUL, 0x996effb961e55839UL, 
			0x23442a1f34932a4fUL, 0x57f255d42c39560fUL, 0x311abe53c8293e8eUL, 0x1c4e5f1029dd682eUL, 
			0x100477121b504b40UL, 0x1af46588b4cdbc0eUL, 0xfbd82ba72dbf9c19UL, 0x5c18c5583daf8351UL, 
			0x32d2902169b5e52dUL, 0xcd3426a8234109b8UL, 0x30110ca36875dadaUL, 0xd328d4ef2325e3bdUL, 
			0xcae4e2e15b29b751UL, 0x47ede62347d37037UL, 0x8fb1bae500c037c2UL, 0x1795e5ca26432902UL, 
			0x3b72533f3515e285UL, 0x70c4c6852aa1abd2UL, 0x3997edceb6a8722bUL, 0xe869c295cb07f0d5UL, 
			0x018b38019f8aaf10UL, 0xbad0a16c694f71ceUL, 0xfc80ee1351193f68UL, 0xe2473ea8a8e5ec3aUL, 
			0x906075363694ed44UL, 0xcb2598dd80307f36UL, 0xd0ea4ce01305b080UL, 0x7331c352d23e5089UL, 
			0xc99b86b44edb8af5UL, 0xa70cb41052e8f138UL, 0x7247ea5a72960993UL, 0x2a80971b5c968e62UL, 
			0x66cd543c1469871fUL, 0x05a3d63f0805376fUL, 0xa348dc6f561d0c6fUL, 0x0c14915f38b41692UL, 
			0xd4febcad73a15a9fUL, 0xd6066dcae0757665UL, 0xf4cba34993b5273eUL, 0x822cf2bb91cac643UL, 
			0x34e8b89610b0a78eUL, 0x416d147bb328643cUL, 0x513ca336c4ef3896UL, 0x7e6bb696752c6904UL, 
			0x965dd40ac91d34cbUL, 0xdecedbeef28a1636UL, 0x0d416cc69b940f40UL, 0x39ecae3c52cd2266UL, 
			0x2b8adb0818936d9cUL, 0xdb9cdbbbb0bfbff2UL, 0xa4eab5d1bebc3cc7UL, 0xe5f0d8dc94bbd189UL, 
			0xa49cdb6cc8b3a3c1UL, 0xa71e73c81e6a50b7UL, 0xe42562902d545a8eUL, 0x322b62bbc01531d9UL, 
			0x6cf112e4ee41471cUL, 0xedbe03d826b6a141UL, 0x641289ccb5134282UL, 0xd67fbad19d23009bUL, 
			0xa7cb2e831750d946UL, 0xdc0748dc2a1f07daUL, 0xdea5193e9393e0a5UL, 0x421777ceac29975aUL, 
			0xd77ce8b5d46a4d41UL, 0x1f8e145135d5919aUL, 0x160ba3de9c211232UL, 0xb515d4f1b5fc06b8UL, 
			0x4ca26b10b4384e70UL, 0x1c46637dcb8c34f2UL, 0x6b58b8fe1a611305UL, 0x68fe0a7791142168UL, 
			0xa6f4a34d14bede29UL, 0x602351df20b62b8fUL, 0xecfbcd64cb2e86b2UL, 0x82d575b0cf44c9a0UL, 
			0x80ddede1391845b4UL, 0x8942092082a5a42eUL, 0x64cad06c9200485dUL, 0xe81d8ff7393eca67UL, 
			0x40bbfdf00ec2c33eUL, 0x3a19b72f70136062UL, 0x07ee36d5af20cab3UL, 0x4170b115f6cbe3abUL, 
			0x36ba774a1187207bUL, 0xd022409d58dd2b17UL, 0xed5e2bc73b83399cUL, 0xa8db2693196a2286UL, 
			0x8eae0cf215482a37UL, 0xc8a74f3c10ce48d5UL, 0xe333163eed210c12UL, 0x16b9247dcb640187UL, 
			0x08e1c1a7f19672beUL, 0x86f9009fc7392af2UL, 0x7ad1622b08d256a8UL, 0x4dfb437b03f2f396UL, 
			0x803b556c3b16400bUL, 0xea8ec855e1168ae2UL, 0x637624cea22b455bUL, 0x6f31364a726022f1UL, 
			0xde75858b039fcf71UL, 0x228d172aae24b862UL, 0x32191eb8c8c8c1e6UL, 0x338dbd27689647aeUL, 
			0xfd17f2deaaf660afUL, 0x8001b79a9731e3cdUL, 0x148abbf3a5b3a346UL, 0x8ab1fe3155d8fbb0UL, 
			0x9a4767ce7b556beeUL, 0x1245ca8a654b2550UL, 0x5e031aaee271fce4UL, 0xc2b47674c27cad1bUL, 
			0x21e890a06134b47fUL, 0xa9c7baf9390ca8b0UL, 0xe087414746419612UL, 0xafdac1d2da5aae3fUL, 
			0x664d65aeef14479aUL, 0xd154c6a95d138652UL, 0x31f77cdc4588b16cUL, 0xe52822142a5e4855UL, 
			0x5864889f85381e15UL, 0xe8a0c2471f9cf217UL, 0xac5a166fc9419808UL, 0x4bf5480c081dd7f3UL, 
			0xbc6581da051248e5UL, 0x545c1a3555e244b1UL, 0xcd7d79071ab27fe9UL, 0x5b7dfef585ef43a6UL, 
			0x901eb60fefdea246UL, 0x41ee50346bd4321bUL, 0x3f11866d8288d537UL, 0x5afb8039acef862fUL, 
			0x606e862b621f7b06UL, 0x38dfc27305b8965fUL, 0x2c5b92fa6259d488UL, 0x9e81e6030c3798caUL, 
			0xad2691706d14c742UL, 0x3e6e9b65ef20f018UL, 0xa3a36400dd933607UL, 0x775a85e050912792UL, 
			0x17316ba9cf6b9179UL, 0x20233a81cec15dc4UL, 0x494d56434da13036UL, 0xe2c16c951045e176UL, 
			0xb0b98886d110b514UL, 0x0d486fafc0eb6a83UL, 0xa1d4b2f79e4d86f1UL, 0x73cc8b086b82be0bUL, 
			0x0f2aaddd5a8d8cccUL, 0xee310c24ae423255UL, 0xc52185cd4e9bc5f6UL, 0x5caa758cd74ee567UL, 
			0x3947b6343e0e779aUL, 0xf25e45c6ffd84543UL, 0xeab70fab06ffd4e8UL, 0xf977c011de65ac19UL, 
			0x3ca8bc55109f7a4bUL, 0x68661dbc58130ae9UL, 0x7fee12f5182a5574UL, 0xa4e209d165202b57UL, 
			0x5a04f5307c9c488bUL, 0xdb53ebb174479a3eUL, 0xc3e492f48da269b5UL, 0xdb60bec5cc9c776fUL, 
			0x547a2c49eab2950cUL, 0x23a4a5cc6783a35fUL, 0xb185a5acd2f55853UL, 0x1a7205c39084efa2UL, 
			0x2abef84dfca72606UL, 0xfce122c8ce694ed7UL, 0x5353c01268d9524fUL, 0xabbac17438feb5d1UL, 
			0x7e907a2c7ed3d28aUL, 0x99fb7ef30bdbdff7UL, 0x278e80d708395708UL, 0x3c3cb3c2eb08c37aUL, 
			0x1814a698111b19ecUL, 0xb4f91f87db60a234UL, 0x23afc312663fe8c7UL, 0xed0681095ab40bbaUL, 
			0xf7128e9536145745UL, 0x609051b1a4b8c26bUL, 0x0f76444495d7e1fdUL, 0x9af6977cbf307c02UL, 
			0xc135882d892d1289UL, 0x9259d43106185387UL, 0xcaf92666ac7a20eaUL, 0x312dac093ba4f166UL, 
			0xf14846df5944d295UL, 0x08c00b2494ca1281UL, 0xd8d6276e183efcd5UL, 0x468c7efde183a922UL, 
			0xb10ec5552b105871UL, 0x80935721183f4b4aUL, 0x34ced324bc9a5685UL, 0x8353bddaae2f142fUL, 
			0x1ba1269279b9e4a8UL, 0x66716659b0d3ddaaUL, 0x2f2f4d33fe81fe85UL, 0xbbf3a55b9d7553b1UL, 
			0x09230ffe2c1ae3f2UL, 0x1870ea5d65812792UL, 0xb9ff12284521f426UL, 0x08d4f7878bfaf990UL, 
			0x87e8a33cd32221b7UL, 0x6f57ad8e240d4636UL, 0x7119cabbcb1cee08UL, 0x68636e176f22174eUL, 
			0xf99fb68ed0af03f9UL, 0x4b95b7658f8e7235UL, 0x2e392329ba9d2716UL, 0x6cae7e04dd6b7873UL, 
			0x23277ab912bebc67UL, 0x1ef46c076cff84deUL, 0xc89d751238408c92UL, 0x2cd20ad785dfe3ecUL, 
			0x631bb16229536dffUL, 0x900acce6d3d3235fUL, 0xa6b4d1aae4dad100UL, 0xd3d0a0b17b7d1d4dUL, 
			0x6fc8d621a24c6a84UL, 0x39761d784705c22bUL, 0x6787a67838f36864UL, 0xa9823f89ad5f3237UL, 
			0xe6bb390265256c48UL, 0xaf180d1e02fa8d42UL, 0x57e96156bb325ed0UL, 0xf3bb9a98b976cc88UL, 
			0x52a17519e5017e12UL, 0x56562286e387d348UL, 0xf4fdc82332826b38UL, 0xa782b1c525e7174bUL, 
			0xd98b00cf7fbe109eUL, 0x833963165b4191ddUL, 0xe7d336a56fd8fe76UL, 0x2bb9dc637a187c6aUL, 
			0xe25df748922b2cfaUL, 0xe8f6eb6dd47d0c85UL, 0xcd92fbb620a4cff0UL, 0xc2c564b49817e199UL, 
			0x49fecd6313aeab7eUL, 0xd0306dd27ca9e387UL, 0x6465cf95ee910ddcUL, 0x84ff8a7fad83b3f4UL, 
			0x3e3ce260e3df074dUL, 0x409c81f7465aaae0UL, 0xa4928fbf05f59bd7UL, 0xaea983204d1481a8UL, 
			0xbcda9b41ce174b59UL, 0xdafc17dc1a239b5bUL, 0x0107d07a5734e059UL, 0x05aef3bf768a376eUL, 
			0xe53a5f7a7016b15dUL, 0xa236fe0c6b12e4fdUL, 0x6103fe7e86b5a05aUL, 0xa9418813208fcb8bUL, 
			0xea7760a6be3c9585UL, 0x6c3dc21c09d6958eUL, 0xdb9b0faeb9218eaeUL, 0xeae8c4345da6e44bUL, 
			0x541792e32ec6342dUL, 0xff073e8aad96dea8UL, 0xfe2f773f76215d3fUL, 0xebcb889233087cb4UL, 
			0x768a78b005344d22UL, 0xc1b58811895b5d14UL, 0x91cb187eddb97d97UL, 0x2b145f3d08099221UL, 
			0xbc11cfe2236da47dUL, 0xf322f5815454a7deUL, 0xf34cc6a46ab2290cUL, 0x61cf732095e54b99UL, 
			0x39ccd48e04248d65UL, 0xaff2e3683a1efec3UL, 0x0687d5d23a317d55UL, 0xb66f1a65f07bfdcbUL, 
			0xce6bc75366dc8c19UL, 0x9cd31edbe0c1ac43UL, 0xafea7af684f0343fUL, 0xbc71077457c97096UL, 
			0x1a9c71e8a928768eUL, 0x1de881df74d420a4UL, 0x91ca69b4f0b2f374UL, 0x28fd86b7b8a14122UL, 
			0xd99db21b6be901e6UL, 0x1ff083e239d6ec6bUL, 0x644e17ddf18e3847UL, 0xed3824eb7dba36e2UL, 
			0x8da25dff80d03474UL, 0xfa9fe05257d0e06eUL, 0x1d0a49789bf5dd36UL, 0xb3bb97e4f61081ffUL, 
			0xea190d8a7c1787a8UL, 0xa84c00cc5565f582UL, 0x2478ef0ed31fdff4UL, 0xb0cb606027a0af24UL, 
			0xb6f4ea79346b5aafUL, 0x95506609ba18224cUL, 0x8b5393bc22235833UL, 0xf8e71cf144203a68UL, 
			0x8467927c7146f1b2UL, 0x97835c37f57ae9b3UL, 0x400232a1ad003654UL, 0xf8d3d0055b3f13e5UL, 
			0xea603ebdc36c7509UL, 0x29fa8030b7a808b5UL, 0x0b824d66e6032fe7UL, 0x7055f8c4ddbd9932UL, 
			0xab47b7569f704e87UL, 0xc04cfd20eb1f3747UL, 0x9ee47f3313fe65d9UL, 0x3d3d8ed3194ec1b1UL, 
			0xd02f91e5d6658bfeUL, 0xd9a485492c7d7a91UL, 0x4236871a43e32b5dUL, 0x1c70b4719b078849UL, 
			0xdca757ec350be849UL, 0x22f134cd284b1026UL, 0x00a39e9944283209UL, 0xd315ecc61a35f0baUL, 
			0x7f69a5f88f9e7586UL, 0x02e5a6cfec151436UL, 0xeca46d2fd25f7f4cUL, 0x771e8a25ae8d8204UL, 
			0x659aae5df012b442UL, 0x725e0a925e97557cUL, 0x0481932ef26aa62cUL, 0x1011e2e88f134a62UL, 
			0x543e8a90fabda7e2UL, 0x52afcddf82f3cc0cUL, 0x98df1bcb37bfac95UL, 0x29cd386789730ad0UL, 
			0x4304e3397807a58cUL, 0xc184bae7655cdf11UL, 0x17d2e1e766c3e6e4UL, 0x28ce63733c19f766UL, 
			0x9504e7bdb1806084UL, 0xa5b96b1fa62b8cb3UL, 0xf76d33175711ffb4UL, 0x157016c3da03df72UL, 
			0xd67d9dd92f45a806UL, 0x3a58aa46459a146cUL, 0xc95f8aff35f3536eUL, 0xafdbc4e4faac77b8UL, 
			0x4787e5791f95b7dbUL, 0xf6ffb525adf4531eUL, 0x0b431f5ba70e7943UL, 0x2dc682bfefff0f71UL, 
			0x9e5abdcf35210960UL, 0x1112044bfa0df71aUL, 0xe84cd26ab2167731UL, 0xb6da3d160943c5c6UL, 
			0x1f7445a32424f55aUL, 0x7278e3f6a6a4e5c1UL, 0x52102db7550076b0UL, 0xc0558320d04cc6adUL, 
			0xb8bdcc2018c332ecUL, 0x36900fccc5ce8705UL, 0x340cd0f270027d1cUL, 0x047d9c6f7384c4adUL, 
			0x7422bf105c7a2381UL, 0xeb6fd40ee904025bUL, 0x3994584be5ede4d9UL, 0x4843cf7170e1da2eUL, 
			0x2b8a16106cd55756UL, 0x5a303dd6b0a244dcUL, 0x84f8e124011df720UL, 0x318bc9523e1fedfdUL, 
			0x16f8cd3ea0448566UL, 0x5abbcef63485fcbcUL, 0x25d7398f4d0898e2UL, 0x41f975123b55371dUL, 
			0x2b54e6605f3b4ab7UL, 0xff8df7c9c8392c25UL, 0x63ac5d9c82b022bcUL, 0x4ba3c37579d2de4bUL, 
			0x391c00b9c0062e68UL, 0x453cbd80d3d817fcUL, 0x03f9971659210836UL, 0xbea060b82d005bbbUL, 
			0xb2144ee0c85bb643UL, 0x6244da297b92811eUL, 0x84dcebafaab8fe77UL, 0x3033d92c90b476b8UL, 
			0x3a2d0328989be15dUL, 0xbe7f979ea40dd065UL, 0x86b4241fd691062fUL, 0x75bf85ec53c89279UL, 
			0x79d75140e643be40UL, 0x315519f6cf502d45UL, 0x3ee4ef7eb6694900UL, 0x20ca08e16cdf506aUL, 
			0x9a5d69da479592f1UL, 0x8785a9ef077d03f0UL, 0xfaa3cbf347c6aa62UL, 0x1d68d5950f26da8fUL, 
			0xf76b6e2a4957680cUL, 0xd31821fb97c5cb56UL, 0x6155d25ce7725357UL, 0xdf665514988072c3UL, 
			0x1c47c45f0031f0a3UL, 0xea4ffefc9d024c98UL, 0xd17b422986fb582bUL, 0x0bc02c7687872a41UL, 
			0x35c1ff6ba7b5570aUL, 0xdaaa1c7b8e6d9bd2UL, 0xd884ad6ee666729dUL, 0xf96b2d21aeb27f19UL, 
			0x6d6f696fa412d62eUL, 0xfb8cff2aff57eea4UL, 0x66e0fdf87acb096dUL, 0xd97e79b60ba3fee2UL, 
			0xca60bc91b38cf2adUL, 0x3ede3de16f604779UL, 0x6cbf3c7c5fca5e78UL, 0x821f8c77ee7f9159UL, 
			0xaf8afdfe6e2077abUL, 0x2f2538cd3f0c60f7UL, 0x738bad3a02f75d47UL, 0x011f2af85bf9751eUL, 
			0x7f6bbceb77537195UL, 0xe5f0ebb1c0325395UL, 0xb57eb04269e9e80aUL, 0xf476c6bf7ab22091UL, 
			0xd3ce49f690bc0281UL, 0xd4682f745a017c44UL, 0x3084ee3d033bcdefUL, 0xc0571b133155afe7UL, 
			0x1c3f8e7e1849ce7fUL, 0x0152e32c28063228UL, 0xcacbae7a5ed29128UL, 0xedf9e47d0150d9c1UL, 
			0x7b6eb94ad4fb5d57UL, 0xf73ffdf0d3f9f9b1UL, 0x73dbd29c4bbf790aUL, 0xb2b54d12154a0446UL, 
			0x57441bf053c2cce2UL, 0xaeaafbc9e029d9f7UL, 0x4b9f489363e85fd2UL, 0x3d014564803a2638UL, 
			0xc66935fd57a21ad0UL, 0x5787cbd97b196a62UL, 0xe7bd8b12eedb7996UL, 0x72c4d8f94b8f4b2eUL, 
			0xf6bea879f5398520UL, 0x4c66a479da75050fUL, 0x97913ec6d6420458UL, 0x7737bf59dfe0494eUL, 
			0x18c4ce984f53e71cUL, 0x430a752da740f8edUL, 0x6692b125cb2d22d0UL, 0xff91d21ba5d56f25UL, 
			0x622369c2ad60d467UL, 0xc1fc99dba5fad81cUL, 0xe8bf6d4d97c22b11UL, 0x5718f4e9fbaba2d3UL, 
			0x4e2056d422166b39UL, 0x0715e356125075c0UL, 0x797bfed4ac3756bcUL, 0x213c4ac1f008297fUL, 
			0xbd745ef7e54d3a79UL, 0x15a9999aea0355a8UL, 0xd3e6985565c4c9d1UL, 0xf33bb01940cec70aUL, 
			0xdf62976fc5d97901UL, 0x6199985535184793UL, 0x4b85779818eaac82UL, 0x3b7709d7ad5d131cUL, 
			0x76bdf906d7809ef4UL, 0x5e34be2a96433972UL, 0x630a54172ff6047bUL, 0x8a59d5aebaac6cb1UL, 
			0xa2aae0c04d61fc5aUL, 0x9e6caa8045bd0e25UL, 0x2e37a996eef40ed7UL, 0x1305b9fc15839adeUL, 
			0x96465623ab2cb591UL, 0x5770e8e441f83281UL, 0x7b591d761e4b43ddUL, 0xbbb7b409b713970dUL, 
			0xbf51fce1cb080888UL, 0x10ad755fed2a2b46UL, 0x63c0cd8220057209UL, 0xe91f65702f4f722cUL, 
			0xa2119cbf8f6bb3b1UL, 0x4fbd82b7cbeaa879UL, 0xb4c0a855e1b72a67UL, 0xee8fbc5d8b32e33aUL, 
			0xa243afda5d6f2834UL, 0x1bf86004d167e432UL, 0x331543a113c8219cUL, 0x47ea7641d594a604UL, 
			0xa7735e99bcd9069eUL, 0x33dfea7425fe170eUL, 0xbf7c1b25c0df71b6UL, 0xdcd41d870e19fda9UL, 
			0xd69ae13fc2ae8aacUL, 0x1be64db74deb1dc0UL, 0x9a9ca49791d60f0bUL, 0xb5e70c73b68618aeUL, 
			0xd983b967ea51100eUL, 0xb9a1f125b278919fUL, 0xbfce58ef43c976b6UL, 0xbf55ee937d20919dUL, 
			0xd621f0c562985bbeUL, 0xff83dda361f712d9UL, 0xe5d0ec657b3e3023UL, 0x697eb43bcdd1fa9cUL, 
			0xa570675148e57ed9UL, 0x445fc2d71326084aUL, 0x4a7536ac8a812837UL, 0xa17507c8cbef3d83UL, 
			0x725eb5c77328e742UL, 0x6e8efaf86ae42cd7UL, 0xd259e785c5f68dd4UL, 0xacfc245dc4cff60bUL, 
			0x0c70b64b6e549f7fUL, 0x599fe7685cd43688UL, 0x5354785bec2999a1UL, 0x9f7eeda9cd5840dbUL, 
			0x5b60c9fab6e6ad9aUL, 0x4ec50572e172a93dUL, 0x30f1df30be310001UL, 0x8fe414f489787ef9UL, 
			0x0e142624bae22d61UL, 0xd6823267a02c35eeUL, 0xbc8b24bd8da7395eUL, 0x95c1e4cc7762f93aUL, 
			0x66ab02972b67d906UL, 0x678938d7c004fa28UL, 0xcf9dc6a0fa52fa27UL, 0xec96040ca452d709UL, 
			0x87bfa0908b9e979fUL, 0x9812d1ee44d9c334UL, 0xeef4cf365ea7fd32UL, 0x1e5041e6cacd59b8UL, 
			0xfc0a08149db82e39UL, 0x635aca72e1b2d310UL, 0xf418e8a7e3d083e4UL, 0xb954724cc1717b2dUL, 
			0x9c8a2f1f78e89275UL, 0xc6f87f1618aecc40UL, 0x307a0070a179bfdcUL, 0x334862e3de547f68UL, 
			0x844b56657f1e34b5UL, 0x69621adf05026091UL, 0x52de4e744dd4b84cUL, 0x2066275fb4fb07dcUL, 
			0x1fbc5850483133bfUL, 0xecd445fa4443905eUL, 0xc323d58ee709b3dcUL, 0xd99cbd13e3924cd0UL, 
			0x8188cd131af322bcUL, 0x5cda0c49469ff9ceUL, 0xb4aead9b626ec85dUL, 0xfac086869a8eacfeUL, 
			0x69e669849136f5ccUL, 0x6476d745f15a5e08UL, 0x999e64848956b8b3UL, 0xfc871da2e09ddcedUL, 
			0xf97f68cd254b2796UL, 0xf0ece878c4fec6baUL, 0x31893ef6f9265287UL, 0x2284a6800524b613UL, 
			0x5a86ad919bc16e07UL, 0xbb6a3b8b919c592fUL, 0x39e1b8d2e1db9331UL, 0x8d14abdf5544928cUL, 
			0x3d95fec59756232eUL, 0xbeea5546dc8dba0dUL, 0xcc35e263ce43a526UL, 0x3dc0da05984c9cb6UL, 
			0x236b1d52076544beUL, 0xe58d814a6b58929aUL, 0xd07ab1c75351d90aUL, 0x50a01df4e8a167e6UL, 
			0x12b112640f4b6d35UL, 0x7951102291ef043fUL, 0xdcb8df4874aa0f31UL, 0xf9696267c7961398UL, 
			0xd577f04053e84701UL, 0x087ff9be7433ae14UL, 0x0c7533dad617a967UL, 0x3459c3260db42fafUL, 
			0xe00e75d3a2e86247UL, 0xd4822e4cd8de26a7UL, 0x91104432bdcefebdUL, 0x648954a88840126fUL, 
			0xf6004d0159efde32UL, 0xddd679ec4ef53293UL, 0xce64642551a3a28cUL, 0xf3d9ec1428518b3cUL, 
			0x39c7e836a63cbb10UL, 0x0f93c8034181be78UL, 0x3a7cf9d76ff6c32fUL, 0x7b8eec0ce3bbbfb6UL, 
			0x4c2ee1f2f296bc19UL, 0xea75c3821b3e9301UL, 0xb89b37e888e37e80UL, 0xeb6c5231bf0ab86cUL, 
			0xf556e68841f6111eUL, 0xd67e549197f20b08UL, 0x1160e7ec6aeefb32UL, 0x3fb66e51d560a952UL, 
			0x885ec10af1677050UL, 0x3b3f92169eff02e8UL, 0x5ef8e32c4d2b4dbfUL, 0xc862923920ad92cbUL, 
			0xb7ef24a9a3a5cac2UL, 0x3eb27dfb99e57cbaUL, 0x114ed4c1cd7380f0UL, 0x89823db5ce32a393UL, 
			0x233fda513f5fab9cUL, 0xf73bfe516e4a214aUL, 0xd91a89150838c947UL, 0xa66349d5b1e7d4a3UL, 
			0x8ce2459635f6fcccUL, 0x39cbc751c3f0d1e5UL, 0x98b9bcb8f85c5d97UL, 0x134df847b73f51c1UL, 
			0x60a4a8404025433fUL, 0x1d205989c1a1241cUL, 0x851d9211a04c3232UL, 0xb0b1a8f5f029d746UL, 
			0xedb40de87a3c4e87UL, 0xac1b0d27e9ed11aeUL, 0x67eecd9d8e41e89bUL, 0x5dc7ab9664f8fe6aUL, 
			0x5992afe15b4ce388UL, 0xe05c5e49b6b327bbUL, 0x73281a3f91637bd3UL, 0xed2b3c0fcb52795dUL, 
			0xa33eadbed6d9e0f7UL, 0xb0bd8a1ce227292fUL, 0x3f03a9d55db17418UL, 0x5fb5055042478a88UL, 
			0xb7cfa147eaa9d556UL, 0x763a917fc1c485aaUL, 0x56aa54f2195beb49UL, 0x52486769648d51bbUL, 
			0x09b21c2c40090f5bUL, 0x5a5768d14f33e7f4UL, 0x65f4ec118f384917UL, 0x1ce149c3f3a2307cUL, 
			0x8c971a267513fb1fUL, 0xf14e9f72a2273781UL, 0x40fbc3a8dea61d6bUL, 0xdb7f2cd0e1ce3991UL, 
			0x6178cf1654f642e2UL, 0x1a22a90948a18b5dUL, 0x58c0e9f5c92f1043UL, 0xe22c69d5aec6fbc6UL, 
			0x278c64aa3712c499UL, 0x25db315658f9d8e6UL, 0x17255d96dcaadbb6UL, 0x283046a6ffc834e3UL, 
			0xdbcf54f33e25e3d5UL, 0xa78442a671b72c26UL, 0x9eada90674a35f33UL, 0xf903276441912e39UL, 
			0x9b5927f39a619168UL, 0x37658c5d9305aa52UL, 0x8b3df67378c1f680UL, 0xc60ec3f62bf4fcfdUL, 
			0x1f1d1a3cca1d0c19UL, 0x660a60ae163e5e93UL, 0x6af33451ba27489eUL, 0xefd288d20b29c2c9UL, 
			0xf3742dd51f110a22UL, 0x374f563a2a50c3b0UL, 0x2bd35f24956111a0UL, 0x0d724b26f20f1637UL, 
			0x0c05ff4c397680bbUL, 0xb81888122e3d0a45UL, 0x55363384d1800c10UL, 0xa1da3f9b9b4a5ecaUL, 
			0xf6837bb86870f7c3UL, 0x385940ec9d313290UL, 0xfdbc5fe0e6b6e15cUL, 0x8b20ab2eeb4b97f0UL, 
			0xa171083f70855213UL, 0x80e81e0f72d73e85UL, 0x8d77b07d1c5a2a1dUL, 0xa6c36f05736f1b68UL, 
			0xe1a83c554f39008fUL, 0x44d9e4876ebe4edfUL, 0x4602de2e2b90b978UL, 0x3a264f370c70fc2cUL, 
			0x92420567d5be1c9bUL, 0x1af52afcd3fa8ad1UL, 0x128cafa4061684c0UL, 0x602c796a425ebbddUL, 
			0xa397e77c5f1dd8b9UL, 0x7f7b960fa9f11002UL, 0xe3829b35748fbe03UL, 0xf1b54aa652617e97UL, 
			0xc607d5c7ebfeb118UL, 0x1e414fc38d7693e1UL, 0x83a781aae8e4f3c8UL, 0x2cf26ab66e6d129dUL, 
			0x04737dc269ba4493UL, 0xd70949a3576d8abcUL, 0x3b7ef1249252435bUL, 0x4666a94642561c2dUL, 
			0x999e38be7b84c84fUL, 0x3bf4cdf5360518beUL, 0x07c0bf3f3232d1faUL, 0x8525be56e37115deUL, 
			0x15a8c75117d17dabUL, 0x6221e5fb3dea6b11UL, 0x9ad4407547a02415UL, 0x0803fa1c61941546UL, 
			0x133146580bf2e8d9UL, 0x7698b02eb89b130dUL, 0x37c482d2ad48e94aUL, 0xc68be0956c8d883bUL, 
			0x6c0c75cbf5bd14edUL, 0x9f82d76229bb8460UL, 0x2c2ba460d1c7f333UL, 0xef85c8ffe5617c10UL, 
			0xfd7a47873ce40fcbUL, 0x39cc06fffefef689UL, 0xd5813da81ba967c7UL, 0x2c6f0f0e9457ec24UL, 
			0x8153fb547b6f58e7UL, 0xdf8cac16b7a4bcf7UL, 0x1a9a8e7859a9d8aaUL, 0x989cd07e5075fc1cUL, 
			0xaaeaed76cc4f1a3eUL, 0x1a43ac5422f82b5eUL, 0x243dc06848f595a4UL, 0x4f73c2a50e8b5d3eUL, 
			0x49f83076dac34971UL, 0xef91f8221d3c809fUL, 0xab6b6d3fdf35a477UL, 0x7cc81fb22d23766eUL, 
			0xaa4b75ad34122539UL, 0x6fbde2db7c73cd50UL, 0xf9d7f00fe34a3c5aUL, 0x3d06d5184a66fc25UL, 
			0x1270870964e34aa5UL, 0xf10a42fdd0a110a3UL, 0x720c4e760135f88fUL, 0x3e19b85234b87899UL, 
			0xbc93a9249ced8d8fUL, 0x6f2cd182f86b103fUL, 0x76e9f5d1ad7042f5UL, 0x1bf554c8c2ec883bUL, 
			0xda1fde6105af51d8UL, 0x186d9b253b88fd80UL, 0xded18d4003074a40UL, 0xaa12439003230f30UL, 
			0xd80c6a1b50951c43UL, 0x30f62e72b031a294UL, 0xc972bbf026518596UL, 0xc107876e726c0ad4UL, 
			0x497848b98b72e3e0UL, 0xe48c0581db1f1afaUL, 0x26dfdc2040559d2fUL, 0x26505e589d1d5a7bUL, 
			0xe3feff13e193526eUL, 0x47b4df58689aa932UL, 0x74128061c89bd587UL, 0xd09fa61b2665eafbUL, 
			0x5b2b57f47327e551UL, 0x77c39032f5cb0e3dUL, 0x4689a6990561fe37UL, 0x70a16bfe4d456c00UL, 
			0x6c2ea99e5d9805daUL, 0x3dac55c537e5d225UL, 0xc5d2f183ae1877faUL, 0xc9d74b723a6f0abaUL, 
			0x512bfe1b088d73bbUL, 0x816655d1de756dd7UL, 0x9fede2e9faca5d37UL, 0x078a8b919b363f44UL, 
			0x5f8d35f170785b0cUL, 0x54d4824159163ca8UL, 0xe95d23e03b7a1584UL, 0x4be15b017968e805UL, 
			0xddf24fda4cc5ca07UL, 0xc19925865498692bUL, 0x3d2d64e4bce9a292UL, 0x460ca62f498dd109UL, 
			0xd141090d42d1b20fUL, 0xa44fe44e8feabd84UL, 0x53633a209f65966aUL, 0x37ef65118ba1ee75UL, 
			0x7f1c3f7403862fe2UL, 0xc867d14442a1b906UL, 0xb51c38b711704992UL, 0x6ae3d4cc34b1054bUL, 
			0xd94406a5a1f9a02bUL, 0xbfd5bd7a5d593e4bUL, 0xe63b742400a104c8UL, 0xbd028dac823a8a4eUL, 
			0x2728736ffca719d3UL, 0x8a52261d74f71d5bUL, 0x65737dbb5f6eb469UL, 0xdb47e62b3fd46855UL, 
			0x928be6098ec1a2d2UL, 0x485354c19289239fUL, 0xc13ef923a46b9c17UL, 0xd0aac54ebbed3f94UL, 
			0x2a3b439f5c594ac6UL, 0xf2042a894cf9f19fUL, 0x0a75ee29a555c9d5UL, 0xf46bf6c0043e7702UL, 
			0x03e5941aa137a3f6UL, 0x630c28921819b2b7UL, 0x5be6e2693bb47c96UL, 0x2d6eb4ef71205504UL, 
			0x9b9d1afef69c6f26UL, 0x52a1b897f8edc076UL, 0x02d6ede1491a5d18UL, 0x713ee907048e9ed5UL, 
			0x8190313ef3400044UL, 0x6641b44b3c9eb900UL, 0xe2f17cdbc8c5dca4UL, 0x4798a295cf0eaa7aUL, 
			0xbb40f9ef19947eb2UL, 0xdc2c9d7d34e52d59UL, 0x9a364aca61798b77UL, 0x72e6380f145962e5UL, 
			0xe7aeac31226da136UL, 0x121c1f429be440ecUL, 0xa75bb2ef82acdf68UL, 0x288b8e632059dd2dUL, 
			0xf721ad2148ef9ed3UL, 0x02ba26cf2166260eUL, 0x915613c0d054a954UL, 0xfaa8ff5eff4fd2efUL, 
			0x3d9a1190a306604fUL, 0x727898cf602c0310UL, 0x4cdbea1714a7b1b0UL, 0x7bc637cd58d6bc6fUL, 
			0xd2e49a87d3012093UL, 0xa1a0ccdeb9c4d5e1UL, 0xe63f97f50bfcdbedUL, 0x0e6ce74761c68244UL, 
			0xa8e318831b2d3bc2UL, 0xd14641edd142e00bUL, 0x0be0daeb2e026af6UL, 0xebd912e6de7f5130UL, 
			0x71923d9fa6305d5fUL, 0x291b1a002c6c77aaUL, 0x9d9bc7a66589ef0eUL, 0xf1676ed9153c94d8UL, 
			0x65d477995b2579dcUL, 0x6ed41814356df104UL, 0x059b6c38e52dae18UL, 0xe868ab0340ba751dUL, 
			0x387d819d46d3313fUL, 0xdb89fcac7787b218UL, 0xf87846da181c3536UL, 0x638c7605f0ead8e1UL, 
			0x859f5e32a7fc80d6UL, 0xb80737e89085d9d4UL, 0xc26491f693d9da1fUL, 0xea22a300df4f8bcaUL, 
			0x6659d1a5a07b8c7cUL, 0x95eb5979ded1efacUL, 0x677cdd257a66845cUL, 0xc9eccfda016288f8UL, 
			0x4c03584782ff68d4UL, 0x931b76b00f9512dcUL, 0x072f3d666025d538UL, 0xeea17fa3520327d3UL, 
			0x25fd82e1a7d0aa27UL, 0x1758c0700eb14edfUL, 0xbdb9b9b638b2afe1UL, 0xfe8fb6312caf43f9UL, 
			0x8e4ad1bd74cc67d2UL, 0xb44127358e76e609UL, 0xb45fd3a6fb789147UL, 0x03cae27f8d72dc1dUL, 
			0x220cc57db6291460UL, 0x65874bcf83f24a0eUL, 0x6ef7b36fbba49045UL, 0x2761d492ecf2953dUL, 
			0xe245a3b38f074effUL, 0xf63535429ebf5bbeUL, 0x83e5374944d626a3UL, 0x857216e58ed9db44UL, 
			0xde627c0baa74769fUL, 0x1ebc638f9fabd154UL, 0xb33b4dd0c15fd159UL, 0x6bc0784027dfe91bUL, 
			0x404704d81f9cdc28UL, 0x1ae02eac95e37ef8UL, 0xbdcd361128678c11UL, 0x5701844ae298538bUL, 
			0xcd303c0d579d509bUL, 0x37b6a3739bebd334UL, 0x3768cee3914fe5aeUL, 0x02b73ad14c8ef205UL, 
			0xf43bf85e8c0e45a9UL, 0x3db9714bcc4e771eUL, 0x082f773c6611dcbfUL, 0x79f42ae04524ff56UL, 
			0x2c9218c10eadab45UL, 0xb1042dd8c9d4d68bUL, 0x82708946f760b11aUL, 0x7e3b9c39ce4c6b60UL, 
			0x2928f684d13a72fdUL, 0x1c5cb9dde81e6d30UL, 0x3fb6382113af5819UL, 0xe64d830418cf1e10UL, 
			0x66412be99b4a54baUL, 0x44c576b69b3b58e6UL, 0x4eed96b3be769dd8UL, 0x92abc61ba612e231UL, 
			0x2c2c6405e1ab87c2UL, 0xfda923c19cdba108UL, 0x5ef2296abfa9c516UL, 0x6a6d13372553bd1eUL, 
			0x859095b4190ba449UL, 0xfad09d0c026c99a9UL, 0x05a05bfa3cf62344UL, 0xaa56fd719f1e2292UL, 
			0x6d0fc525d5db7eeeUL, 0x1912df1e0914a306UL, 0x3b41aadfab4d1a33UL, 0xde9dcd4f9001553cUL, 
			0x3a6a3189f54f76f4UL, 0xe59f77a5cb87f001UL, 0xf582d190eee107bfUL, 0x486b2b9385c54a30UL, 
			0x49fe0809718805a5UL, 0xe221b3fa0fa48f24UL, 0xbe7d2ad619bc73e3UL, 0x93a46180073ce4feUL, 
			0xd1f8e331dd00b111UL, 0x4c451d6190357e8cUL, 0xb50d66b45f99ac1dUL, 0x5608fb8d671018a4UL, 
			0xa7c9beb4ce29c704UL, 0xe404de135c5b1bd0UL, 0x9c7e08d6a56f5b71UL, 0x21441d54ffa5eeb5UL, 
			0xfb1d6376ab1e32e4UL, 0x2a724671a1f41378UL, 0xc86fb64edcbee2abUL, 0xb2bce7689e153aafUL, 
			0x878a62b247323d86UL, 0x5cdf64da1673d14cUL, 0x360a94d43b19a957UL, 0x3e2be19602d5dc18UL, 
			0x3d04867b53a78352UL, 0x4dc57e8e69a73278UL, 0xcfda225def144101UL, 0x06e187034452ac78UL, 
			0xf9589fa4e08e7843UL, 0xba594a21f18758a1UL, 0x9e659455ced32523UL, 0x394afbc5abb2f21aUL, 
			0xe7a8386c7e7bd2a1UL, 0x694fd2b7dad8730bUL, 0x0b3d49f2f2c3f029UL, 0x2e6e67ff68513e92UL, 
			0x6d3aacc78143285fUL, 0x3891c7eac95eecddUL, 0x18361f8ada9bcbccUL, 0x8df1fdb269b3ec0dUL, 
			0xa717aa10d5313d80UL, 0x971d07062fa613d6UL, 0xb35e740585cc629aUL, 0x8948e6fca4cd3371UL, 
			0x6faa50681d41be98UL, 0x5eaba92c155ab93eUL, 0x2ba5c66d20175889UL, 0x1712899dd8a9a5a3UL, 
			0x1a3a61349c90833eUL, 0x726746a066a90018UL, 0xa6782a6530498227UL, 0xf1234c79db73e8a4UL, 
			0xa39ac6badb2e5f96UL, 0xffca0eecbc9816edUL, 0xe6d4f0ea0f701c43UL, 0x3fa81a89c6ff98d0UL, 
			0x3a3000a0891c4cbdUL, 0xf8f62139ce35685cUL, 0x25effd11ab50a7e6UL, 0x147a58d7e6fd9d32UL, 
			0xb931552d7e7b59c4UL, 0xaf4cff481e8a11feUL, 0x722a32fbd6924accUL, 0xec5c1488445ce3feUL, 
			0x822b987fe2e76e30UL, 0x76507d96a48f2d23UL, 0x08392011a537d899UL, 0x66b5b7df856db982UL, 
			0x84ea80751b0fff4cUL, 0x050413871edadac9UL, 0x1d496dc2f06ce004UL, 0x5c5c45253b9e1b1bUL, 
			0x26b6238a0615bb9cUL, 0xafa46ff4427b2bfdUL, 0xfd99aa3889353f08UL, 0x6785079b3df3901bUL, 
			0x51835bc39326285bUL, 0x69aaa3ad7923af6fUL, 0x6fd0d546261a8751UL, 0x2094322c1092f96eUL, 
			0xacbcdfc44b7ca610UL, 0xd97d4ccf29bf210cUL, 0x4edc66c9740945e2UL, 0x7c32d68789e6b798UL, 
			0x7b560e2c2c2c1388UL, 0x006042e0b8d48f6aUL, 0xd49bac169503f2a4UL, 0xdd63cd9f36a3b70bUL, 
			0x00109cbaf29287c6UL, 0x9df7cda4edac5250UL, 0x2568e58c4e52508dUL, 0xbe1f5732c57423b6UL, 
			0xd4546ea16aaf3409UL, 0x382f6a65064bddbdUL, 0x6202d06126899942UL, 0x795b89c4701cfc9fUL, 
			0xc8580d651a63b896UL, 0xcbe166d86f929a08UL, 0x147d47242549d507UL, 0x485b72ff45a6584bUL, 
			0x760e21880faf300cUL, 0xf8e1ffcf0de4380cUL, 0x2b006c92089fcd5aUL, 0x8f810aa2c66f92ceUL, 
			0x849bfd63c05f712cUL, 0xad5e63662a2c8290UL, 0x7dbe2980f60a7bfdUL, 0x773c2f64ecfb6107UL, 
			0xbe65bd43e158ae35UL, 0xec2518e3f59f8125UL, 0x4206c8197ecceb15UL, 0x8aff81313611af56UL, 
			0x0cab262f31cbdb44UL, 0x88c4a34c2aa2c8edUL, 0x5ae9613409e05bb8UL, 0x6cb787100e3a98e1UL, 
			0xcd52ff1288769d27UL, 0x0c49bef2577a841bUL, 0xc58a4340b632fe50UL, 0x515d2a86d2cc333bUL, 
			0x8b615be25cf7e1e2UL, 0x41b55d6788938b7eUL, 0xfdb8b395ec9328f9UL, 0xf0ceab026569b18aUL, 
			0x59a53b39886ea4bbUL, 0x838e86bd839f2a68UL, 0x3860e00d42a4b456UL, 0x57a7cdee9bf4fc75UL, 
			0x35256d3909bae12bUL, 0x8cd9a723578c8a71UL, 0xf393ca53fc0e2cd5UL, 0xd1233ed4cce189feUL, 
			0x2d883980a1a6754fUL, 0x9810f17aa1ad8bacUL, 0xe66e309ac6659328UL, 0x9b88e07085445b33UL, 
			0x3acbe64b120b65e9UL, 0x974c9f65140b23abUL, 0x186641d2f265b376UL, 0x626038715409983dUL, 
			0xd5ac75d4dc6eaa5fUL, 0xc5d3b3b83b49b386UL, 0x5b19d109acc205cdUL, 0x9b8b0e7040fc3fd6UL, 
			0xa0de0107836525e7UL, 0xc6a9886def3bce1bUL, 0x78c289517d6d4222UL, 0xa9737336722d5a54UL, 
			0xa0db2edc8cb0888bUL, 0x3555d834dd9c37ccUL, 0x57495d039629478aUL, 0x9d4430d615506e23UL, 
			0x4c92d49302c76091UL, 0x55d4cadc35f154d7UL, 0xd3a912acf7109594UL, 0x0169cab888839289UL, 
			0x2f865b9b81ba08f6UL, 0x195ede0afd463012UL, 0x154561b3bca43865UL, 0x4befda08a1509cc7UL, 
			0x77cc8ec0e16382f3UL, 0x1e31cdfb8423dad7UL, 0x99c6d6f75eeea8f0UL, 0x3e8b8256d998efc9UL, 
			0x44e897e7081e7653UL, 0xe1321cac51ed7cfaUL, 0xfaa5c12b9b97e86fUL, 0x444d80b09afa77a0UL, 
			0xf6ea8951df99d630UL, 0xfc19f993145e811bUL, 0x488dedc8bedcdf7bUL, 0xbc7c32b10c290cccUL, 
			0x15e4d0782f137159UL, 0x82e5ada7ba1441cdUL, 0x7c8b51622eddef57UL, 0x0cf3e9e2d76b546bUL, 
			0xa1e0140f89a048bcUL, 0xfc891757cce1b434UL, 0x84cfbb1bd23bc9d9UL, 0x7f9baf94dcfa99ceUL, 
			0xe24c5e7596af1babUL, 0x5c6682b1a4279891UL, 0xcf86eda4bbd9422bUL, 0x3af40c2b5a6593f8UL, 
			0x3a96e36062aec448UL, 0x41b905c7ab32ce10UL, 0x9facbbe7f20ff7fcUL, 0xec418fbadcf0c055UL, 
			0xb9d1b8e0fe5f46e2UL, 0x079632e8e3acfbf3UL, 0xeba7c8b13b5a60abUL, 0x84459c5ca19fc8ebUL, 
			0xe9ba32bd41a9c6f4UL, 0x60d025fa630e7bd2UL, 0x5c5eb95f1d0511d6UL, 0x145b82e5f07b0d60UL, 
			0x25afd70b4aa52e50UL, 0xb0ac9b8dad3d0418UL, 0x4c10a1b2868b3f93UL, 0xa1a06bee307864b5UL, 
			0x0b35fa161cc5c91fUL, 0xc5c52b88e5e5e80fUL, 0x4a3c75da80544189UL, 0x6f58e759c531c0d3UL, 
			0x94daba5f0ba9880aUL, 0x89776e085f31a55eUL, 0xf9efc74d7c9a96e8UL, 0x7ad6e50c955f30f0UL, 
			0x9e85b7468d05b51cUL, 0x2387593aa2cc4010UL, 0x5bc726722e7938beUL, 0xf47049c8ca874c07UL, 
			0xc4308bc61bea9e25UL, 0xbbbffd0510b1023aUL, 0xac6b9036304f0c3eUL, 0x5aebd08504863b4eUL, 
			0x93cc2fd0903e2942UL, 0xc63f80a1e6aa23c0UL, 0x2d962b780b260e31UL, 0x96c6859debeeb515UL, 
			0x2980f48302140777UL, 0x62257771c2d43ef1UL, 0x3e18a15a26c080cbUL, 0xbe63c7fc2fbe29b0UL, 
			0xe2cdae31e6fe4815UL, 0x91d601257b3c26bfUL, 0x887911337c2fd41cUL, 0x254bbd629c30c8f1UL, 
			0x8f4b9489638cf93dUL, 0x41d844a3ba84183aUL, 0x1291f4b33e0feb44UL, 0xff438ec086c08833UL, 
			0xc5830c2330447387UL, 0x61169cf04a2bf75eUL, 0xb44935573d252104UL, 0xe93d6997c343b6d9UL, 
			0x8f7d6a3d15dadf67UL, 0x573702416af4c1b7UL, 0x72612670d05c6612UL, 0x1ce05be697e98be5UL, 
			0x989a3cb3ebdff0b5UL, 0xabdb0d6db232fd5dUL, 0x437c4a97263a4a87UL, 0xb84f377e96ad45b5UL, 
			0xbd6cceadab7c0912UL, 0xda3186ba692d8440UL, 0x34bda18eaad32841UL, 0xb6fbf2076c3a02a5UL, 
			0x48efd578ce97c3e5UL, 0x85bae3b6c817b64aUL, 0xe18564fc6d3283c3UL, 0x0337e8e685c544e8UL, 
			0x5bfa07c8b155cb0eUL, 0xf8754a40c0f50b0bUL, 0xfce902ca61872a95UL, 0x3d2832a809421303UL, 
			0x399ac9b53c38817dUL, 0xd920baf26374ad81UL, 0x93f8c3cfff34c1f2UL, 0x92277a174b9775afUL, 
			0x158408dcac104567UL, 0x705574261ad294a1UL, 0xf4d9d83ad5a201d3UL, 0x26e1413f99f8592bUL, 
			0x6c9808bc1983845cUL, 0x5c9be9e7d1a0af64UL, 0x91617e3f0c0ea8fbUL, 0x8b023e27d0e71ba8UL, 
			0x26331295379d8523UL, 0x30842685cd242af2UL, 0x7b30ed759ec4ddc1UL, 0x3742ba4e32fcd693UL, 
			0xcf164019b572145bUL, 0x4295a901582f9a5dUL, 0x77290d452d538f3dUL, 0xc1258276dc063802UL, 
			0x0860a6cd596717f1UL, 0x7a2cd84a9973b2c0UL, 0xbb2eff46f48785b4UL, 0xa7d540e2f87377b8UL, 
			0xff6b24ca2830a332UL, 0x70994f2a7b126554UL, 0x49c2b382b49cef09UL, 0x0f2ad6fc6ff9dac4UL, 
			0xfda776e6296deb02UL, 0x72b5f4c30162b9daUL, 0x99402523f94d9458UL, 0x34782f31903e3990UL, 
			0xbb8a114778da3b75UL, 0x8ec256d3767e20eaUL, 0x8e4cf8b282a623daUL, 0xc848a1e9fc816fa1UL, 
			0x0058ccc0781452adUL, 0x7632c0cef19a8dddUL, 0x2dc57a51365a1b0bUL, 0x1a5b2ce6d24275a4UL, 
			0xc346ebc8fd715cb6UL, 0x58653dab02ffeda2UL, 0x8819372061d872c1UL, 0x015cfc610ba87da6UL, 
			0x10452bf1dab022edUL, 0x95f574475ba09071UL, 0xe6ff1040c44ece08UL, 0xbf4d4f388eb9e54cUL, 
			0x0e6565ae3a846c48UL, 0xd85c938b509d7b88UL, 0x0b618834a5d72383UL, 0xa066d8d6cc27099cUL, 
			0x61885d9386b19926UL, 0x6436eb3ba10fb9d4UL, 0x5d3c257da2cbd286UL, 0x4ed454a7b56768a7UL, 
			0x43fdf710114d261eUL, 0x20ec36936b2be3e5UL, 0xa50b68bf9e6947f9UL, 0x72cdd1d9cce5fb4bUL, 
			0x91dde2b0a76d339fUL, 0x4fef45a4f7c3a9c4UL, 0xa498df73b468f2faUL, 0x7cb7ea08fab1814cUL, 
			0x64c9b3c16372b0a8UL, 0x94c141fae2cfbe76UL, 0xd0f13b1dc61328f0UL, 0xad5454957649dd32UL, 
			0x561e7fe2ef92e094UL, 0x6eebfb0e3dd40b5aUL, 0x4269c0546bbe3018UL, 0xd0a20ef2aaeb96c4UL, 
			0x72da1eb197d37443UL, 0x9d7f02f345161af3UL, 0x1151b1cce83f159fUL, 0x11499d34749ca663UL, 
			0xc387abcacc12e2d8UL, 0x64b44d4574a9c9e8UL, 0xd9abc33f58b1ab7cUL, 0x4c4a69b993dc970fUL, 
			0x70ea5c9272e3c43aUL, 0x06bc581ec3aab3ffUL, 0xb6cedb750aaecb3aUL, 0xeb9c59954285c3acUL, 
			0x5216b9ac8d78210fUL, 0x552ad5e82bee8fcbUL, 0x559315d75b8cfeb4UL, 0x937945a63413b0c3UL, 
			0x4a12d305a4da3ac2UL, 0x25a1fd56b710591bUL, 0xfddd829a837cdcf8UL, 0x23e65686f4abd154UL, 
			0x3a74a636191ff865UL, 0xbbc58cc6004d8be5UL, 0xff813f935d554e3eUL, 0x8f29f1b665c6b1dfUL, 
			0xf3e4c1d6a88c2432UL, 0xa1bd44b9124f6f9bUL, 0x7466fe1007d13dc5UL, 0x466508c3f522ff66UL, 
			0x13276c5299645b4fUL, 0x71aa5202402fe2aeUL, 0x8080a5f2f5845f81UL, 0x60324c73e9fa48d2UL, 
			0xda7a8240ddb6fb97UL, 0xcbc3f215fd0e2ffaUL, 0x237a0fb4b093622dUL, 0x2f8657911443f14aUL, 
			0x74ecf320b7ea57e3UL, 0x89b7c0ae52a05dd2UL, 0xe28ea71097890230UL, 0x71d476626d742c5dUL, 
			0xb7a78f03241688bdUL, 0x09e6bbd2fc2a0f41UL, 0xbb654b4437da2ac4UL, 0xafb865a480428643UL, 
			0x40fc425537626523UL, 0x874f0b7d03d51db3UL, 0xb5f365d084d1745fUL, 0x9be24aac3b25c896UL, 
			0xefaccd364d278f1eUL, 0x768e8828ee1de57dUL, 0x345ca4d99d193463UL, 0xd789cc8e387f5a41UL, 
			0x94c42d5d4b4fc6ddUL, 0xab90492960501d09UL, 0xeadea0f8ce7ece3dUL, 0x69659208d86b60d3UL, 
			0xf5383636baaf8145UL, 0x93fdae5bb54e3ab0UL, 0x9f9e587a110a0e91UL, 0xe5dd3853cbda9533UL, 
			0xcb63f632d6bf3599UL, 0xf169fc6264768d59UL, 0xc65a2a133defc32eUL, 0x35eec7bcc7b04b6cUL, 
			0xb8fcdfeb23e35ef1UL, 0xc850fb7f44e6ecb4UL, 0xccfc13d6c246c735UL, 0x326d8df70f618b0fUL, 
			0xac199396be30bf64UL, 0x4b415e1b748cf590UL, 0xd160f2ae2eca2c53UL, 0xe2635c4a5559c556UL, 
			0x3410e6eb731298bbUL, 0x4c701163ce6a16bdUL, 0x57d83d372234ad1cUL, 0xe80456b1fa31623fUL, 
			0x0a09a8218855e7feUL, 0xa50135440cadc9a9UL, 0xb8ef12f620e22d6eUL, 0xc4f73a41b23c949cUL, 
			0x95aaa793eba23ae1UL, 0x6c369ae34f907ab2UL, 0xd2d1b434e8ee4331UL, 0x305d80b3b284f6c9UL, 
			0x19a07600216a7d25UL, 0x0046ad4d63db70c9UL, 0x40ee8397a08d0339UL, 0xb51147590795e3ccUL, 
			0xce7ea8694137bb37UL, 0xd5a6e61984c5af5dUL, 0x51ab1c48f9f09e59UL, 0x26d37bf9ed26dcfaUL, 
			0xfd190d491ddaa359UL, 0xb0ea158507165fe6UL, 0x25affbddd1e00395UL, 0xa72e83777b5ae76dUL, 
			0xab089c1322a234ecUL, 0x0db4f95e96a760bbUL, 0x82fa2d67822bafa9UL, 0x722709bb3f1c1df5UL, 
			0xf6618b212842a59bUL, 0x734459881c626d8aUL, 0x8a26c32b4a3ba9d6UL, 0xabdcc16714ec1191UL, 
			0x2717a62ad329b94cUL, 0xaf31b8fab081d732UL, 0x7ed07527290b148aUL, 0x9935b7952f9f8949UL, 
			0x4573f59bf297eb11UL, 0xd0717b3ed734ca68UL, 0x2932d8f22c18d34bUL, 0xe8adb901b00af9bcUL, 
			0x373aad563617d375UL, 0x4521001a281b2ec3UL, 0x2c3b5168f852708eUL, 0x97a5548508c9d4c2UL, 
			0xbce842da5ad2c002UL, 0xcf72b86644784569UL, 0x8947fa957856c8b4UL, 0x4e3fdf11c2cc1e89UL, 
			0x2d510b13467118a9UL, 0x926715e76c739bc4UL, 0xade3ea1267cbbd3cUL, 0x4620b16326f0ef4eUL, 
			0x4e204a8ac5ba8a5fUL, 0xe5acfcba685e8e43UL, 0xf2653f7f214d659eUL, 0x8eac86e925d1d8d8UL, 
			0xe63afe4e672ebd3bUL, 0xe76ee5b9f1a294b6UL, 0xfece2ceaf067326cUL, 0xa733c073d7678bc8UL, 
			0xc73c7f541f956fdcUL, 0x46a14af6e1a6b967UL, 0x7a43db0d0c9cd7adUL, 0xbf633846e70fe134UL, 
			0xb0ba5b265570e868UL, 0x537014042831f6caUL, 0xefdbd19adec29ad3UL, 0x980fe3384c713c96UL, 
			0x948dae2ed8b3eec5UL, 0x2eded4939593388cUL, 0xf9617a7bae8576dfUL, 0x1647974d261f6c05UL, 
			0x708118deba6538c2UL, 0xa9deac0a0432fda2UL, 0x48b3c632c1fd0d75UL, 0x4e96f7ec7be80182UL, 
			0x902755e406385d43UL, 0xa304e390029a89e9UL, 0x3caa6cf9f08a28f5UL, 0x4f3c661c8a9bd9c7UL, 
			0x60a0fac49c7197e4UL, 0x07b1295993abd35bUL, 0xb19c9dadf83b6e6eUL, 0xa3839237345de0aeUL, 
			0x254e3566c4d79f56UL, 0xfde3ed3dbf542f72UL, 0x708177d17eb13321UL, 0xcf81896aa2a13053UL, 
			0xe49edcd32f101d91UL, 0xa1c02b446b835b26UL, 0x2b50afa0f5eac9e0UL, 0x1b23f05ac873dd62UL, 
			0x2aed3d9f7857671eUL, 0xb1e652b634b7bcf1UL, 0xba05c082f3aeecb3UL, 0xb387114b7294fc5cUL, 
			0xb4f2278f730509f5UL, 0xe76b760d555a5364UL, 0x8bffcc95a4deb9e0UL, 0x8a26c5cd2fd9e327UL, 
			0x178a4bb1052da115UL, 0xd0d52ff73621eda6UL, 0xa68d452309511e1bUL, 0x7ce4fbcd89f2c5b5UL, 
			0x9cf1d97af61ab9ebUL, 0x0c3305beee24773dUL, 0xf12cf8ad05230de3UL, 0x7a1a48799cfd6b7dUL, 
			0xded4873b21ee9d40UL, 0x94c2dc80a690896eUL, 0x45f130f395555d59UL, 0xbd655f8db0545c37UL, 
			0xd6af7b8a19c04040UL, 0xe1cefd44115a61e1UL, 0x47a4daeaa69d438cUL, 0x5afe3d9d71a7a0ecUL, 
			0x8e8271d14250e015UL, 0x2200026a92a5977eUL, 0xf043f576d43661f9UL, 0xab51b1fdb20c1955UL, 
			0x9a8460b53415fb27UL, 0xb643c1e77e7e2055UL, 0x05d4fe513edf5a37UL, 0xff9768632ac1fcfdUL, 
			0x3a7623f228e03f99UL, 0xacac2d89770ae905UL, 0x4e8a1370e3698700UL, 0xc18e6d9667c28a57UL, 
			0xc8f92e1976343e1aUL, 0x81e401b48f861a89UL, 0x874da7b3a2a54816UL, 0xa5645e9c4068fe1dUL, 
			0x6efadcec078073d0UL, 0xb9237c97dcec2d93UL, 0xbca4e2a0a4236fd3UL, 0xd6f5e3e3d47e9bd9UL, 
			0xc149a433ae25587dUL, 0xf72e04cfdcf6f220UL, 0x4b3d64da3bbf0c18UL, 0xbe9ba0730b11931eUL, 
			0x842e25d0a0e23607UL, 0x53bf70a63abd3fffUL, 0x99db73bc8ea25b49UL, 0x4fac9954f87cd957UL, 
			0xda6c59a537a21d97UL, 0x69546a6ee8e8f5e3UL, 0x0d735740ff3b3bcdUL, 0x80e66c2e54196cb0UL, 
			0x81824a47da093febUL, 0x262fd61bddcd81ffUL, 0x8d8f80b670f6c3f3UL, 0x5988562e42661df7UL, 
			0x667b2b6dbbfc3db3UL, 0x6025db57c180cc48UL, 0xa48b31e2dba10a32UL, 0x151317abca9f8048UL, 
			0x103c2685483486d4UL, 0xd51e9b7a84703d30UL, 0xc5630aa1f78e57a8UL, 0x9263fb19b7ae9b2aUL, 
			0x3b2a593178579bb8UL, 0xd622f265c525f7bdUL, 0x56659534845d8795UL, 0x81fec817b201caa7UL, 
			0x46275b423ccbe820UL, 0xd05f7e0f7fb39268UL, 0x5a3d5d1a4ef994a7UL, 0x7bd3299e9e6ef011UL, 
			0xe79d1f975513a97fUL, 0xa9fcc7e2ab3bd36cUL, 0x16061435cbbf7475UL, 0x57bef46d124650a6UL, 
			0x94b8e132904230e7UL, 0x50af1fbd69bb0147UL, 0x7aa76967dda3cefcUL, 0x278c31269ec635cdUL, 
			0x7993f5694a3c9c2aUL, 0x9d8b120407c1753dUL, 0x57562c0af0724040UL, 0xb246ea1fd8b627c8UL, 
			0x2fa53c300ee3b8ceUL, 0x1da617faebabdd96UL, 0xd9bd84f54ec7424cUL, 0x9f6ef080d56d9150UL, 
			0x6954d294cb5c10c0UL, 0x422731332744be71UL, 0x97ba8a222fd2f86aUL, 0x0731f725129fdbcbUL, 
			0xe396585627d62c12UL, 0x79486eb631d9c66fUL, 0x6bba86aac233942dUL, 0xca119cb1ed1416c7UL, 
			0x3d8092f1fd277253UL, 0xbc7ba599b35eff08UL, 0xa7b1709e7d504228UL, 0xb9284599937f99f5UL, 
			0x2f660dfaee6336b8UL, 0xee65c2be5c43e8e4UL, 0xb28d22c9d38132f4UL, 0xbebfbd8c0a9090a1UL, 
			0xabf7077fab3cdcdeUL, 0xee7cd1aed4237e52UL, 0xe8272b8c53150f68UL, 0x590dc572ca484895UL, 
			0xb31f83bbf8535c23UL, 0x7424d879060829cdUL, 0x456c8789f4a5879bUL, 0xd4c18fe9c3f2e1c1UL, 
			0x5aba7e0b9ec702d8UL, 0x5d75ef28712817c6UL, 0xc2bba37deeacd613UL, 0xb42e406ef216f603UL, 
			0x691d08fe213f5713UL, 0x1f8540ef00f69144UL, 0xefd6f51236f45279UL, 0x882ddce8da658e19UL, 
			0x9f630618059fb9a2UL, 0x964d0c44a5b12674UL, 0xca73fe0e2cc62fe6UL, 0xcad8c3786307b773UL, 
			0x4ea467b8508cd49fUL, 0xc1f29535da8cc639UL, 0xddcd73cad4b9700aUL, 0x3bee33f4068714f6UL, 
			0xd55aee3becdb2d39UL, 0xf6689d84844945b2UL, 0x3a29ebefdd1cad9dUL, 0xe9439f86703cc318UL, 
			0xaa85f81773f4db3fUL, 0x2f56c767b503122bUL, 0xfb57f1e976a86a7dUL, 0x912607651476c4fdUL, 
			0xf0fd2cd626ffe9b3UL, 0x6890a90bb333c51dUL, 0x52d04318cb6fdc37UL, 0xf47086e2e97b1e8cUL, 
			0x9a98af4324d438e9UL, 0x857d863534d67369UL, 0x59a23a0c6ac88fffUL, 0x648dccf7936a91bcUL, 
			0x8858e25596970061UL, 0x0594dec1048c2688UL, 0xedb6bc8199d3efa6UL, 0x9a3ea33e582403d7UL, 
			0xd0b12297907b8768UL, 0x5f3b81f0e8ac1482UL, 0x111371e0787b37d2UL, 0x8ca351f3a892061aUL, 
			0xf73e465d06c1f1d6UL, 0xc71c14dc49f65d17UL, 0x00ac205ba5fe9faeUL, 0xf1cd63b27208d53aUL, 
			0x7aa25af093a10424UL, 0x18461b1ed6759458UL, 0x32eeb53ac00acf8bUL, 0x1b7f573c9812c44bUL, 
			0xb2725e45dee7194bUL, 0xb988f88a32b6c53dUL, 0x05eb8f4bcfcd9a9eUL, 0xc82e26a7d25f3c9dUL, 
			0x6663251d71327d6bUL, 0x49e39e7d8a8a297aUL, 0xb311ad964855e021UL, 0xf78cb4c5feb95c2dUL, 
			0x9e42988546c54015UL, 0xd2bd03d1bf9c449dUL, 0x3d30fc51f282d0aeUL, 0x47967aadf18cb7d9UL, 
			0x0466dacb342269fbUL, 0xbe6bb3665a38495aUL, 0x935541553f05d9f5UL, 0x5bb535d756f0c84dUL, 
			0xc7db5a73020a7554UL, 0x3632b7ef6244a129UL, 0xcad44467daa93ae8UL, 0x357aeeacafd1097dUL, 
			0x892041c7c717cf19UL, 0x8f83efc09e0fdb7bUL, 0x9666f340f4531d7fUL, 0xbab151b505129fcdUL, 
			0x6bb1b4d19fa81a4aUL, 0x259e512cd30feaadUL, 0x71f504e5fdeda353UL, 0x6f2ff7fdc98ac613UL, 
			0x230b9f076850934dUL, 0x6064f6d1a13d94f4UL, 0x00984a3e18367e70UL, 0x6c685e17c8d7a926UL, 
			0xac5a63f0b8c89071UL, 0xa1b9e107b60f39d5UL, 0x63548d27a47a8ab1UL, 0x5a4f145d66e0e98cUL, 
			0x8ca728a1f71ec3f4UL, 0x755f2350ae4472feUL, 0x3036b44cea22faadUL, 0xedd9d5697f34bee3UL, 
			0xb5b55420fd6d3116UL, 0xe1d45974aab88632UL, 0x8af04dbb634bc3d1UL, 0xab091bee190b3713UL, 
			0x5f6b0e927a49e8b1UL, 0xea62ac55cfb80eb5UL, 0x9c259be97d3ab8b4UL, 0x94fd5cf77a9895c4UL, 
			0xc814f370936a2acdUL, 0xaf06668104973ac5UL, 0xd0b303fd5773bf4cUL, 0x6856b6001ec4c29eUL, 
			0x9cb07385e60d183aUL, 0xdae2cc6d7a52c78fUL, 0x366c8f0d880f37bfUL, 0x5541f8976862d26eUL,
			0x92af214f929d7734UL, 0x29863cc6ffd3a228UL, 0x760a06575f320fdfUL, 0x324e66eae0975057UL, 
			0xeac1d96a6aa79438UL, 0xabff2dbfd2bac339UL, 0x18171e45f22aaf39UL, 0x5b87834cbd03b909UL, 
			0x8788837ba3e2ad2aUL, 0x29b60d5de1b00fdcUL, 0x813ec41019434333UL, 0xa3c549c45ad5f152UL, 
			0x275374f9749141d0UL, 0x49345ef038b1ac76UL, 0x015fc6c7484fcf54UL, 0x3fc468d8a4d6ad9aUL, 
			0xd0c163e540034ce7UL, 0x88d5c39ed7f9493bUL, 0xfa547625c46af8f7UL, 0x6012b0271256b399UL, 
			0x481fa3e9dea2451fUL, 0x2ceb6f83a2dd4387UL, 0x5c0e826f48f81f6aUL, 0x2a28c055aa8ca624UL, 
			0x1b944a90fbfc65a3UL, 0x5712dd0d4417c85eUL, 0x5322f2c8afd0ca00UL, 0xdc21e6da0afdd8b0UL, 
			0x37fb67762fff3b42UL, 0x55d6191d7862f642UL, 0x98adee5ca3868a10UL, 0xdfcc6605a354eb6cUL, 
			0x188ebb85f21d3bd1UL, 0xc83ad6d216ed8485UL, 0x0f75e997e5837aa3UL, 0x9b934009b60e8981UL, 
			0x034a6f01b8a47723UL, 0x437bcf6f8c9f5562UL, 0xb45fd08bddf80412UL, 0xcc241f42a1e0424dUL, 
			0xa129204b4a77ea3cUL, 0x003f69b2a26fd80dUL, 0xc4f0caff1ea04f82UL, 0xfd84cc519f642c87UL, 
			0xe5c9038d2771bb96UL, 0x78e9cb7967526838UL, 0x4374497306edcc4dUL, 0xb6e4f91d62817fedUL, 
			0x2a742eb50c19f333UL, 0x3ad6b5e56ee7a472UL, 0x175e5ee1e837d9d3UL, 0x10c848791ff89f13UL, 
			0x125d3db1d5d060a4UL, 0x1cd6cbb1c7641ba8UL, 0xf0149c57f2208aa3UL, 0xbfaaed8fe370e524UL, 
			0x6a4111b14e1237efUL, 0xb4880d57b89a4886UL, 0xcad5e54520c50b42UL, 0x205f27afb700577bUL, 
			0x9bdc0e8d1a1ece8fUL, 0xf8fe52a3d6bac8e1UL, 0x0f23cfb0896955a9UL, 0xa78a28e86bb259e1UL, 
			0x30c23c4207dcfdd3UL, 0x21c56e08e30b59f8UL, 0xb99d0f1cde968018UL, 0xc6b3e1db637cefaaUL, 
			0x389ff99827267b34UL, 0xbcd215fa2d5f4073UL, 0xda191ba05d7e9a26UL, 0x64d9c37309a603e2UL, 
			0xceca9fde856babedUL, 0x612f380d5b0f173cUL, 0x0d17cd0c07527b44UL, 0x00be18b44b91d620UL, 
			0x336170b9c695bf05UL, 0x102d3c0341f2f205UL, 0xe45bd0f98c6a65eeUL, 0xbd5ddb4e74acddd5UL, 
			0x0be4ec324d773abcUL, 0x63a2e2fd2253d8b8UL, 0x6a75a52c62f01d0cUL, 0x60bd9a7db7a8cc82UL, 
			0x389e884eb9668391UL, 0x3e183605d6607ce0UL, 0x34df2e44a36f5f0fUL, 0x54d670e648383d65UL, 
			0x6ea69884f62795e8UL, 0xb3f7c973f5c011d2UL, 0x6670baf7e0fa8415UL, 0xbdf7588eb6f5739aUL, 
			0x933359d7904395b3UL, 0x763915a57bb80405UL, 0x1fc9e1c599db6460UL, 0x7dfe935743e10c33UL, 
			0x9eb6b0a99c8adb14UL, 0xddb63bdba8d8a154UL, 0x8f47e73902260f59UL, 0xbeeb77928a0b504fUL, 
			0xad111ec6e34dd9a0UL, 0x868266441caef23cUL, 0x00d443f5e637979dUL, 0x79bac16e892d2989UL, 
			0x8896587f39e73cc0UL, 0x67f26d70542e5207UL, 0x9f6453c3d3d91073UL, 0x9b189763ea337b1aUL, 
			0x39a3e70082a16235UL, 0xe9366ef4f53f7a45UL, 0x91b3e30822dcfb10UL, 0x748b630ddcc5ff8cUL, 
			0x260d2cc8725fb0f7UL, 0x178e03f4c937f5f4UL, 0x1a06f96a8fcf7ce7UL, 0xcaecaa829b813edfUL, 
			0x58c52e4720e3620aUL, 0x5491cc584506effcUL, 0x7f10e3136b2ba934UL, 0x640d516bf686ad96UL, 
			0xad8051eada279ad9UL, 0xa5003ae9e096982eUL, 0xbe01b844e1292b44UL, 0x0f48604a08281054UL, 
			0x8e310ea3a774fc79UL, 0x5eb111d8084e6ca1UL, 0x61a09166ec5373b6UL, 0xa8bc8307676293acUL, 
			0x6cc4e7ec3f7771ccUL, 0x156bce68e9fd4e78UL, 0xd4858a47eee3ba53UL, 0xa6b9c9d261013090UL, 
			0xa0aef2f60178b6cdUL, 0xd70a3c309ae9c157UL, 0x715e52a4c80a46eaUL, 0x35d84b03d8f68f88UL, 
			0x465713da939e3de7UL, 0xa97627cbf388d8e8UL, 0x359c075411b24dbeUL, 0x937c747db38f17c9UL, 
			0x9af6c2b11dc07c36UL, 0x802e5a89be1d4692UL, 0xe93b1d14d25bbb14UL, 0xb88cbb733c17d6b1UL, 
			0x029b856a7ee8e7d3UL, 0x0f54888156bf3e35UL, 0x9788ef105c851f39UL, 0x2ea198eb58260129UL, 
			0x466f714b1db2b3c3UL, 0xe00e935647aa9740UL, 0x724625787e940dc4UL, 0xe48198162e67b04cUL, 
			0x75ccecbb904703fdUL, 0xb00b02b35dac1cb1UL, 0xeaaef0962b84f75cUL, 0xd79841efa976f969UL, 
			0x930ff7112bdd5010UL, 0x36853fc2be25082fUL, 0xfadfbf9f80fee537UL, 0xf785877fdc4e62b5UL, 
			0x9b8f787c0daa06deUL, 0xd917a09bdac84acbUL, 0x320ab7bf8c97cd68UL, 0x7cff58d70def5342UL, 
			0xae963a817173d1f9UL, 0x0e39f7e596d976ccUL, 0xe27fd0f564589664UL, 0x62b5c1629fc58d65UL, 
			0xba6a74403e8e2cf5UL, 0xe39a66533021c4c0UL, 0xe89b2844cd49de63UL, 0x8747831f8ce6d1bfUL, 
			0xbb6b1a8ca41a0117UL, 0xf11d3ac429df58a2UL, 0xcd691fd843494202UL, 0x2727c92a3d184886UL, 
			0x853d31a05b8bb1b8UL, 0xf7564ddf3d871e39UL, 0x8ff74b25bf5e63eaUL, 0x9baba9e3c0c21e97UL, 
			0x5b08004969800fd1UL, 0x6e3cb6059ea7e294UL, 0xbc3c9d959ffc3dd7UL, 0x6c0f4068beced94bUL, 
			0x77f3f4c2db36771eUL, 0x891d410fc997ffeaUL, 0xe1480d28963c0465UL, 0x3225dc718b38bf1cUL, 
			0xe03243a1d2050bb0UL, 0xb55ed1399c76e5f0UL, 0x854c934a0289cd29UL, 0xce80ac08279994a7UL, 
			0x9660046f1ff7c542UL, 0xa6a64a0f5ea07392UL, 0x02789ce1b704e483UL, 0x2cd94e94e7ff9418UL, 
			0x1e86665d12536662UL, 0x9b10d39d1f5bbb35UL, 0x6080f9da6c53aa3fUL, 0xd84a46775995d6afUL, 
			0xa983e025be0e7fa3UL, 0x507eb810c06bfe72UL, 0x5a24c58f6a675235UL, 0x5ccb908a98598e57UL, 
			0xa48b22121b21e0dcUL, 0x666637936a624850UL, 0xbfcc194309e20ae4UL, 0x3a95fb79dfa2b73bUL, 
			0x9ab75b1ff3afb3f1UL, 0x81bc521e02557a61UL, 0x9467149adaa418b3UL, 0x140e9dd936321eabUL, 
			0x19f0e5bd9d9c0943UL, 0x6f9160b767c6dbb7UL, 0x25faf3aeb640d5daUL, 0x4719ebffbfcfb84bUL, 
			0x5810760d18c991b1UL, 0x7274da949ff91af5UL, 0xb960f284df2cb3b4UL, 0xd88c2e3b41308e84UL, 
			0xa806823103015d60UL, 0xccf5c731fd1738f1UL, 0x23bf3bc840082c1fUL, 0x6830c55fef650947UL, 
			0xc305cd6b47510c47UL, 0xf0d8d18705d4eb1aUL, 0x2990dab8767d8a77UL, 0x0fbfcb9444be3dc6UL, 
			0xd6f7a63502460e28UL, 0x15610e9fbc97216aUL, 0x6767594a436c2df7UL, 0x91751c3594c5ef97UL, 
			0x9adb27daecd25ca3UL, 0xa3d6dc358fd95250UL, 0xdb3c8008f349678fUL, 0x553a228e068ecf4aUL, 
			0xdcfcef1e46f66fa2UL, 0xa88f6c542cbeb178UL, 0x6a5ed6b67f849602UL, 0x91f298fc1dd0afffUL, 
			0xc8712c1a148dc516UL, 0x87704948e8678efaUL, 0xbc29fc3646398197UL, 0x7b17c2f437e7a62aUL, 
			0x89383d74af421e0eUL, 0x5a42557b1c44350aUL, 0x625ad609b47cf782UL, 0xf430a3612d1667aeUL, 
			0x7fd4a2c397c27fd2UL, 0xccb41dc56196128aUL, 0x1963f5bbf52afb6eUL, 0x7cab61dbf1e315cdUL, 
			0x8cee73013f8f8048UL, 0x62f9457a68d2b871UL, 0x88d821b455f71045UL, 0x0bc2853a711fea5cUL, 
			0x68f31c3e314a5fcaUL, 0x5ee06601974ec477UL, 0x1909a931401d975bUL, 0xafb2ca90ddb647c3UL, 
			0xf0c2213409cb0c2fUL, 0x028d166896ce4fa1UL, 0x2cb25654327c959bUL, 0xdc9d98b797c779feUL, 
			0x6238026f7e3f91abUL, 0x08cd5423a911e805UL, 0xd4f67f909207220bUL, 0x3f05505e01cee704UL, 
			0xa30212a069c22a75UL, 0xb1085d5dd6487facUL, 0x947f9f87fdb1aca8UL, 0x198b8562dc2b92dbUL, 
			0x1145b9c374a8bf8cUL, 0xb73e0e98aa7a14cfUL, 0x4ffd8aa3a455adbcUL, 0x2e4dfffc929ab10dUL, 
			0x127960325ab7db87UL, 0x5173d173ed3128ccUL, 0xbaa0479dbebf4b76UL, 0x72b3892fe8526656UL, 
			0x1a645f4081481470UL, 0xc2b5e4f7323bfa9fUL, 0x74d82167fce23716UL, 0xedc3b5515586d225UL, 
			0x0cceb4874d4cfdadUL, 0x78962c8ad55e8599UL, 0xba0d16dfe7f50a78UL, 0xbf6d2cc7423c839aUL, 
			0x46a452b968cf029aUL, 0x5f764f8e6438660bUL, 0x87f8db0e731f50eeUL, 0x77cf1d785cfea887UL, 
			0xcf8dce11d6d96d28UL, 0xcaf0ccbffa97107fUL, 0x9f515e0185859b6bUL, 0xacfd769c330a7309UL, 
			0x667b63dc75cbfb48UL, 0x6121e600186c1a35UL, 0xa82a41c4703d0147UL, 0xfa303cfbc6e90ddfUL, 
			0x596da7644499f60fUL, 0x37ca5c0fe5d124b1UL, 0xf69fc020f7200c6aUL, 0x9ba6c1df280299dfUL, 
			0xf16fee7a27fb1b53UL, 0xc0364863716b2302UL, 0xc71325aaa824fd9aUL, 0x87d4cf38f7184178UL, 
			0x3f92864d2d25f8e0UL, 0x7f68d1783f6b9e1bUL, 0xba8802fedf665717UL, 0xb32956846af1ddb8UL, 
			0x8a1bbc89d84417f7UL, 0x02eb786864180577UL, 0x135cd333c4af40d0UL, 0x80f357b1f7a4bae8UL, 
			0x33ff3a772e330ae6UL, 0x6e0526a6e44c8e30UL, 0x88e79c7c540ac5d7UL, 0x3ada2570887b982cUL, 
			0xd6fae5e324934f0eUL, 0x40d063926c325ff2UL, 0xe0953c82b8f434d4UL, 0xf2ebb7d244c616d2UL, 
			0x2e80c70d1f29a51cUL, 0x30e5f1fc1993cca6UL, 0x34e139a5cf05cf40UL, 0x0c5d529a7fe8a726UL, 
			0x877e104eefeecb00UL, 0xaa657183d234bfd4UL, 0xa2cdac817847d942UL, 0x5bc2a9329b4566b4UL, 
			0x55d9b0fbfa01b6c4UL, 0xd9da16dcd744de2cUL, 0x6317c3f17e51c01bUL, 0xb8f091061a5d6a01UL, 
			0x00cafe4ff10a566aUL, 0x6043d06308c406f4UL, 0xdf036c565cffd839UL, 0xd9b6d995427fcb62UL, 
			0xfc237bf215326d48UL, 0x77803da6e18b18f6UL, 0xd242c0a577a4cd7dUL, 0x11e40b3ecf734891UL, 
			0xfbad679f17f75578UL, 0xae669186b9268756UL, 0x07e40eed1b6c2d48UL, 0xb60d800fc39ecd81UL, 
			0xe4728322ed071a28UL, 0x16d10b28b8b774b0UL, 0x49be23c4e94ce7e9UL, 0xa1163686f2e0dc31UL, 
			0x110f88340f649b96UL, 0x77af3f8abd1d90f1UL, 0xc533ea1a334bd4d8UL, 0x17e67cdd90e93770UL, 
			0xbb559752c967302aUL, 0xac78e7cbe6ceb19aUL, 0xdd88d3cafc831486UL, 0x6280c139a6c0ec01UL, 
			0x932e05d96e5a469bUL, 0xc448f137c0cc26f3UL, 0x09fdb77d1c443199UL, 0x9627939ce16a5fdbUL, 
			0xa88f435b5f3c84d6UL, 0xb4e3a916fd00245eUL, 0xfea36aa20a91c192UL, 0xd56090c448d407faUL, 
			0x6a3e3d6ddbdc322aUL, 0x693074a6229ace27UL, 0x7996ecf1f5f77795UL, 0xf052b268d59ada84UL, 
			0xbae279c0535d98a7UL, 0xb149a9a6b7986325UL, 0x48f42bd390cf3c32UL, 0x9ab0a3654d3e723cUL, 
			0xd09075c09d505ad4UL, 0xbbedeeb5b355c6ebUL, 0x83f044053d2ce112UL, 0xd26c262070ca0609UL, 
			0x6f58b70a4e7e8132UL, 0xfce720124cf5e43cUL, 0x7af4609be34faa3dUL, 0x01bbe258f6b93f01UL, 
			0x6cc322ebe28f9ab5UL, 0x2672b942ad5d99faUL, 0xa960cf58e743e923UL, 0xd16dcc1e497480bcUL, 
			0x9d32e22f7e9bea40UL, 0x233506495872a606UL, 0x53078c5741f708dbUL, 0xfd06de82dff06e2bUL, 
			0x12b6d18a68350b70UL, 0x7ab644d1cb964a79UL, 0xf947ece94a43c3d6UL, 0x007faabebce90658UL, 
			0x53cf5503c0fb9efaUL, 0xdf3820fb0126e671UL, 0xf84aadca296daedcUL, 0x7de05a7717427a97UL, 
			0x330f6845f46b29cdUL, 0x84f9e3873d42c082UL, 0xf2362c2ba26d1c80UL, 0x821eb167d2ee8de7UL, 
			0x16ac880f8668901eUL, 0xf9362e67a7ee7689UL, 0x51c730158c187697UL, 0x66efc506e9d2f2b2UL, 
			0xee743caba52b268eUL, 0xfb640a739c465b89UL, 0xd5517c04f23fc359UL, 0x5fbbe100982e20e0UL, 
			0x3eb76e2d99970963UL, 0xd49d7753e8beff71UL, 0x0dbb1ea39c37d57bUL, 0x7b98e1a27c152d49UL, 
			0xf775b14ca2980015UL, 0xcf7174f5ee566022UL, 0x3e8ac6c27c8a3113UL, 0x847621ab35c3c1e0UL, 
			0x7deddea76d10a3a0UL, 0xfcb7bc897e05de69UL, 0x51ef998f39020ab5UL, 0x7fd07e55b7f3da55UL, 
			0xbfe06bca7b62c957UL, 0xbe68e96ccfb74e29UL, 0x00061aae8f505c8dUL, 0x02b630df6c947f75UL, 
			0x3334016c77f586f6UL, 0xb1da23b535471191UL, 0x983288360b0b10d0UL, 0x886431bb8d7602abUL, 
			0x1e9bb348ddc5310bUL, 0x7d645eee21dfae58UL, 0x925287e72f1f61ebUL, 0xdb79c84780a62423UL, 
			0xf266ae406d655baeUL, 0x542b773a3b87644cUL, 0x76ec5cec37d3b4a7UL, 0x259dccc0e06e73e9UL, 
			0xb265866d12ae4780UL, 0xccda794466480749UL, 0xc3b9db9c60c64bc8UL, 0x5002c2081789df32UL, 
			0xb314f7bd7d8195feUL, 0x4627ece92ecd6f7dUL, 0x445eb421bfebbeb4UL, 0xfd8b923c53c82880UL, 
			0x0b14f2aa0702b9f5UL, 0xadb3885372dd57d2UL, 0x35ce67bdab54d97dUL, 0x966705e5c4f10029UL, 
			0xb7d280d3943f7719UL, 0x79f11aa9c1aba969UL, 0x96ea7369de7ce2bcUL, 0x2fec08d968eab6bbUL, 
			0x4d3ccecd9a444788UL, 0xa459682b3f21e723UL, 0x5ab599901e4f0d76UL, 0x982e148b7cd8e94bUL, 
			0x644f8122c8f82e82UL, 0x4ff9f69948f039ffUL, 0xd0bfb1c751e1a040UL, 0x072ba163f87e9f3bUL, 
			0x235b1fdefbec666bUL, 0xad40ce821bd1389eUL, 0x24cd53943a5cf3edUL, 0x14a780df96da652eUL, 
			0xee648bf06bc95d3eUL, 0x1501becf83ef9bf7UL, 0xdd441bc72eb25bfcUL, 0x7f220feb74082e39UL, 
			0x919e18cd6c42a5efUL, 0x616253b3d16b21faUL, 0xccf2eec7690c60f0UL, 0x969febf69e155389UL, 
			0xf43241a3f94234a2UL, 0xff752d5243c59065UL, 0xe11ba5ec328398e3UL, 0xe4c5e1f20d96d281UL, 
			0x0007417125b404b1UL, 0xefa6485b8c4bd0c1UL, 0x9db28ab27bd9f1cfUL, 0x11cc85538ad8f535UL, 
			0xdd2922061b9e5253UL, 0x83d24a6284301b9cUL, 0xa1001ac6378d5da3UL, 0xd23d8e42f23e5b3dUL, 
			0x1eae4e2512bb957cUL, 0x7e7ed2643d773304UL, 0x2bfb1ea05c030c78UL, 0x297e5c52a4623091UL, 
			0x57b52333f15e5823UL, 0x2c0c48625aaa1db7UL, 0xb99e7cdbea678223UL, 0x568c992c50bf7d0eUL, 
			0x0813b08918faa1f0UL, 0xedaa0616c497fc1bUL, 0x88ce01e5311396ffUL, 0xa4237aea8e8e53ceUL, 
			0x46070353696e44d9UL, 0x62b95d88d30703fcUL, 0x0e04aec1003b580dUL, 0x933c153d2325e807UL, 
			0x5998bb644de8bfacUL, 0x807bc1450e2f831aUL, 0x342e75a7a9206997UL, 0x28a90175b9c866e2UL, 
			0xfc65d341c6489176UL, 0xfa5d3360f0835670UL, 0xf43d86c5f160d6caUL, 0x21c94513fb814960UL, 
			0xff4beac7da3519adUL, 0x50666cdea707566dUL, 0xec4ab4399b32537fUL, 0x322fcb29bc1ca468UL, 
			0xee4b89816b0a0873UL, 0xc611d74bb2368344UL, 0x631ac33c6e43f985UL, 0x46ebb9676256c097UL, 
			0xb56dd04ca193dc49UL, 0x6cedeebfe821a36dUL, 0x04d61cc969b044f0UL, 0x6e9ac3e5cc22d623UL, 
			0xc8bda0b1898639d2UL, 0x29bdcbbbbb0bfffdUL, 0x7e4bad5b1be3cccaUL, 0x9f5d0d89cb4db81eUL, 
			0x194dc6bccb8a3c3aUL, 0x7177ec3186e5ab0aUL, 0xe246592205d548aeUL, 0x9226bb2cb1035d13UL, 
			0xcfc11e2e44e41176UL, 0x1bd0ed328b6a641eUL, 0x21482c9bc1543826UL, 0xb76bfda912d0390dUL, 
			0x6c72b8e1357d049aUL, 0xa0c47d82c1a0fd7dUL, 0x5ae15399e93e3a0dUL, 0xa1277c7ae2c99574UL, 
			0x177ecb8d5644a4ddUL, 0xa8f1e5431d595911UL, 0x206abd39e2c11321UL, 0x815d5f4b1f50cb6bUL, 
			0x0ac621bb034487e4UL, 0x24c6673cd8b3cf41UL, 0x55bb8f81e6a11036UL, 0x8f80e7a971124616UL, 
			0x9f6a4431db4de2eaUL, 0xf2053d12fb0268b6UL, 0x2fccb6dc42b8eb6eUL, 0x0d275b5c04fc4a98UL, 
			0x4d0eded311948b58UL, 0xe49022980a2a5248UL, 0xdc4da609c0240586UL, 0x7188dff3739ce6aeUL, 
			0xeb0fbfd00cec2334UL, 0x21ab9277f1063603UL, 0x3e73ed6a52fc0ba0UL, 0xb71b011f5c6eba34UL, 
			0xbb67a471a8127703UL, 0x72042905dd82d1bdUL, 0xc5d2ecb378b3399eUL, 0x6d82b9613692a82aUL, 
			0x7ae0efc1245283a8UL, 0x5a8473f1cc04ed8cUL, 0x2331336ee2d011ceUL, 0x7b62974cd6b04811UL, 
			0xee8c1a1f79176b20UL, 0x2f60990cf3729fa8UL, 0x8da61220bd852a67UL, 0x6fd4b730bf3f2934UL, 
			0xb305b653c1b46008UL, 0x28ace58e51186eaeUL, 0xb7326c4ae224b556UL, 0x13f31467a6220f26UL, 
			0x17e85850b93cf7fdUL, 0x2821d27aa2eb4682UL, 0x61219bec8c8c8e13UL, 0xe83bd2d679846a73UL, 
			0xf1df7d2aefa66a0fUL, 0xd00b1979a37e1c38UL, 0x684bafba3b5a3431UL, 0x0baf13e51d5f8bb8UL, 
			0xe4a67c77e5b65eb9UL, 0x042c58a6a452b551UL, 0x40e13aaee72f1ec5UL, 0xbb27476c472ac1dcUL, 
			0xfe198a06031b4742UL, 0x0c9b7fa3909acb8aUL, 0x280474147469116eUL, 0xfdfcad1d25aaa3eaUL, 
			0xa466da5ee1f44976UL, 0x251c4a6591d8356dUL, 0xcf177dc4c85b8613UL, 0x5252812245a4e58eUL, 
			0x56884988f35181e5UL, 0x7a8c042179ffc12eUL, 0x85c1a66cf499108aUL, 0x3fb45080c18ddf74UL, 
			0x56c85d10a1542e8bUL, 0x1541c3a55e542b45UL, 0x97d7d0917ba72efcUL, 0x67bfdfe85e54fa35UL, 
			0x610be06efdfae429UL, 0xbe15e3064db34124UL, 0x71f81668d82d8353UL, 0xffa8b30a9ec8f265UL, 
			0x6608e266b127f0b6UL, 0xfd8cf7203b598563UL, 0x85c9bf26a52d9842UL, 0xa8ee106033c97c89UL, 
			0x22d9671601dc447aUL, 0x86e9e6be52ff0103UL, 0x7a36304d09d3306aUL, 0x2578ae5509021977UL, 
			0x93761abc96f9b711UL, 0x420338ac1ce51cd2UL, 0x6495d4643ad31304UL, 0x6c2619c1540e571eUL, 
			0x4b08988d611b015bUL, 0x34d68afcfe06b8a0UL, 0x1d1b4f2974e8df6aUL, 0xbc38c0b688bb20e7UL, 
			0xc2faadd5d8a8dcc0UL, 0x53e012ca44e3252eUL, 0x62581c54d9ecbf5cUL, 0x7ac7a85dc47ee655UL, 
			0xa49b736340e7e973UL, 0x3524ec5f6df4845fUL, 0x8ba07af0bf6dfe4eUL, 0x979c710d16ea51cfUL, 
			0xbacb85c15907f4a3UL, 0x96816bd5c1803ea6UL, 0x4ef1ef215285a757UL, 0x7e402d96125205baUL, 
			0xb0af435709c4c885UL, 0xe5be3bb7144973adUL, 0x5e394f284ad62b9cUL, 0xf6bb0cec59c7c67dUL, 
			0xc742a4ce9ba92055UL, 0xfa3a4c56c87a3532UL, 0x381a5a5dcf25558bUL, 0x27a02c59380e4af1UL, 
			0x6bafe48fdac27062UL, 0x7ec21c2c86e49defUL, 0xf53c31062d859425UL, 0x1bbca7134f8bed5aUL, 
			0xa9e702a7cded3827UL, 0x7f97bfe03dbdbff9UL, 0x8878ed0073859072UL, 0xa3cbcc3e20bc8733UL, 
			0xc18a49618111be91UL, 0x4f4198fd76ba032bUL, 0x7a3cf136236efc82UL, 0xa0d860159ba04bbeUL, 
			0x517829e35165447fUL, 0xb9050b1a1b4c8626UL, 0xd7f464494d5e7f10UL, 0x2fa9677bc3f700c9UL, 
			0x93185288d291d82cUL, 0x752d934011658839UL, 0xafa2966a67c2ae0cUL, 0x621ad0c39abf4613UL, 
			0x54148d65f49d492fUL, 0x1c8002b94c41a820UL, 0x5d826671e38fedcdUL, 0x2867cfeed81a3294UL, 
			0x101ce55251b5078bUL, 0xa90532711384f4b8UL, 0x5c4de23b49c5a863UL, 0xf53b3ddaa2e1f248UL, 
			0x8ab219672b9e9a41UL, 0xa766156b9d0d3ad6UL, 0x52f4f3df38ef18e2UL, 0x1fba3559b7d55b3bUL, 
			0x22903ff2e1ceaf30UL, 0x278e05a6d8521971UL, 0x6f91f224825f124bUL, 0x0d8f48787fbfa990UL, 
			0x7e7a833dc2322b18UL, 0x65fa78d2e044d366UL, 0x811c9bacb1bec0e7UL, 0xe68631e672f12476UL, 
			0x999bf86dea00ff3fUL, 0x59bb567858f7e324UL, 0x63e2923b99a2d172UL, 0x3ac7e0ed46d7b786UL, 
			0x72377ba19b2be6c2UL, 0xefe640c67fc8fd41UL, 0x2987d153248809ccUL, 0xcdc02da87d5efe32UL, 
			0xf13bb61225963fd6UL, 0xf00caecd6d323539UL, 0x0b6d4a1ead4da01aUL, 0xdd3a0b0717b1d4ddUL, 
			0x4cfd826a1426c8a6UL, 0xb79167d4807c5223UL, 0x487a77638f863686UL, 0x789328fa95d3f32aUL, 
			0x8b63b096225654ceUL, 0x21f081d0ef28a4daUL, 0x0e76951b63b52de5UL, 0x8b39b9ab879c68cfUL, 
			0x2a27115e905711e5UL, 0x8562682e683d7435UL, 0x8f4cd283382623bfUL, 0xb87b2c125e51747aUL, 
			0xe890bc07fbf1e90dUL, 0xd336913564b91d18UL, 0x6d733a665dff87eeUL, 0xabbd96c731a786c2UL, 
			0xa52fd4798222bfceUL, 0x5f8e66bdd740d8ceUL, 0x09df2bb26a0c4ffcUL, 0x9c265b49418e791cUL, 
			0xef9ce6d13a3ae7b4UL, 0x73060dd72ace983dUL, 0xc64c59fe59e01dd6UL, 0x4f48f7aaf8db3f38UL, 
			0xd3eec62e0d30f473UL, 0x0908cf14756aaea4UL, 0x79482bf0ff595dbaUL, 0x8ae8923401d84a1aUL, 
			0x9dc9a4bc11e475bbUL, 0xbfa1cd71c2a935bdUL, 0x901d7705a37e4500UL, 0xea5feb37f863a670UL, 
			0xd355a7f7a10b651eUL, 0xd32f60e6c1be2f4aUL, 0xa01f37e8eb6a5506UL, 0xb4981182380cf8baUL, 
			0x57a67a0b63e9c85eUL, 0xe3ccd120cd996856UL, 0xe9b0bafbe2981aedUL, 0xbeac83454ade644eUL, 
			0xd1497e223ce36245UL, 0x80f378eaa9dd6aefUL, 0xf2e7f377f26513dfUL, 0x4cb8b98320378bceUL, 
			0x2867ab80035442d7UL, 0x4b1851881595b1dcUL, 0x7c11b78debd799d9UL, 0x11b543f0d0899222UL, 
			0xd1cc1ef2263ad74bUL, 0xe23f2855154a4d7fUL, 0xc43cca664ba2209fUL, 0x9c17f2390e5459b6UL, 
			0x5c9dc840e24846d3UL, 0x3ffe263381afeceaUL, 0x586d7d5323a715d0UL, 0xb661f6af570fbcdbUL, 
			0x96ecb5763d68c1ccUL, 0x3dc13deebc0a14c9UL, 0xfef7afa86f43034aUL, 0x67c017754c77990bUL, 
			0xe9a7ce1a82978861UL, 0x4ed88d17fd424a01UL, 0x4c16ab9f4b0f2739UL, 0x2f88db6b7a841212UL, 
			0x699bd126beb09e1dUL, 0xbff80e332d9e66c1UL, 0x7441ed7fd813e486UL, 0x23d28e47bbd3ae6eUL, 
			0x4ad52fd8fd030748UL, 0xe9aef5052d7e060fUL, 0x60d4a7998fbd53d1UL, 0xfb39be7f41680f1bUL, 
			0x81a098d7a1c87a7eUL, 0x2480cc05c65f585aUL, 0x474e80fde13dfff2UL, 0x4c06b6020a7a02fbUL, 
			0xff6e47a39645baabUL, 0xc556006b91a28429UL, 0x35b93b32c2253388UL, 0x8e817fc4124306afUL, 
			0x26497727c41f6b18UL, 0x387533cf775eab99UL, 0x40032a2a10d30564UL, 0x5d8d300553b1fe3fUL, 
			0x96a30becd637c05eUL, 0x5f98a30b0a708b82UL, 0x78b426de60623ef0UL, 0x250f5c8d4bd9d397UL, 
			0x74bb757967f408eaUL, 0x740fc2de01b3f47cUL, 0x9ee743f13f36ed59UL, 0x13d8dde1349ceb13UL, 
			0xe209fe1d56685fbdUL, 0x1a98445297c7d9adUL, 0xd3286174ae3235b4UL, 0x97cb074910b87481UL, 
			0x9ac57e73c05eb48dUL, 0x6f231c42d481b202UL, 0x9a0939e492438020UL, 0xa13e5cc163af5b0dUL, 
			0x66fa9f5889f7e857UL, 0x6e2a5c6ef1c15340UL, 0xcac642ddf527f4feUL, 0x4178e2aa58e8d027UL, 
			0x295aa5efd10b2446UL, 0xc520e9a529e57757UL, 0xc849123fe62aa260UL, 0x210e1e2881f436a1UL, 
			0x2348e9af0baade75UL, 0xca2cfdd8ff2c30c5UL, 0x5d81fcb3bb7af9c9UL, 0x0c93d68877903da2UL, 
			0xc03e4337908a7854UL, 0x181b4ea6755dc1fcUL, 0x4d7e2e167c6e3e61UL, 0x6c86e73331cf9672UL, 
			0x405e4b7bd8160809UL, 0x3b5691baf268bbcaUL, 0x4673d135717f1bffUL, 0x27510c6d30ad37f1UL, 
			0x6769ddd294fa508dUL, 0xc5aa84a46951a643UL, 0xe598ffa3ff55363cUL, 0x8dfcbe4f4aa7cb7aUL, 
			0xb87e775199fb5d74UL, 0xef6bf25a5fd5413fUL, 0x34b135fab077e490UL, 0x1cd86b2efff0f7f2UL, 
			0x05ebacd3f2501699UL, 0xa110244fb0afd171UL, 0x148dc62ba127637eUL, 0x6d63a1d0649c3c5bUL, 
			0xa7f44a52324f4551UL, 0x172e8f3a6a6e4c57UL, 0x01220bd570570b65UL, 0xd508523d040cca6cUL, 
			0xcb8cd2c10c833e2bUL, 0x59600cfccc58e073UL, 0xc04dcf07200721d3UL, 0xd749d6c7f83c4a40UL, 
			0x124b21f507c2a837UL, 0xb6bdf04ee090452eUL, 0x9995448ebe5edd43UL, 0xe48c37f71e0d12a4UL, 
			0x68b1a1660dc55572UL, 0xc3a30ddb6a042d45UL, 0x0f4e8210411fd278UL, 0xd81cb59321eeffd3UL, 
			0x6f6c83dae4084651UL, 0xcbacbfe3684f5bc5UL, 0x2d537894f0d98e82UL, 0xdf17915325b35174UL, 
			0x75be466503f4bba2UL, 0x58ffdc7c938292cfUL, 0xca35c9d8cb220b26UL, 0xbabc37375d9d24e4UL, 
			0x8190cb0c900266e3UL, 0xc35bc8dd0d318f74UL, 0x6f39917562901380UL, 0xbae60b0280d50bbbUL, 
			0x31244eec058bb46bUL, 0xe42d42a799b82116UL, 0x7d4ecabafbaf87e8UL, 0x830d3299cb074b63UL, 
			0xd2a0d239898eb513UL, 0x57e9f97ae04dd60bUL, 0xfb62414df9601268UL, 0x9b58fe55cc398727UL, 
			0x0d95741e046b34e7UL, 0x55115f9c65f204d3UL, 0x0eee47fbe6649093UL, 0xac00ae861dc5f602UL, 
			0x15a6dd94a9795f29UL, 0x087a5e90f770df38UL, 0x2aac3fb43c7a66afUL, 0xf6dd895052fd68a1UL, 
			0xc676b2e4a596708fUL, 0x61328f19bc7c55bdUL, 0x751d552ec7752536UL, 0x36f5615948870c2dUL, 
			0x34cc7540f30f1a01UL, 0x84afffe9c0d429ceUL, 0xb714b2289f65b28dUL, 0x1cb207c8687274a0UL, 
			0xac5f16fabb755073UL, 0x2aa1a7c8b6e9ddbdUL, 0xd88a46dee667692dUL, 0x9692b2da1be721ffUL, 
			0xe6d6f69af14d2266UL, 0x48bfc2ffa5fe7aefUL, 0xde6f0fd78ca0b696UL, 0x2797eb906abf3eedUL, 
			0xd6ab09cb183fca2cUL, 0x9de3eed616f40773UL, 0x8bc3f7c5ccf5a7e6UL, 0x9128f7ce77e9f518UL, 
			0xb8ffafd6e2e70a7aUL, 0x72f35c83d0f6cf02UL, 0x783ab3d0af2574d7UL, 0xe112ffa58fb79150UL, 
			0x56fbbec7b5773917UL, 0x5f5e0bbc1305293eUL, 0xa75be4062e9e908bUL, 0x174c6b67fba2290fUL, 
			0x1c34ef996b00c82dUL, 0x464287f540a714cdUL, 0xf80e43e0d33cbed3UL, 0x750171b3351a5efcUL, 
			0xf3c8f7e1e48c97e1UL, 0x851e2232c0836220UL, 0x8caab7e5ade9221cUL, 0x1fde9740d51d0c9eUL, 
			0x76bbe49daf45b5d7UL, 0x137fffdd0f3f9b9fUL, 0xad3db924cbb7f097UL, 0x6b2451d12450a44bUL, 
			0x24714fb50c3c2ec5UL, 0x7aefacbe920d9f9aUL, 0x29b4f9863e358df4UL, 0x80d416584302a363UL, 
			0x06639f55da712dacUL, 0x287c7db791b696a5UL, 0x6b78d1be2de7b99eUL, 0xec2d4f8498b4f2b7UL, 
			0x0b6ae78f9358925fUL, 0xf6ca674d97a05054UL, 0x89731ced64602549UL, 0xe37b75fd9ef40497UL, 
			0xcc8c49e485fe3171UL, 0xd037a25ad47f0e84UL, 0x096b221c52b2dd26UL, 0x59fd112abd5652ffUL, 
			0x72263c9a26dd0646UL, 0xcf19cd9abf5da18cUL, 0x1b86f4d9dc7221beUL, 0x317f8e4f9ababd25UL, 
			0x92e50d62412663b4UL, 0x017e553165270c50UL, 0xc79fbdea43c57b67UL, 0xf314ccaf10028792UL, 
			0x97d54fee7453d7abUL, 0x8a59999ea0a53a51UL, 0x1e3965865c5c4d9dUL, 0xa33bb1049c0ce07fUL, 
			0x16f9267cfd57909dUL, 0x3919958351548976UL, 0x28b759718e8aa8c4UL, 0xc7b07d9a75d1d133UL, 
			0x4b6fd09d68790fe7UL, 0x23eb42e9a4633795UL, 0xb035a1427ff06746UL, 0x15ad9a5beaa6cbc8UL, 
			0xaa2eac0406df15caUL, 0x56eac8a40b50d2e9UL, 0x73ea799e6fe04de2UL, 0xe03b5f91c8593da1UL, 
			0x1465626a32bbc959UL, 0x177e0e844f138825UL, 0xd5b197d164e4bd37UL, 0xdbbb704b9179307bUL, 
			0x85ff1ecc10b0888bUL, 0x6a07d55ef2d2a4b1UL, 0x9c3c08d220075026UL, 0xc196f75204f7f22eUL, 
			0x11291bc8f6fbbb3aUL, 0x9bf8db2c7ebaa784UL, 0x7c4a058e5b1276abUL, 0xa8ebf5c8d3be233eUL, 
			0x442a3df5a6d2f38aUL, 0x2fb6800d461e7341UL, 0xc1345a311c328913UL, 0x4e77a46d195a4064UL, 
			0xe77539eb9dc0996aUL, 0xed3ef7a24f51e073UL, 0x67f1c2bc5d07fb1bUL, 0x9dc148d071ef9addUL, 
			0xc96ea31cfa28eaadUL, 0x0eb46bd47ed1bcd1UL, 0xb9aac9497d1060f9UL, 0xee5077cb38616a8bUL, 
			0xe89b369e75a1100dUL, 0xfa9f121b5729891bUL, 0x6cf5ee84fc379b6bUL, 0xd5fe59e732d9091bUL, 
			0xe26f1c0659258bbdUL, 0x98fd3ad63f117d2fUL, 0x3d5e06c753b3e20eUL, 0xc79be34cbddf19a6UL, 
			0x975605741e875deaUL, 0xa54cfd2172306484UL, 0x77a35a68c8a21384UL, 0x37105c7c8eb3f8daUL, 
			0x252bec57723e8477UL, 0x78efefa68ea24dc6UL, 0x452e987c5f586dddUL, 0xbfc2c54cdc4ff06aUL, 
			0xf7cb0466b5e947f0UL, 0x899ef6758dc34865UL, 0x1537458eb2c99a95UL, 0xb7feeadc95d48d09UL, 
			0xa6bc0f9bae6a69d5UL, 0xdce0575e271a2394UL, 0x1f0d13fb03e01003UL, 0x9ef14f4847978fe8UL, 
			0x11e2426b4ea226d0UL, 0xe863262a7203ce5dUL, 0xe8c2bb48dad3759bUL, 0xac5e1c47c67f2399UL, 
			0x6a60b92276bd7096UL, 0x88739d8c700f42a6UL, 0x79fcda6f05af22acUL, 0x99c0604ac54d207eUL, 
			0xfb7af90809b9e978UL, 0x418d2e14ed4c9339UL, 0x2fec43f56aef73deUL, 0x85e40e1c6ca5db91UL, 
			0x90c0a1894bd283efUL, 0x053ca7ae2b1d2136UL, 0x414e8a8e7d380e3fUL, 0xd597442cc71712bbUL, 
			0x58c2a1f7fe898729UL, 0x0f6781f16a8ce4ccUL, 0xc700a70a071b9df3UL, 0x84368e2d35e746f3UL, 
			0x5445b66751f3eb48UL, 0x16912da0f0562906UL, 0xcd24e7e44ddb4485UL, 0xc602657bff40bd72UL, 
			0xfbf5c58403831b31UL, 0xedc44f54a449350eUL, 0xc23d385ee07b9d3cUL, 0x099bc1de39342dcdUL, 
			0xc81c81d13fa23b28UL, 0xedc0a4c4996ffc95UL, 0xda4ae9d6b62ce58bUL, 0xeca8086968aaefcfUL, 
			0xce966899431f6c56UL, 0x874d647f5515a0e6UL, 0x3996e848459b6b89UL, 0xd8c17ade290ddecfUL, 
			0x6796fc82d452b97fUL, 0xae0ec78c8f4ceb6fUL, 0x78139fef62956823UL, 0x382a4860025b4162UL, 
			0x78aa69d91cb610e5UL, 0xf6b3a8b9b915c29bUL, 0x1e9b1d8e2d19b333UL, 0xc1da4db5f4594828UL, 
			0xe9df5ce955726233UL, 0xdee5a45d68cbd0abUL, 0x63ce562c34ea325cUL, 0x6cdd00a95489cbc3UL, 
			0xe631b5d026745b42UL, 0xa858d416a5b9892eUL, 0xa70bac15753d109dUL, 0x6a010fde4a861e75UL, 
			0x5b21162044f6b3d1UL, 0xf59112092e10f347UL, 0x1bcd84f78a40a3fdUL, 0x8696962c7971693fUL, 
			0x175f74050e34807dUL, 0x478ffb97e34a31e0UL, 0x77c35d3da16a7690UL, 0xf54c92306bd24af3UL, 
			0x7007ed5a3e26842eUL, 0x784224edcd82ea6dUL, 0xd114034b2cdfebe9UL, 0xf8459a4884810626UL, 
			0x206400d51e9df3efUL, 0x3dd76e94cfe3592dUL, 0xc6e642455a1a382cUL, 0xcd3e91c2458813bfUL, 
			0x0c9e738a636bc1b3UL, 0x89fc3084381b17e0UL, 0xf7afcd967ffc6233UL, 0x68bee0cecb3bbbf7UL, 
			0x92ceef1f292b61c4UL, 0x17ac583123b9e03eUL, 0x0983be788e8738ebUL, 0xc6b5c32b10fba68eUL, 
			0xe55e68648f11611fUL, 0x8765e9491f7020bdUL, 0x261e0e76ceafe3b1UL, 0x2bf665ed165a0593UL, 
			0x058ce01fa6177508UL, 0x83b9f1296fe0fe23UL, 0xffee8234c2d4bbd5UL, 0xb68923229a09dc2cUL, 
			0x2e72fa4a9a3c5cabUL, 0xabe72fd9be975bc3UL, 0x041dec4c17d83f01UL, 0x38932bdc53ea2938UL, 
			0xc33df5a315faf9b2UL, 0xa37fb5e614e2a41fUL, 0x7198a190538c849dUL, 0x36643d9b5e1d7a4aUL, 
			0xcec429536f5f6cc8UL, 0x5c9cb57c1f3d0e13UL, 0x7b8b9bcf8585c9d9UL, 0x143fd48b7375fc11UL, 
			0xfa0a448402045336UL, 0xc2d5089c9a121141UL, 0x2159d12a1403c328UL, 0x6b0a1f8f520d947bUL, 
			0x7bd04ed783a4c8eeUL, 0xe1c0b2de7e91da1aUL, 0xbe7ce9d8d4ee1986UL, 0xacda79b66f4f86e5UL, 
			0x899a2ef514bec835UL, 0xb505c4eb9b623b7eUL, 0x323183a9f6173db7UL, 0xd2d3b0ccf5b7259eUL, 
			0x733aebdded6e9f0aUL, 0xfb08d1aec222729bUL, 0x80fa3d955bd71143UL, 0x8bf05554042878a5UL, 
			0x6c7af41e7aad955bUL, 0xa369a71cfc184a57UL, 0x9a65af41259eb4b5UL, 0xb42686769845db15UL, 
			0xbb9122c4c00095f0UL, 0x45a67d8413fe3f75UL, 0x7f5e41c813f48196UL, 0xcec41c9f3a332701UL, 
			0xf9c172a7615f31b8UL, 0x1419e7fa2223787fUL, 0xbf0cba3d8ae166d4UL, 0x17b2fdce0c13e99dUL, 
			0x271c81f56f446e26UL, 0xd2aa20949a8815b1UL, 0x3c8e0f9c5291f405UL, 0x6226cd9a5cef6cbeUL, 
			0x9876ca43a17c2942UL, 0x6d53b5156f8d9e82UL, 0x627559dd6acdabb1UL, 0x33840a6f6cf38e42UL, 
			0x5cb5ff4332ee5d3dUL, 0x68744a276b1272caUL, 0x3aead0976a4533f9UL, 0x90923674491213efUL, 
			0x85b29f7936a91619UL, 0x267855c9d03a55a3UL, 0x03bfd7673c8f1868UL, 0xd06cef326fbf4fccUL, 
			0x91f1d3acc1a0d1c1UL, 0x3066aa01e365e9e6UL, 0xefa3354b12a47986UL, 0x9df82d8022bc9c2eUL, 
			0x27324dd151f012afUL, 0x0475f362a5ac0b33UL, 0x0db532f946511a12UL, 0x77d422bf6021f360UL, 
			0xb0cf54f3c7986b00UL, 0x5188818223e0d4abUL, 0x0353683d481001c5UL, 0xad13a9f9b4b5aceaUL, 
			0x38673bb6878f0c7fUL, 0x05849e09c3d31923UL, 0xcbd5cefe0b6e651fUL, 0x02ba02bee4b9bf78UL, 
			0x37101387f805512aUL, 0x5e0180e7fd2378e8UL, 0xd7db7701d5c2a1a8UL, 0x8c6630f75631f6baUL, 
			0xfa1385c453f0980eUL, 0xfd4e98467be4ede4UL, 0x806d22e2e9bb0794UL, 0xc2a463f077cf02c3UL, 
			0xb420265d7b51e9c9UL, 0x1fa25fadcf38ada1UL, 0x082acaf041686c41UL, 0xd207c694a52bedb6UL, 
			0x993e7775c1fddb8aUL, 0x27f9b06aff911007UL, 0x383923b7584bf0eeUL, 0x7b145aa5662719efUL, 
			0x806d7c5e7fbbe11cUL, 0x14e41cf837d96e31UL, 0x8a387a1eae8f4c38UL, 0xdfc62ba666e1d922UL, 
			0x37473cd62b94a940UL, 0xc0749a953678dbadUL, 0xb7bfe21945242533UL, 0xd66a6494652162c4UL, 
			0xf993eb87e8bc4489UL, 0xefbc4fd350615b83UL, 0xac7b03f3f32d2f10UL, 0xe25b55ee67311d58UL, 
			0xba5c85711d771ad1UL, 0x122e1f53bed6a1b6UL, 0x5da447045a720149UL, 0x608f31a6c9114450UL, 
			0x933415608fbe2d81UL, 0xd96b820be981b037UL, 0xac784d2a24de8493UL, 0xb86eb90658c8d38cUL, 
			0xd0c7cc5fbb51de46UL, 0x08fd26722b98b649UL, 0x32cab64d0c1f7332UL, 0x08fc5f8ef65711ceUL, 
			0xb7d4a8737ec04cffUL, 0x9c90cf6fffefe863UL, 0x78531ad18ab69c7dUL, 0x46c0f0f9e54e72c2UL, 
			0x751f35b746b5fe88UL, 0x78fac1cb6a7b4fcdUL, 0xa9a8a7e58a9d9a81UL, 0xc98dc705e70f51c9UL, 
			0xeeaea7dc64c1f3aaUL, 0xe4aa35c24f2285b1UL, 0x434cd6048f895a52UL, 0xe7fc3a2058e5b3d4UL, 
			0x1f93870d6ca43794UL, 0xf9ff128123d8c90eUL, 0x76b6b3ddf3fa574aUL, 0xecc18bf222d73667UL, 
			0x94a7ba53d142235aUL, 0x0bfedd27b7cc35d6UL, 0xad9f700ef433a5cfUL, 0x50dd615486af62c3UL, 
			0x572800769e443aa1UL, 0x3014af2dda011a0fUL, 0xf024c7e0631f5887UL, 0x91eb95832b478983UL, 
			0xf9ca32994ec8d8dbUL, 0xf2fdc81f2681b306UL, 0x5e6f9d5a17d40f27UL, 0xbfb55c4c8e28c381UL, 
			0x81adf6e01a55fd1dUL, 0x0689d2b358bf88d1UL, 0x0de814d0003474adUL, 0x01a42930023033faUL, 
			0x3086c1a5b90154cdUL, 0x4f0267eb230a1923UL, 0x679b2fb20568195cUL, 0x40187677e620cdacUL, 
			0x07948b8897be2e34UL, 0xa840c85d11b1ffaeUL, 0xfd6df2c4050952d2UL, 0xb56505e981d5d7a2UL, 
			0xef3fe1fe3915362eUL, 0x2b7d45f6898aa394UL, 0x7148260c198db857UL, 0xb90af162b66e5fadUL, 
			0x12b5bf77423e7555UL, 0xdc79330f2c50b3e7UL, 0x786a9960965f13e4UL, 0x0a061fb4e4d650c7UL, 
			0xa2cae995e9d08d56UL, 0x5ad5cc535e7d5223UL, 0xad5f281a31e78f7cUL, 0xad9477b326a0fbacUL, 
			0xb21fb1e0b887db35UL, 0x76156d5d17e65dd8UL, 0x7efede2f9ca5a3d9UL, 0x4878a9b913b364f0UL, 
			0xc8f3df57170580b5UL, 0x8d48442511936ac5UL, 0x4592de3307b1a85eUL, 0x5eb510b7169e8084UL, 
			0x7fd42df4accc50adUL, 0xb91298556946829cUL, 0x22d6de4b4eca9923UL, 0x906ac264f89dd014UL, 
			0xf4101094dd2b102dUL, 0x444ef448eefba8daUL, 0xa63332a906f95665UL, 0x5e76f1581abe17e3UL, 
			0x21f3c7f048326ef7UL, 0x668d74144a2b109cUL, 0x2153cb817714099bUL, 0xbead3c43cb401456UL, 
			0xb4904a6a5f1a920dUL, 0xbdfb57d5a5d394ebUL, 0x8367b2404a001c4eUL, 0xe0d82ad8c328a4abUL, 
			0x3277863ffac17d92UL, 0xb5a22167df4175d8UL, 0x97573bd5b6fbe646UL, 0x54be7263bdf6458dUL, 
			0x282eb0689cea1d29UL, 0xf5853c4918229934UL, 0x731fe29a3649b1ccUL, 0x4a0ca45beeb3d9fdUL, 
			0x63a4b935f5c49ca2UL, 0xf02248a49fcf991fUL, 0x57ae52ea955c5d90UL, 0x264fbc600347e07fUL, 
			0x6e39514aa31a7f30UL, 0x7032c981218b9b26UL, 0x6ebe66239bb749c5UL, 0x46dbee47f2150052UL, 
			0x69b1dfafe966c2f9UL, 0x6a2b198f7e8cd705UL, 0x8d2e6ed41195a1d0UL, 0x531ee0907849ede7UL, 
			0x4913031fe4300408UL, 0x046b1443b9cbe096UL, 0x4f271dccbc8d5aceUL, 0xa97a892c50fae7a4UL, 
			0x24bf0e91f9974bebUL, 0x92c9c7d3de4255ddUL, 0x73a46ca6a71897b9UL, 0x5e236081f5469e27UL, 
			0x6a7ae3c2162ad31eUL, 0xc121c4f92eb44e01UL, 0x857bbe28fa2dc6faUL, 0xd888b6e2350d92d2UL, 
			0x327a12d41e89fdefUL, 0xeb22ac62f6126060UL, 0x45116c3d050a4599UL, 0xfaaf85ffe4fdfe2fUL, 
			0xf9d1a91a00366403UL, 0x07298c86f200c137UL, 0x0dceb1a17a4b7b14UL, 0xfcb36c75dd8b66c7UL, 
			0x3e2948ad7032190dUL, 0x1a1c0dcbec9d4e5aUL, 0xd369ff705fbdcebeUL, 0x46eec4767c186420UL, 
			0x2e81388132b3dcbaUL, 0xb4146e1dd41e200dUL, 0x6ebd0ea2b0e62fa0UL, 0x0db19e2d67e5f31eUL, 
			0xf91329daf36505d7UL, 0xa191b0a206c7ca72UL, 0xe9dcba76685e90f9UL, 0x86167de19359cd4fUL, 
			0xcd57497592b75d96UL, 0x4de14183465fd016UL, 0x8956b3ce825ad1e0UL, 0xd68a80b43b07a15eUL, 
			0xf788d914dd633313UL, 0x88bf9ac7c87b712dUL, 0x67848d61a183c35fUL, 0x1837c06f5e0dae86UL, 
			0x6955f3ea2f78cd08UL, 0x40837e79880d5d9bUL, 0xf6294f196d3d91acUL, 0xa2aa203d04f8fcbeUL, 
			0xc56d9a1a5708b7c6UL, 0xce55b79d9dee1af9UL, 0xc77dc2d756a86546UL, 0x8e9ccdf0a6182f8cUL, 
			0x40c534887f26fd84UL, 0xc137bb6009f15d29UL, 0x8273f6d6620d5350UL, 0x3ae71af530223d7eUL, 
			0x7f58de2a1d7a02a2UL, 0xf57c87000be41de1UL, 0x1bdb9b936b8a2efbUL, 0x98ebf3621ac4ff3fUL, 
			0x24edab17dc46cd78UL, 0x9442137857ee606bUL, 0x754dfa3f67b9841bUL, 0xdc3ea728f7dd21c0UL, 
			0x002cc75bd2619642UL, 0xe8547cb8ff3420a6UL, 0x5feb763bfab94406UL, 0xd67d194e2fc92352UL, 
			0xf42a5b3830f47feeUL, 0xe36354592be5fbbfUL, 0x3e3354749d426a68UL, 0x47512e685ded94b8UL, 
			0xf6e720cab7a7496dUL, 0x4be6c839fafdb511UL, 0x9334bddc051df51bUL, 0xbcb704820d7ef196UL, 
			0x84007d4189fdc2c4UL, 0x8ea20ae9ce573fe1UL, 0x1cd3d162168871cbUL, 0xb078144ea9258835UL, 
			0xb3d300c5d975d90cUL, 0x4b7a67393ebdb333UL, 0xe67c8ee9341efa53UL, 0x5b237da418cfe020UL, 
			0x934fb588e0c4ea5fUL, 0xebd7941cb4c7e173UL, 0xf287f376c16d1bc0UL, 0x6f924ea4025f45f7UL, 
			0x59c12c801aead4b2UL, 0xb0124ddc8d9d486bUL, 0xa728049f667b0118UL, 0x03e9b3cc94e6c6b7UL, 
			0xd29f886d4317af22UL, 0x05cbcd9ed186e3d1UL, 0x9bf362811a35f183UL, 0x0468d0314c81f1eeUL, 
			0xa4621eb994b5ab46UL, 0x6c475b6963b5be84UL, 0x8ee9db6b37e96dd4UL, 0x1a2cb16ab16e2329UL, 
			0x22c6c04e5a18bc72UL, 0x8ad29c391dcab01fUL, 0x6fe2269baafc9155UL, 0xe6a1d332755b31d6UL, 
			0x99590b51409ab448UL, 0x9da900d0c629ca9fUL, 0x4a550fb3afc26430UL, 0x25af67d911f2e92aUL, 
			0xe0dcf25d5d57bee6UL, 0x619d21f0e19a4031UL, 0x34ba1daaf4b1d3a3UL, 0xc9ecd4d9f005135dUL, 
			0x46a3a81f9457ff63UL, 0x1957fa7c58bf700eUL, 0xf85d291e0ee01b7fUL, 0x0682b9b83c5453a4UL, 
			0x5f90e08798108a54UL, 0x422b1f30aaf842feUL, 0x37e2dda16b97ce3bUL, 0xea364810037ecf49UL, 
			0x1f1e833d10db011dUL, 0xc4c156d9130758e4UL, 0xd056db6549fa91cbUL, 0x406f88b6d1710a85UL, 
			0x4c7b9bec42ec907aUL, 0x004d41e535c1bdbeUL, 0x17c0ed8a6655f7b9UL, 0x541145df4afe5be2UL, 
			0x41b6d73a61b3ee2fUL, 0x87a4276a1f114732UL, 0xb68bf46debceea2cUL, 0xfb2ec67981e35aabUL, 
			0x6876ab24237328d8UL, 0xcdc6fd41a76d3415UL, 0x7069ad4341ba9593UL, 0x82eeb9106d2d51c3UL, 
			0x20d84765ba387533UL, 0x8cd758e6ea937724UL, 0x1df2a52ed1f4401cUL, 0x8e681074354a27c0UL, 
			0x35998afe4807e48fUL, 0x15a492af18157a8bUL, 0x36e9554c5de23259UL, 0xa49facba5bbf2123UL, 
			0x1a738687c7edba2eUL, 0xb49dfb2d7da78036UL, 0x93b4df9f2c2f3200UL, 0x26e6ef76f58319e2UL, 
			0xf3daacc874123586UL, 0xd98c1e7ca59eedc3UL, 0xc38168fda9acbcb1UL, 0xdfdf1bd62b9e30c8UL, 
			0x017a71ad035318daUL, 0x6170d0726af16d39UL, 0xa537e0485c56c92bUL, 0x149e8f6acc43d738UL, 
			0x8af5a60184db19e6UL, 0xeaeab291c55ba395UL, 0x9abc5662d1057882UL, 0x3178299dda8a9a51UL, 
			0xe3a6a31949c80331UL, 0x86247a660a609107UL, 0x776286a35408922aUL, 0x421437cd97be3a8fUL, 
			0x693cab6da2b5e9faUL, 0xdcf0aeebc9c18e6fUL, 0x3d6f4e00a7f104ceUL, 0x0af188ac9f69fd83UL, 
			0xd3a00a080194cbc3UL, 0xcf82631c93e6558fUL, 0x6e5ff1da15ba0e72UL, 0x2745ad8e7f69d3d1UL, 
			0x43951257d7e5bc9bUL, 0xe4ffc4f188e1af1aUL, 0xc223af2db9642ca7UL, 0xe5c1c844854ecf3eUL, 
			0x0229b78efe2673e8UL, 0x356709da6842f2d7UL, 0x9382910a135d7980UL, 0x2b6b5d78f65bd896UL, 
			0xce48a70150bff4f8UL, 0x905148317dedaca0UL, 0x44d69cdf260ec001UL, 0xb5c4c25359b1e1b5UL, 
			0xcb626830a16b59b2UL, 0xdaf64ff44722bfbaUL, 0x89da93a8839350ffUL, 0xb8705973bfd93106UL, 
			0xb8153cb932326585UL, 0xfa9aaa37d29a36f6UL, 0x1dfd04526168a576UL, 0xe9034221c90f2692UL, 
			0x0bcdccf447bac16aUL, 0xc794dcc2fb92f01dUL, 0x2de6cc6790449e54UL, 0x83cd28687e9b6977UL, 
			0x85b062e2c2c1c837UL, 0xa6040e2b0d8846f0UL, 0x494ab1c9605f3a2dUL, 0xb6dc39d3fa6b307dUL, 
			0x61090bcfa9282c70UL, 0x0fdc7ade4ad5c529UL, 0xd65e8854c5e52802UL, 0x61e5f37c27524b3bUL, 
			0x95464ae61aa3f04dUL, 0xd286f6a0546dbbd3UL, 0x202d260218699496UL, 0xf598bc97410fc9c7UL, 
			0x658086d156ab398cUL, 0x8eb61d6689f920acUL, 0x7744d272445d9051UL, 0xb587bf24fa556484UL, 
			0xc062e8108af3f007UL, 0xce8f1cf0fed3408fUL, 0xa0b609c0298cf5d2UL, 0xe8f01aac2669fc28UL, 
			0xc94fb6dc3507f218UL, 0x05d6e63262a8c92aUL, 0xdbd2e89f0067afb7UL, 0x7372c6fe4fc6b017UL, 
			0x56eb54de351a83ebUL, 0x52c15e8f3958f21eUL, 0x502c61879ceec1b4UL, 0x6fa8f313116a15f8UL, 
			0x4ac2b263fc1db4b0UL, 0xdc8a4432caac2e88UL, 0x8ea693104e950bb5UL, 0x1bc8717003e9ae86UL, 
			0x75df21f8278962dcUL, 0xb4cb9fe52778a140UL, 0x0854a43b036f25ecUL, 0xb512d8ad6c23c335UL, 
			0x26b51eb52fce7e18UL, 0xeb1556d8798837b4UL, 0x9bdb893e59c23f8fUL, 0xac0ae0b6265b981fUL, 
			0xba9353b8968aeb45UL, 0x8838eb68d932f6a8UL, 0x668e0004da2b4543UL, 0x5a7c7ed9efbf47c5UL, 
			0xb25653d09b9ea213UL, 0x1dca92753878c7a8UL, 0x593c35af30c2edcfUL, 0xe2133dec4ec81f9dUL, 
			0xf8d3889a0a176452UL, 0xc18f071aaa18dab9UL, 0x8663e90ca669523eUL, 0x38be8708045543b9UL, 
			0x9caeb461b026be53UL, 0xb479c6f15042ba39UL, 0x66846d1f262b5731UL, 0xd623078510499386UL, 
			0xfa57cd5d46cae5adUL, 0x6d5b3b3384bb983cUL, 0xd1bd901a9cc02c55UL, 0x68b0b7e40f03cdf9UL, 
			0x7d00e01876325e5aUL, 0xba68968ed3fcb1ecUL, 0x2c88259716d4d227UL, 0x479733376225d5aaUL, 
			0xbd02bde8cbc8088aUL, 0xc55d538d49d3cc73UL, 0xa47590d932649875UL, 0x34d34d01655602e9UL, 
			0x19cd29403c267904UL, 0x7d5c4da3cf551d45UL, 0x4a319a2fc179095dUL, 0x961c9ba888893820UL, 
			0x68f569b8bb10af82UL, 0x259de0efa4d36101UL, 0x54565b1b3ac34681UL, 0x7ebdf0aa85190cc4UL, 
			0x3c78ccee06183f27UL, 0x73ec1fd8b24d3da1UL, 0x0c9d6f657eeaef89UL, 0x98e8b52d699e8cf3UL, 
			0x3e498e707187e564UL, 0xa3112ac5ce17dfceUL, 0xfaac5219b9be768fUL, 0x0448db090fa7aa74UL, 
			0x0e68a59d19fd936fUL, 0xb1cf99913548e11fUL, 0xb88edcdb8dedc7f4UL, 0xc7c3cb2012c09ccbUL, 
			0x9e5d470281f73511UL, 0xde2a5adb71a44c18UL, 0x78c5b6122deed5f7UL, 0xbfce3e9d2675b640UL, 
			0xce110048fa940b8aUL, 0x48c1957c7ecb134fUL, 0x9c4bf1bdb32cbd98UL, 0xe9fab9fd4fc9ac97UL, 
			0xb425c7e95a61fabeUL, 0x16c86b2a12497985UL, 0xb8fe6adb4db4922cUL, 0x8fa042c5b6a95f33UL, 
			0x89ae66360a2ce443UL, 0x0b109c5a73bc21e4UL, 0xcafbcebf702fff79UL, 0x54c81bfdafcc050eUL, 
			0x2d9b1e8f05e4fe6bUL, 0x39736e2e8a3fcfb0UL, 0xbabc7b8315b6aa0eUL, 0xb44955cac91cfe88UL, 
			0x4b93ab24da1c9f6eUL, 0x2d020f56a037edb6UL, 0x65cbe591f0d15d15UL, 0x0548be2f5700b6d1UL, 
			0x0a5df074baa255e2UL, 0x8a09c8bad3d0d14bUL, 0x31ca0b182863b9f4UL, 0x5a160eb3e7068b4aUL, 
			0xf3bf51a16ccc5190UL, 0xfc5258be8e5e508cUL, 0x93a7cd58a5044814UL, 0x35fe857c935c1d06UL, 
			0xad4ba5a0fab89089UL, 0xe79670e583fa1558UL, 0x8e9cf477d9c9ae6fUL, 0x0dae6059c553ff07UL, 
			0xc8eb547860db5159UL, 0x0835739aac24c102UL, 0xecb2776227e39b85UL, 0x77440c9c88a470cfUL, 
			0x53480cb16eb9a2ecUL, 0xabbff0d15b00132bUL, 0xe6c9b3036400f3caUL, 0xeeadb800584364b5UL, 
			0x2c32cdf90302e499UL, 0x0368fa0e1a62ac3cUL, 0x19d267b082b063e2UL, 0x5c68695edebbe159UL, 
			0x789f084031204772UL, 0x1227577c1d234fe6UL, 0xb1ea8512ac680c03UL, 0x06ec3f72cbf2eb9bUL, 
			0x5c2ad3ee1f64e18eUL, 0xfd10621753b2cb69UL, 0xc781931732cdf148UL, 0x145bb6d923cc0f82UL, 
			0xd4f9b846983fc398UL, 0xad148a4b38a14384UL, 0x492f1b4330eef4b1UL, 0x34f83ce80c68038fUL, 
			0x785032c33407483cUL, 0xe1196fc402afb576UL, 0x4443955372d2501bUL, 0x9396d99c743b3d6eUL, 
			0x77f6d3a1dd5da6f8UL, 0x737074261fac4b15UL, 0x2622176d0506c167UL, 0x5ec50eb96e789eb1UL, 
			0x5983abce3dbffb09UL, 0xddb0b6dbd32f25daUL, 0x7734c9a27364a8a4UL, 0x5483f779ea64db5bUL, 
			0x26dccaead7b0c19bUL, 0x03a81b66a298d44dUL, 0x1b4ad81aeda23483UL, 0x5f6fb02673c0aa2bUL, 
			0x5e8df75c89ec7e34UL, 0xab5eab3c618b7468UL, 0x38165f46c3d82c3eUL, 0x833e7e886c545e40UL, 
			0xefb0ac7b851c50b5UL, 0xb78454ac0f0050bfUL, 0x5ec09c26a81279afUL, 0x32d38a2084912033UL, 
			0xd99cab9353c88713UL, 0x129b0fa6273a48ddUL, 0x2f3c8c3ff3fc4f19UL, 0xf22771a479b77a59UL, 
			0x78504d8ac1c40651UL, 0x150752416da92a47UL, 0x3d4d938daa502d1fUL, 0xbe641319ff958292UL, 
			0xc9c08b81c8983546UL, 0x49cebe9d7a1a06f5UL, 0xb61713e0f0caef89UL, 0x80b322ed7e017ab8UL, 
			0x336139235978d252UL, 0x2802486c52d24fa3UL, 0x13be07d95ced4cd7UL, 0x347b24a3ef2dc963UL, 
			0xb1f4610b9751254cUL, 0xd92a50951289f5a4UL, 0xd27094d255d833f7UL, 0x2218572d60c3608cUL, 
			0x168a0c65d6917f70UL, 0x02adc489a79b3c27UL, 0x42bfe4ff68487b5bUL, 0x8d745e0f27873b7aUL, 
			0x26f2bc42a38a033fUL, 0x490492f7a1b62557UL, 0x9c9b283b294ec0f4UL, 0x42fdaf66cffd9ca0UL, 
			0x2ad77e62669ed0bfUL, 0xab2f5c40361b2d97UL, 0x8492025f3499d549UL, 0x074283f91303e993UL, 
			0x58b1a4177d83a7bbUL, 0xace52d673762ee08UL, 0xa4efcb882a226d38UL, 0x148a8e1f98c61afcUL, 
			0xd50c8cc701854a20UL, 0xd36c2c0fe918add7UL, 0xbcd755a31561a0b2UL, 0x45a2becd64272a51UL, 
			0x643e6cbf87d51bccUL, 0x26835ad0bf2efad5UL, 0x118392760d178c28UL, 0x651fc6c01ab78ad0UL, 
			0xd4025fbd1ba20e21UL, 0x1f5754457ab90709UL, 0x8f61f40c044ce0eeUL, 0xc4f4d3f88bee945bUL, 
			0x86e65a53e8a644c0UL, 0x8589c835b907d8bdUL, 0x36b8138a4d527830UL, 0xc60d6d8c62c0799aUL, 
			0x681589d83b691296UL, 0x434e63bab01bfd96UL, 0x63d2c75adc2db825UL, 0x7de54a4b76567a84UL, 
			0xef3fd1710412d164UL, 0x5e03c96632bebe32UL, 0x9056bb89f6e49f7aUL, 0xbc2ddd1c9ecf54b7UL, 
			0xfd1edb2a0673d939UL, 0x4ef4fa5f4c7a3c94UL, 0xa94d87fb364f8f2aUL, 0xcbce70af8ba81417UL, 
			0x8c4b9c36173b2a06UL, 0x6c441f1eac2bf7e9UL, 0x0f0311bcd1623f8dUL, 0x25d54947546d93daUL, 
			0x4167eefe29fe2905UL, 0xaeefb0b3edd045b6UL, 0x862c95064bb3e104UL, 0x4a002fea2ea9bc6dUL, 
			0x3d21abe91d773447UL, 0x37d0ff2431516fa9UL, 0xf51b1c1ec381f951UL, 0x341993d7494ac661UL, 
			0x883a7cbca1ce2d2cUL, 0x8b4444d75a4c9e96UL, 0xca9cb335fb8a17bdUL, 0xf4c6ab999d39c074UL, 
			0xae05a9c72e2c3347UL, 0xfb65c18cea3baf30UL, 0xac6de7b05aace3bbUL, 0xc9b5c994582c5a3eUL, 
			0xf12b6a98c7d28015UL, 0xb25dae528eb8ecf5UL, 0x49513d5578bfcbe5UL, 0x37349a53614b3c09UL, 
			0x21ad203a5d43aca4UL, 0xba5f15db61750192UL, 0x8dd8d928a73dcfcfUL, 0x4e35686f6a4db512UL, 
			0x57aa4361619ff683UL, 0x5cb85cc06408debbUL, 0xe8f319f535d453efUL, 0xf2ff9b166c5b6d18UL, 
			0x2e3c4d1a6882c34fUL, 0xbb14db419426f9faUL, 0x564f61e00d731cd7UL, 0x66605c8f325f26f4UL, 
			0xf23675c9269544b1UL, 0xea15a024220efa27UL, 0x180a0f5f285548f8UL, 0x230427ce3f94ad86UL, 
			0x77a8a42d0bdf69bdUL, 0xacbf312f50d2effcUL, 0xd730abfb49063222UL, 0xa8f56971144f3412UL, 
			0x3e4fc23b0e75ae77UL, 0x2b9c7a05ea250dd8UL, 0x082ae1790870932eUL, 0xdd17466627d245c7UL, 
			0xda7870f231486b8bUL, 0x1e9b6dbf22c0a4f0UL, 0x46b454b34d72acabUL, 0x3bf68a584408246aUL, 
			0x3f04c52356762254UL, 0x3470f7b0dd315bd8UL, 0xff563d580d47154bUL, 0x6eb42aa3c2bc5989UL, 
			0xeafcc3d462d871feUL, 0xd868e28e81eed757UL, 0x354acd4991d39643UL, 0x187c98c3e785f4adUL, 
			0xdc4245d4d4bcfd69UL, 0x99b40296950100daUL, 0xddaaef0c87ece3eeUL, 0x3699502d8686bd06UL, 
			0x5353836b6aa8f41fUL, 0x0f3ad5ebb453eba9UL, 0x19f5e781a010a9e9UL, 0x3d53d58c3db9a35eUL, 
			0x96bf336d2b63f95cUL, 0x961f96c6274865dfUL, 0xe562a1a33edcf23cUL, 0xce5ceb7ccb7406b3UL, 
			0x1f8dcef2be353febUL, 0x458f07b4fe4e6bccUL, 0x5fc1cd3c642c637cUL, 0xf628dfd076f810b3UL, 
			0x41c9993b63eb06faUL, 0x04b511e7b84fc954UL, 0x361f0a22ece2a5cdUL, 0x662534c5a55c955eUL, 
			0xb14e0e67b1392b83UL, 0xd7c1061c36e1ab64UL, 0xcd7383d2732a41d5UL, 0xf0854b6f13a6132eUL, 
			0xe0aa9288158e5f70UL, 0x905314504accda9aUL, 0xee81ff226e0226dbUL, 0xcf4374ab1329c94cUL, 
			0x1a5aa97e3ab32ea9UL, 0x23c96ea439f70ba6UL, 0x1d2b134e4e84e33dUL, 0x9508db0b382f4c63UL, 
			0x5a9700620617a2d1UL, 0x940a64d6dd37bc00UL, 0x9e08e93a7800d334UL, 0xc1541570997e5c3bUL, 
			0x77eae684931b73bcUL, 0xda5e61689c4a55fdUL, 0x9a11b4cf8f9905e5UL, 0xad673fbe92dd6fc2UL, 
			0x91d094d19ddaa53fUL, 0x6e01a85051756efbUL, 0x5a5ffdbdde100932UL, 0xd278e73775bea67aUL, 
			0xc0b981c23a232e4aUL, 0xbbdf4599ea667b00UL, 0x9f22a6d8722abaf8UL, 0x52207b93b1f1cfd7UL, 
			0xb66812b2148a295fUL, 0xa435489186c628d7UL, 0x62ac6234b3aabd98UL, 0x1dbcc6117e41c91aUL, 
			0xc17a726da23b9492UL, 0x23fb1f8ba80d137aUL, 0xade702527091b847UL, 0x939b597259f8f499UL, 
			0x175f395fb92e71b4UL, 0x870713bde37c46adUL, 0xb39d2f8221cd8432UL, 0xca8bd09b100fab9eUL, 
			0x537aa5d3616d7733UL, 0x32501102a182bce4UL, 0xe3c5b61f85872802UL, 0x2a7558405c8d9c49UL, 
			0x2ec48d25adac200bUL, 0x97fb26846744865cUL, 0x449f79a7558c6b88UL, 0x93edf1fc1c21c8e4UL, 
			0x95d011b437611a82UL, 0x46217e5677c93cb9UL, 0xcede31a62c7bb3daUL, 0xe26b06123f6e04f4UL, 
			0xf2e408acab58a5a4UL, 0x3a5fcbc6a588e4eeUL, 0xe62357f2f416d95fUL, 0x8ae8ce629d5d1d88UL, 
			0xb36fa4e6e27be3deUL, 0x667eeb5f9a192b4eUL, 0xcce2eecfa603762fUL, 0x837c370d36787cbaUL, 
			0xc377c5f149f65dfcUL, 0x7a641fae6a1b6694UL, 0xd4ad30b0d9cdca77UL, 0x46f3348e607ef31bUL, 
			0x8b05a2b5675e068bUL, 0xa7310042438f1c65UL, 0x3dfdb91dace92daeUL, 0x608ef33487c319c9UL, 
			0x584ad2edeb8e3ce9UL, 0xcdede94939533882UL, 0xf69717aab8e75d6fUL, 0x54697472d166f0c1UL, 
			0x28011d8be6a35c87UL, 0x2d9ae0c0a34f2adaUL, 0x5b8c336c2f10d7d4UL, 0x29ef6e77ceb08814UL, 
			0x32057e50436584d9UL, 0x903e49300928ae9aUL, 0x5ac6afcf9802af83UL, 0x73f6c168c9adbc94UL, 
			0x4a0f0ca947c91e76UL, 0xbb7215999a3db530UL, 0xe919cadfd386b6ebUL, 0xe8393323754eda0aUL, 
			0x6453e65c6d4975f2UL, 0x2ede33dbd5f247ffUL, 0x18071d771be31237UL, 0x38f8169aaa23150cUL, 
			0x194dedc231f109deUL, 0x6c1204b648b532baUL, 0x05ba0aff0e5cae92UL, 0x22bf350ca78d36d1UL, 
			0xeea3d9d7f5867172UL, 0x1e156b236b4b7fcbUL, 0x30ac580f2a3eebcbUL, 0xc8317417b92f45cbUL, 
			0x5f422877f0305f9bUL, 0x4677b065d555a63eUL, 0x0fbcf9ca5d4bee98UL, 0x72ac6c52ddfe9238UL, 
			0x5874abb0125ad111UL, 0x6d025ff3726e1addUL, 0xb864d250359111eaUL, 0x5ecf4cb8df9c2b57UL, 
			0xbfcd179fa16bae99UL, 0xd3c03b5ee2e74370UL, 0x321fca80d2503edfUL, 0xd1a4a7899fc6d7b7UL, 
			0x0de84372be19e4ddUL, 0xec4d28ca09680699UL, 0x9f531f09355555d4UL, 0x76d558fbd50543cbUL, 
			0x0a67f8b1ac94040dUL, 0x1c1fe4d14516ae1eUL, 0xca7d4eaaa964d834UL, 0xcfa3e9d7da1a7e05UL, 
			0x58e72d14152e0108UL, 0xe0200629aa295772UL, 0x940f375d63466f1fUL, 0x55bb1f1bd021c59aUL, 
			0x78a64b03514f52b9UL, 0x546c3e1777e2e50bUL, 0x7d5f45e31de5f3a0UL, 0xd9f676823caf1fcfUL, 
			0x97a26f322e8309f3UL, 0x5ac2c8d7907ea09aUL, 0x08e1a73e06389074UL, 0x7816e9d66c7825acUL, 
			0xaf8291e7936341ecUL, 0x9e104b1848f168a8UL, 0x647adb7a3a245188UL, 0xd65549e4c60f81eaUL, 
			0x0fedaec0c8770d36UL, 0x329739cd7ec2c9dbUL, 0x3ace4a2a02463dfbUL, 0x9f6e5e3d3749edbdUL, 
			0xd41a934a32e5578cUL, 0x0270ec4dffcf622fUL, 0x83b6dd43abb0f1c4UL, 0xe9eab70031b9113bUL, 
			0x7242ed5a0e032068UL, 0xfb37fa036ba3dff5UL, 0x9d97bb38cae524b9UL, 0x7af9c59f478dc594UL, 
			0x76a5ca935a7129ddUL, 0x359646aeee8f8e56UL, 0xd7d5347f03f3bcb0UL, 0x0e0662c5e1469bc8UL, 
			0xb81424ad70a39ef8UL, 0xf26df16dbcd8df12UL, 0x38d8fb076f0c6f38UL, 0x78958264e6216fd5UL, 
			0x3762b6bbdfb3cbd6UL, 0x820d55bc781c04c6UL, 0x2843be1d2ab013aaUL, 0x81513a7cb9a8f401UL, 
			0x4302c86453884d61UL, 0x0159e7b8a74303ddUL, 0x86503aaf1875ea7cUL, 0xa62f31bb9a79e2b9UL, 
			0x82b5a39715897bb3UL, 0xd26f262c525f5b7dUL, 0x566953584548d975UL, 0x7f1ce18b702c1aa8UL, 
			0x026574b32cceb284UL, 0x8507f0e7fbf9362dUL, 0x73a5d1d4afe99a45UL, 0x1db23999e6efe107UL, 
			0xf971d9f5715a379eUL, 0xcf9cce7a23bdb63aUL, 0x5061634c5bb7f741UL, 0x6b7fe641d4256a05UL, 
			0x7b4e831924032e09UL, 0x7a01fbf6db90b415UL, 0xcaa6769d7adc3fe7UL, 0xd873c2196ce36c52UL, 
			0xa99f365493a9c2c7UL, 0xd8d1b0204c771359UL, 0x057260cfa7042405UL, 0x842e61adfb826c7bUL, 
			0xeaf353c00eeb3c82UL, 0x6ad16f7eaabdb9d1UL, 0xcb98df445ce4742dUL, 0x06ffe80d0659d519UL, 
			0x059d492c45b1cc06UL, 0x12237312347b47e4UL, 0xab78a2a22dff2689UL, 0xb37f1271592dfcb0UL, 
			0x293565826d7261ceUL, 0xf4968be36d1c9667UL, 0xdbb8aa6ca3293246UL, 0x71a91bce11d14c6cUL, 
			0x38d90f2f12d77523UL, 0x87cab95b953fe0fbUL, 0x8b771907e5d4022aUL, 0x529489599739ff9bUL, 
			0x86f06fdea6e33b62UL, 0x46ec5b25e4ce3e8eUL, 0x4822dc2d98331f2bUL, 0x1bebf8d0c9a90a0bUL, 
			0xefb0777af3bdcdcaUL, 0x27edca1de24735eeUL, 0x828278b5c13056feUL, 0x509cd75c24a48985UL, 
			0x3138fb3fb58532cbUL, 0xd24d478090628c97UL, 0xb658c87f9a485974UL, 0x1c481efc9f3e2c1dUL, 
			0x8ba7a0e9bce07d25UL, 0x67de52f782118c75UL, 0x3b2ab73edaedc16cUL, 0x3244e60fe12f606bUL, 
			0x3190df82e315f176UL, 0x48f45e00ff096411UL, 0x9dff61532f65472eUL, 0x928ddecd86a851e8UL, 
			0x26f03160895bfa99UL, 0x4460d4ca4b521769UL, 0x67af30e2ecc26efcUL, 0x3dac8736803b777cUL, 
			0xfae64b75880dc944UL, 0x9f19235d58acc36cUL, 0xacd7dc3dab47900dUL, 0x6eb3ef3048617f43UL, 
			0x955ea3eebdc2b3ddUL, 0x266988d844449b5fUL, 0xd2ae9ebdf1dac9d3UL, 0x849938f7630cc13eUL, 
			0xf8af51877f3d43baUL, 0xb5fc667b70513222UL, 0xd5bf7e179a6687afUL, 0xd2106671574c6f49UL, 
			0x3f02ddc26f6efb9fUL, 0xd98a009bb33c3156UL, 0x7d24013c86bdf3c5UL, 0xc7480e6e2791b8efUL, 
			0x99aa84f23d434e89UL, 0x9758d3635d476638UL, 0xfa9320a6cca88ff5UL, 0xc84cdfc97639ab16UL, 
			0x158e852959607608UL, 0x895d4ce01842c860UL, 0x6bdb68c91d9e3afeUL, 0x73aae335e2804d39UL, 
			0x8b0219297708b67dUL, 0x23f8bf1e0a81c845UL, 0x21173e170783bd71UL, 0xaac53f1a39802168UL, 
			0x6374e560dc6f1d1fUL, 0x7171cd44cf9561dcUL, 0xea02c50abf59eaf0UL, 0xac16db37202d835fUL, 
			0x4aa52fa90a301247UL, 0x848161bde7695541UL, 0xbe2be35ca00ca8f3UL, 0xb7b5f379c18c2441UL, 
			0xb72524ed5ee1749bUL, 0xd89f8883ab2c635bUL, 0xee58b4fcbcf9d9a0UL, 0xd282ea6d7523f9ccUL, 
			0xb6623157d31726d6UL, 0xae9937e8d8a2a794UL, 0x113a19d4658e520bUL, 0xd87bcc4f5be592cfUL, 
			0x54e928845c645109UL, 0xdb20dd3b19f4c94dUL, 0xe3df05cf182d2a03UL, 0x99776aafd81bcd74UL, 
			0xb64d6ca3b2462f90UL, 0xa6ebb63563a4859bUL, 0x5534551350fd5f99UL, 0xdbb35d557f6c0485UL, 
			0x4d75b7a03027a55cUL, 0x936b2e76f42a4213UL, 0x8da4464d7aa39eacUL, 0xd75eaaeacdf01793UL, 
			0x92940c1c717c71f8UL, 0xb8fe3cf900edf7b8UL, 0xf66f643f054137d9UL, 0xdba51b1051592cfbUL, 
			0xabbb1d491af184a6UL, 0xd955e21dc03efaa2UL, 0x3f105e4f5edad537UL, 0x32ffff7cd89ca166UL, 
			0xd039b0f675890432UL, 0x460f4d6a1319df46UL, 0x090483a1e38767e0UL, 0x66c581ec7d8a7296UL, 
			0x15c6af3b0c89870aUL, 0x5b1e901b7063fd9aUL, 0x153842da7748aba6UL, 0xc4a1f546de6e0895UL, 
			0x4ac27a8f117cef38UL, 0xe552f53a04e74f27UL, 0xd30b644ec2af2aa3UL, 0x3ddd965793fb4eeeUL, 
			0x6b55524f06d3d11bUL, 0x2d15479a4ba8836eUL, 0x1fa40bd6b43cbd38UL, 0x30b19eb1e093b17aUL, 
			0x16f0b9e724ae9b85UL, 0x56aa25cc5bf08beeUL, 0x42c95eb793dbab89UL, 0x4f45dfc779a98c59UL, 
			0xd18f47390632acacUL, 0x50f6686019437caaUL, 0xcb003f35d77b34fdUL, 0xe58b60610cec4926UL, 
			0xabc7083e5061d389UL, 0xfeac26c7d5ac287dUL, 0xf668c0f8d083fb73UL, 0xe45f1986768d2625UL
		};
	}
}
